#! /usr/bin/perl
#
# Copyright (c)2011-2014 Roman Bartke.  All rights reserved.
# Author: Dipl.-Inform. Roman Bartke
#
# Created: 2011-11-20
# Updated: 2014-01-03
#
use strict;

use Getopt::Long;
use File::Basename;

#my $petcat      = "D:/storage/project/vice-svn-petcat/vice-emu/vice/data/petcat.exe";
my $petcat      = "D:/storage/project/c64/vice-2.4/vice-emu/vice/data/petcat.exe";
my $petme64root = "D:/storage/project/c64/petme64/";
my $expand64    = $petme64root."expand64.pl";
my $petme64     = $petme64root."PetMe64.pl";
my $fop         = "C:/Programme/fop-0.95/fop.bat";
my $fopconf     = $petme64root."conf/fop.xconf";

my $mainTemplate = "";
my $subTemplate = "";

my $result      = 0;
my $verbose     = 0;

my $large       = 0;
my $upcase      = 1;
my $token       = 1;
my $pageno      = 0;
my $filename    = "";
my $caption     = "";
my $output      = "";
my $author      = "RoB";
my $subject     = "";

my $cmd         = "";
my $option1     = "";

$result = GetOptions ("large:n"    => \$large,   
                      "upcase:n"   => \$upcase,   
                      "token:n"    => \$token,
	              "pageno:n"   => \$pageno,
	              "filename=s" => \$filename,
	              "caption=s"  => \$caption,
	              "output=s"   => \$output,
	              "author=s"   => \$author,
	              "subject=s"  => \$subject,
	              "verbose"    => \$verbose
                     ); 
if ((!$result) || ($filename eq ""))
{
        print STDERR "usage: $0 ...\n";
        print STDERR "           [--large]:    0:large font; 1:small font; 2:two columns\n";
	print STDERR "           [--upcase]:   1:upper case; 0:lower case \n";
        print STDERR "           [--token]:    1:detokenize; 0:raw input  \n";
#       print STDERR "           [--pageno]:   1:show page number; 0:no pn\n";
        print STDERR "           [--filename]: input file name            \n";
        print STDERR "           [--caption]:  shown left of page number  \n";
        print STDERR "           [--output]:   output pdf filename        \n";
        print STDERR "           [--author]:   pdf author info            \n";
        print STDERR "           [--subject]:  pdf subject info           \n";
        print STDERR "           [--verbose]:  verbose output             \n";
	exit;
}

if ($caption eq "")
{
	my @suffixlist = ( qr/\.[^\.]+/ );
	$caption = fileparse($filename,@suffixlist);
}
if ($output eq "")
{
	$output = $caption;
}

if ($verbose)
{
	print STDERR "Options: ";
	
	print STDERR "\n\tlarge font,"              if  $large eq 0;
	print STDERR "\n\tsmall font,"              if  $large eq 1;
	print STDERR "\n\tsmall font 2 columns,"    if  $large eq 2;
	
	print STDERR "\n\tupper case,"              if  $upcase eq 1;
	print STDERR "\n\tlower case,"              if  $upcase eq 0;
	
	print STDERR "\n\tdetokenize input,"        if  $token eq 1;
	print STDERR "\n\tuse raw input,"           if  $token eq 0;
	
#	print STDERR "\n\tshow page numbers,"       if  $pageno eq 1;
#	print STDERR "\n\tdon't show page numbers," if  $pageno eq 0;
	
	print STDERR "\n\tverbose output,"          if  $verbose eq 1;
	print STDERR "\n\tnone verbose output,"     if  $verbose eq 0;
	
	print STDERR "\n\tfilename          = ",        $filename;
	print STDERR "\n\tcaption to show   = ",        $caption;
	print STDERR "\n\tpdf author info   = ",        $author;
	print STDERR "\n\tpdf subject info  = ",        $subject;
	print STDERR "\n";
}

$option1 = "-2" if $token eq 1;
$option1 = "-I" if $token eq 0;
$cmd = "$petcat -nh $option1 -b \"$filename\" >prg.pet";
system($cmd);

$option1 = ""   if $upcase eq 1;
$option1 = "-l" if $upcase eq 0;
$cmd = "perl $expand64 $option1 prg.pet prg.uc";
system($cmd);

if ($large eq 0)
{
	$mainTemplate = $petme64root."tpl/petme-pn-tpl.fo";
	$subTemplate  = "";
}
if ($large eq 1)
{
	$mainTemplate = $petme64root."tpl/petme-small-tpl.fo";
	$subTemplate  = $petme64root."tpl/petmeblk-small-tpl.fo";
}
if ($large eq 2)
{
	$mainTemplate = $petme64root."tpl/petme-pn-2col-tpl.fo";
	$subTemplate  = $petme64root."tpl/petmeblk-2col-tpl.fo";
}

$option1 = "-b \"$subTemplate\"" if $subTemplate ne "";
$option1 = ""                    if $subTemplate eq "";

$cmd = "perl $petme64 -i prg.uc -o prg.fo -t \"$filename\" -n \"$caption\" -c \"$author\" -d \"$subject\" -m \"$mainTemplate\" $option1 >>prg.log";
system($cmd);

$cmd = "$fop -c $fopconf prg.fo \"$output.pdf\" 2>>prg.log";
system($cmd);

$cmd = "start \"\" \"$output.pdf\"";
system($cmd);


