#!/usr/bin/perl
#
# Copyright (c)2011 Roman Bartke.  All rights reserved.
# Author: Dipl.-Inform. Roman Bartke
#
# Created: 2011-11-26
# Updated: 2011-11-26
#

use strict;
use Getopt::Long;

my $petcat      = "D:/storage/project/vice-svn-petcat/vice-emu/vice/data/petcat.exe";
my $tmpdir      = $ENV{"TEMP"}."\\";

my $result      = 0;
my $verbose     = 0;

my $upcase      = 1;
my $token       = 1;
my $cpl         = 40;
my $filename    = "";
my $output      = "";

my $cmd         = "";
my $option1     = "";

$result = GetOptions ("upcase:n"   => \$upcase,   
                      "token:n"    => \$token,
	              "cpl:n"      => \$cpl,
	              "filename=s" => \$filename,
	              "output=s"   => \$output,
	              "verbose"    => \$verbose
                     ); 
if ((!$result) || ($filename eq ""))
{
        print STDERR "usage: $0 ...\n";
	print STDERR "    [--upcase]:   1:upper case (default); 0:lower case\n";
        print STDERR "    [--token]:    1:detokenize (default); 0:raw input\n";
        print STDERR "    [--filename]: input file name\n";
        print STDERR "    [--output]:   output tex filename\n";
	print STDERR "    [--cpl]:      characters per line (default:40)\n";
	exit;
}

$tmpdir = ".\\" if $tmpdir eq "";

if ($verbose)
{
	print STDERR "Options: ";
	
	print STDERR "\n\tupper case,"              if  $upcase eq 1;
	print STDERR "\n\tlower case,"              if  $upcase eq 0;
	
	print STDERR "\n\tdetokenize input,"        if  $token eq 1;
	print STDERR "\n\tuse raw input,"           if  $token eq 0;
	
	print STDERR "\n\tverbose output,"          if  $verbose eq 1;
	print STDERR "\n\tnone verbose output,"     if  $verbose eq 0;
	
	print STDERR "\n\tfilename          = ",        $filename;
	print STDERR "\n\toutput            = ",        $output;
	print STDERR "\n\ttmpdir            = ",        $tmpdir;
	print STDERR "\n";
}

$option1 = "-2" if $token eq 1;
$option1 = "-I" if $token eq 0;
$cmd = "$petcat -nh $option1 -b \"$filename\" >".$tmpdir."prg.pet";
system($cmd);

my $outdata = "";
my $inputData = read_input($tmpdir."prg.pet");
my $j = 0;
for(my $i=0; $i<length($inputData);$i++)
{
	my $c = substr($inputData,$i,1);
	if (($c ne "\n") and ($c ne "\r"))
	{
		$c = map_char($c);
		$outdata .= "\\char".ord($c);
		if ((($j+1) % $cpl) eq 0) 
		{ 
			$outdata .= "\\\\\n";
			$j = 0;
		}
		else
		{
			$j++;
		}		
	}
	else
	{                    # Supress empty lines:
#		if ($j ne 0) # No additional newline if last line was
		{            # exactly $cpl (e.g. 40) characters long.
			     # This is not like C64 LIST works!
			$outdata .= "\\\\\n";
		}
		$j = 0;
	}
}
open(OUT, "> $output");
binmode(OUT);
print OUT $outdata;
close(OUT);

sub map_char
{
	my $c = shift;
	if (($c ne chr(13)) && ($c ne chr(10)))
	{
		my $a = ord($c);
		my $d;
		if    ($a >= 0x00 && $a <= 0x1f) { $d = 128; }
		elsif ($a >= 0x20 && $a <= 0x3f) { $d =   0; }
		elsif ($a >= 0x40 && $a <= 0x5f) { $d = -64; }
		elsif ($a >= 0x60 && $a <= 0x7f) { $d = -32; }
		elsif ($a >= 0x80 && $a <= 0x9f) { $d =  64; }
		elsif ($a >= 0xa0 && $a <= 0xbf) { $d = -64; }
		elsif ($a >= 0xc0 && $a <= 0xdf) { $d =-128; }
		elsif ($a >= 0xe0 && $a <= 0xfe) { $d =-128; }
		elsif ($a == 0xff)               { $d =-161; }
		else { die "unknown char code"; }
		$c = chr(ord($c) + $d);
	}
	return $c;
}

sub read_input {
	my $inFileName = shift;
        die if !open(FILE, "$inFileName");
        binmode(FILE);

        my $inFile = "";
        while (<FILE>) 
	{
                        my $line = $_;
                        $line =~ s/^\s+([0-9]+)/\1/;
                        $inFile .= $line;
        }
        close (FILE);
        return $inFile;
}

