
; vim:ts=8:sw=8:sta:et:sts=8:ai:syn=a65

; "C64 Mr. Ed by Chris Miller Jul, 1986"
; "C-128 port by Roman Bartke Feb, 2015"

.include "config.i"

.ifdef with_MrEd

.list on
.listbytes unlimited
.debuginfo -

BANK0    = $3F                         ; MMU Configuration Mask for Bank 0
BANK1    = $7F                         ; MMU Configuration Mask for Bank 1
BANK15   = $00                         ; MMU Configuration Mask for Bank 15
BANK1IO  = $7E

         .export InitEd,StartEd
         .export OutputXA
         .export OutputXATest
         .import jimm,simm,fimm,JSFRTS

;jimm     = $0d00
;simm     = $0d03
;fimm     = $0d06
;JSFRTS   = $0d74                       ; position of RTS in JSF

JSRFAR   = $02CD
FARVEC   = JSRFAR+$11

MMUCR    = $FF00
MMURCR   = $D506

SYSVEC   = $0A00                       ; used by monitor return


.macro JSK KernAddr
;        SEI

;        pha
;        lda #$05
;        sta MMURCR
;        LDA #BANK1
;        STA FARVEC
;        pla

	 JSR jimm
 	 .byte BANK15,>KernAddr,<KernAddr

;	 .byte $4E,>KernAddr,<KernAddr
;        pha
;        lda #$04
;        sta MMURCR
;        pla

;        CLI
.endmacro

;.macro JSF Bank,KernAddr
;        SEI
;	 JSR jimm
;	 .byte Bank,>KernAddr,<KernAddr
;        CLI
;.endmacro

.macro STF Bank,StoreAddr
;        SEI
	 JSR simm
	 .byte Bank,<StoreAddr,>StoreAddr
;        CLI
.endmacro

.macro LDF Bank,FetchAddr
;        SEI
	 JSR fimm
	 .byte Bank,<FetchAddr,>FetchAddr
;        CLI
.endmacro

         .segment "MRED"

         .global MRED_START
MRED_START:
         .org MrEd_DestAddr ; $1C01

; *** constants ***
columns  =    40                       ; screen size
linesize =    250                      ; max allowed
screenbeg=    1024+40                  ; top of text scr
screenend=    2024                     ; end of text scr
rows     =    24                       ; screenend-screenbeg/columns

; *** important memory ***
vic      =    $D011
bkg      =    53281
bor      =    53280
keychk   =    $033C                    ; Editor: Taste speichern indirekt
rptkey   =    $0A22

; *** rom routines ***
;              C128
;cnvrtdec=    $8E32
moncal   =    $B000
print    =    $FFD2
getin    =    $FFE4

; *** variables ***
ZPAreaBeg= $0a

row	 = $0a ; .byte $00             ; screen row 0-24
col	 = $0b ; .byte $00             ; screen col 0-39
shift	 = $0c ; .byte $00             ; off screen left
line	 = $0d ; .word $0000           ; text line counter
cnt	 = $0f ; .byte $00             ; display line counter
num	 = $10 ; .byte $00             ; general purpose
disflg   = $11 ; .byte $00             ; negative=no display

vars     = disflg - ZPAreaBeg + 1

ptr	 = $12 ; .word $0000           ; utility pointer
top	 = $14 ; .word $0000           ; top line of text window
sob	 = $16 ; .word $0000           ; start of buffer
eob      = $18 ; .word $0000           ; end of memory buffer
end	 = $1a ; .word $0000           ; end of text
txt	 = $1c ; .word $0000           ; current text position
scr	 = $1e ; .word $0000           ; current screen position

ZPAreaEnd= $1F

ZPAreaSize = ZPAreaEnd - ZPAreaBeg + 1

         .export sobpos,endpos,eobpos
endpos   = MrEd_EndPos
;sobpos   = $0B03
;eobpos   = $0CFE; $0B7D ; $0CFF
sobpos   = MrEd_BufStart
eobpos   = MrEd_BufEnd

; Mr. Ed uses pointers to define its own work space.
;
; The last byte of text entered will be pointed to by (end)
; during and following any work session.
;
; One null byte guard required!
;   ^
;   |               __________________
;  [0###############__________________]
;    ^      ^      ^                 ^
;    |      |      |                 |
;  (SOB)  (TXT)  (END)             (EOB)

; ----------------------------------------------------------------------------

InitEd:

         LDA FARVEC
         PHA

         LDA keychk
         PHA

         JSR SaveZPArea

         LDA #$B7                      ; Give F1-F8 same function as 64 mode:
         STA keychk                    ; to skip editor's keyput routine
                                       ; set ($033C) from $C6AD to $C6B7
         LDA #BANK1
         STA FARVEC

         lda #<sobpos
	 sta sob
	 lda #>sobpos
         sta sob+1

         LDA #0
         STA sobpos-1
         STA eobpos+1

         LDA #<eobpos
         STA eob
         LDA #>eobpos
         STA eob+1

         lda endpos
         sta end

         lda endpos+1
         sta end+1

         lda #128                      ; keys repeat
         sta rptkey

; entry for ed command from basic
; or reentry from ml
StartEd:
         LDA #BANK1
         STA FARVEC

         jsr InitScreen

         lda  #0
         ldx  #(vars-1)                ; init vars
ClrVars: sta  ZPAreaBeg,x
         dex
         bpl  ClrVars

         jsr  initialize

         jsr  window

; main key scan loop
getkey:                                ; always return here
         lda  #>(getkey-1)
         pha
         lda  #<(getkey-1)
         pha

         jsr  reverse
         jsr  statusline               ; line, col, mem

getkey1:
;        cli
         JSK  getin
;        sei
         beq  getkey1


         jsr  reverse                  ; y=0

; check command keys
         ldy #0
nextcmd: cmp  commands,y
         beq  foundkey                 ; also sets carry
         iny
         iny
         iny
         cpy  #commandnum
         bcc  nextcmd
         bcs  put                      ; a typing key

foundkey:                              ; jump to routine
         lda  commands+2,y
         pha
         lda  commands+1,y
         pha
         rts

InitScreen:
         SEI                           ; Would like to know, 
                                       ; why this is necessary...
	 LDF  0,vic
	 and  #%11101111               ; screen off
         STF  0,vic

         LDA  #6                       ; blue screen
         STF  0,bor

         lda  #1                       ; white print
         STF  0,bkg

         jsr  message                  ; y=0

         .byte 5                       ; white text
;        .byte 154                     ; light blue text

         .byte 147                     ; clear screen
;        .byte 14                      ; switch to lower case
;        .byte 11                      ; disable character set change

;        .byte 154                     ; light blue text
         .byte 158                     ; yellow text

         .byte 0                       ; end of string marker

         LDA  #6
         STF  0,bkg

	 LDF  0,vic
	 ora  #%00010000               ; screen on
         STF  0,vic

         CLI
         RTS

; put character in text buffer
put:
         tax                           ; save key
         cmp  #13
         beq  f1

; see if line full
         lda  shift
         cmp  #(linesize-40)
         bne  f1
         lda  col
         cmp  #(columns-1)
         beq  r1

f1:      jsr  testpos                  ; are we at end
         bcc  f2

         jsr  pshend                   ; make room if can
         bcs  r1                       ; out of memory

f2:      cpx  #13
         beq  cret

         ldy  #0
         lda  (txt),y
         cmp  #13                      ; end of a line check
         bne  f3
         jsr  insert
f3:      txa
         sta  (txt),y
         jsr  cnvscr
;        sta  (scr),y
         jsr  putonscreen

; The LEFT and RIGHT routines allow lateral motion across any line;
; and UP and DOWN cause vertical travel through the text buffer,
; always to the start of a line. All display and pointer positioning
; overhead is taken care of.

; cursor right routine
right:                                 ; cursor right routine
         jsr  findeoln
         tya
         beq  r1                       ; already at end
         inc  txt
         bne  f4
         inc  txt+1
f4:      lda  col
         cmp  #(columns-1)
         bne  f5

         inc  shift
         jmp  window

f5:      inc  col
         inc  scr
         bne  r1
         inc  scr+1
r1:      rts

; cursor left routine
b4:      dec  shift                    ; scroll left
         jmp  window

left:
         lda  col
         ora  shift                    ; check position
         beq  r2                       ; can't go left
         jsr  dectxt
         lda  col
         beq  b4
         dec  col
         dec  scr
         lda  scr
         cmp  #$ff
         bne  f6
         dec  scr+1
f6:      sta  scr
r2:      rts

; carriage return handling routine
cret:                                  ; handle carriage return
         jsr  findeoln
         txa                           ; x=13
         sta  (txt),y

; cursor down routine
down:
         jsr  findeoln
         tay
         beq  r2                       ; already at bottom
         jsr  unshift                  ; all the way left
         jsr  findeoln
         jsr  addy
         inc  line
         bne  f7
         inc  line+1
f7:      lda  row
         cmp  #(rows-1)                ; last row check
         bne  f8
         jmp  topdown                  ; scroll down
f8:      inc  row

; -------------------------------------
addrow:  lda  scr                      ; (scr) += #columns
         clc
         adc  #columns
         bcc  addrow1
         inc  scr+1
addrow1: sta  scr
         rts

; -------------------------------------
; cursor up routine
up:
         lda  line                     ; check position
         ora  line+1                   ; check position
         beq  r3                       ; at top already
         dec  line
         lda  line
         cmp  #$ff
         bne  f10
         dec  line+1
f10:     lda  txt
         sec
         sbc  shift
         bcs  f11
         dec  txt+1
f11:     sta  txt
         jsr  unshift                  ; scroll far left
         ldy  #1
         sty  num
         dey
b5:      jsr  dectxt                   ; go back 2 cr
         lda  (txt),y
         beq  f12
         cmp  #13
         bne  b5
         dec  num
         bpl  b5

f12:     inc  txt                      ; then forward 1 char
         bne  f13
         inc  txt+1
f13:     lda  row                      ; top of screen check
         beq  topup                    ; scroll up

         dec  row                      ; else move up row
         lda  scr
         sec
         sbc  #columns
         bcs  f14
         dec  scr+1
f14:     sta  scr
r3:      rts

; move window up
topup:                                 ; move text window up line
         ldy  #$ff
         dec  top+1
b6:      dey
         lda  (top),y
         beq  newtop
         cmp  #13
         bne  b6

newtop:                                ; add y to top pointer
         sec
         tya
         adc  top
         bcc  f15
         inc  top+1
f15:     sta  top
         jmp  window

; move window down
topdown:                               ; move window down line
         ldy  #$ff
b7:      iny
         lda  (top),y
         beq  newtop
         cmp  #13
         bne  b7
         beq  newtop

; initialize for start of new line
unshift:
         lda  scr
         sec
         sbc  col
         bcs  f16
         dec  scr+1
f16:     sta  scr

         lda  txt
         sec
         sbc  col
         bcs  f17
         dec  txt+1
f17:     sta  txt

         lda  #0
         sta  col
         sta  shift

;
; Mr. Ed's WINDOW routine is the real workhorse and is used to move ASCII text
; from the buffer to the screen window. The TOP pointer will be set to point to
; the start of a text line. This line will appear at the top of the screen. TOP
; determines the vertical position of the window. The SHIFT variable determines
; the horizontal position of the window for left/right scrolling.

; DISFLG is used to enable/disable the window for operations where having
; display constantly updated may not be desirable (ie. speed is of the essence).

;
;  (sob)                     (end)                                        (eob)
;    |                         |                                            |
;    V                         V                                            V
;     ______________________________________    ____________________________
;    |o#########################____________>   >______o____________________|
;      \        |  Text           Free Mem            /
;       \_______|______                              /
;               |       \                   Memory Text Buffer
;               |      (sob)                       /        linesize = 250 -v
;               |     +-/- -----------------------/--------------------------+
;             (txt)   |o  //                     o                         // |
;               |      ~~~  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  ~ 
;               |     |~~~  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//~|
;               |     |   //      (scr)                   columns = 40 -v  // |
;               |     |   //    +--/-------------------------------------+ // |
;               |     |   //    |_/__________STATUS LINE_________________| // |
; Scrn Line 1 (top) --|o-------o|o                                       | // |
;                     |   //    |                                        | // |
;                     |   //    |/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\| // |
;                                                                              
;                     |   //    |\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/| // |
;                     |<-shift->|                                        | // |
;              cnt    |   //    |                                        | // |
; Scrn Line 24 rows   |   //    |                                        | // |
;                     |   //    +----------------------------------------+ // |
;                     |   //                                               // |
;                      ~~~  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  ~
;                     |~~~//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//~|
;                     |   //                                               //o|
;                     +--- -------------------------------------------------/-+
;                                                                         (eob)

; move text to screen window
window:
         bit  disflg                   ; is display on
         bmi  window0                  ; no

         lda  #rows                    ; screenend-screenbeg/columns
         sta  cnt

         lda  txt+1                    ; save pointers (txt) and (scr)
         pha                           ; the window routine
         lda  txt                      ; has own usage of these
         pha
         lda  scr+1
         pha
         lda  scr
         pha
                                       ; init pointers
         lda  top                      ; (txt) = (top)
         sta  txt                      
         lda  top+1
         sta  txt+1
         jsr  initscr                  ; (scr) = #screenbeg

; process next line of text

; first check for blank
newline:
         ldy  #$ff                     ; set to last char in line, so testeol
newline1:jsr  testeoln                 ; can test first char of next line
         beq  lineblank                ; handle blank rest of line
         cpy  shift                    ; handle right scroll
         bcc  newline1                 ; repeat while y < shift 

         clc
         jsr  addy1                    ; update txt ptr: (txt) += y

; output line
         ldy  #$ff
nextchr: jsr  testeoln                 ; if current col at end of line
         beq  restblank                ; make rest of it blank

         jsr  cnvscr                   ; else convert char in .A to screen code

         jsr  putonscreen              ; and put it put on screen
;        sta  (scr),y

         cpy  #(columns-1)             ; if y < (columns-1)
         bcc  nextchr                  ; output next char

; now find the invisible end of line
; (off screen to the right)
         jsr  findeol1                 ; don't init .y
         jsr  addy                     ; point next line: (txt) += y

addscr:  jsr  addrow                   ; addrow to scr ptr: (scr) += #columns
         dec  cnt                      ; countdown screen line counter
         bne  newline                  ; repeat, until no lines left

fini:    pla                           ; restore ptrs (scr) and (txt)
         sta  scr                      
         pla
         sta  scr+1
         pla
         sta  txt
         pla
         sta  txt+1

window0: rts

; -------------------------------------
lineblank:
         jsr  addy
         ldy  #0
         beq  f18
restblank:
         jsr  addy
f18:     lda  #' '
b10:     ;sta  (scr),y
         jsr  putonscreen
         iny
         cpy  #columns
         bcc  b10
         bcs  addscr                   ; start next line

; -------------------------------------
addy:    sec                           ; add carry and
addy1:   tya                           ; add y to text ptr
         adc  txt                      ; (txt) += y
         bcc  addy2
         inc  txt+1
addy2:   sta  txt
         rts

; -------------------------------------
;dectxt:                               ; back up text ptr
;        dec  txt
;        lda  txt
;        cmp  #$ff
;        bne  dectxt1
;        dec  txt+1
;dectxt1:rts

dectxt:  lda txt
         bne dectxt1
         dec txt+1
dectxt1: dec txt  

; -------------------------------------
; convert ascii to screen code
cnvscr:
         eor  #128                     ; if bit 7 (most sgnificant bit)
         bpl  cnvscr1                  ; is zero, do nothing ->
         eor  #128
         cmp  #64                      ; if char < 64
         bcc  cnvscr1                  ; do nothing -> 
         eor  #64                      ; else clear bit 6 of char
cnvscr1: rts

; -------------------------------------
reverse:                               ; reverse cursor char
         pha
         ldy  #0
         jsr  getfromscreen
;        lda  (scr),y
         eor  #$80                     ; flipping the most significant bit 7
;        sta  (scr),y                  ; of current character
         jsr  putonscreen
         pla
         rts

; -------------------------------------
testpos:                               ; test position in text
         lda  txt+1                    ; check if (txt) <= (end)
         cmp  end+1                    ; compare hi bytes first
         bcc  testpos1                 ; t <  e
         bne  testpos1                 ; t <> e
         lda  txt                      ; if equal then test lo bytes
         cmp  end
testpos1:rts

; -------------------------------------
; INSERT and DELETE are used to open up and remove space.

insert:                                ; insert one space
         lda  end+1
         pha
         lda  end
         pha
         ldy  #0
b11:     lda  (end),y
         iny
         sta  (end),y
         dey
         dec  end
         lda  end
         cmp  #$ff
         bne  f23
         dec  end+1
f23:     jsr  testpos
         bcc  b11
         beq  b11
         pla
         sta  end
         pla
         sta  end+1

pshend:                                ; bump end ptr up
         lda  end+1
         cmp  eob+1
         bcc  pshend1
         lda  end
         cmp  eob
         bcs  pshend2
pshend1: inc  end
         bne  pshend2
         inc  end+1
pshend2: rts

deletechr:
         lda  #1
         sta  num
         jsr  left                     ; input

delete:                                ; number of chars in num
         lda  txt+1
         pha
         lda  txt
         pha

delete1: ldy  num
         lda  (txt),y
         ldy  #0
         sta  (txt),y
         inc  txt
         bne  delete2
         inc  txt+1
delete2: jsr  testpos
         bcc  delete1
         beq  delete1

         pla
         sta  txt
         pla
         sta  txt+1

         jsr  testpos
         bcs  delete3
         lda  end
         sec
         sbc  num
         sta  end
         bcs  delete3
         dec  end+1
delete3: jmp  window

insertln:                              ; insert chr$(13)
         jsr  insert
         lda  #13
         bne  storechr

insrtspc:                              ; insert blank
         jsr  insert
         lda  #' '
storechr:sta  (txt),y
         jmp  window

deleteln:                              ; delete line
         ror  disflg                   ; display off
         jsr  findeoln
         tya
         bne  delln1
         iny
delln1:  sty  num                      ; delete num char
         jsr  delete
         jmp  setwindow

pagedown:                              ; long scroll down
         ror  disflg                   ; no display
         ldx  #(rows-1)                ; 23 lines
b13:     jsr  down
         dex
         bne  b13
         beq  setwindow

pageup:                                ; long scroll up
         ror  disflg                   ; no display
         ldx  #(rows-1)                ; 24 lines
b14:     jsr  up
         dex
         bne  b14
         beq  setwindow

pageleft:
         ror  disflg
         ldx  #(columns-1)
b15:     jsr  left
         dex
         bne  b15
         beq  setwindow

; scroll sideways to end of line
pageright:
         ror  disflg                   ; no display
         ldx  #(columns-1)
b16:     jsr  right
         dex
         bne  b16
setwindow:
         lsr  disflg
         jmp  window                   ; display

monitor:
         pha
;        lda #7
;        JSK print

         LDA #<JSFRTS
         STF BANK15,SYSVEC

         LDA #>JSFRTS
         STF BANK15,(SYSVEC+1)

         JSK moncal

         pla

;        jmp myready

         jsr InitScreen
         jmp window

;        jmp StartEd

; -------------------------------------
; FINDEOLN returns in .Y the number of characters remaining
; in the current line of text (as pointed to by TXT).

; set y=distance to text eol
findeoln:                              ; find eol beginning at col 0
         ldy  #$ff
findeol1:jsr  testeoln                 ; find eol beginning at col .Y
         bne  findeol1
         rts

; -------------------------------------
testeoln:
         iny                           ; move one col right
         cpy  #$ff                     ; if last col
         beq  testeol1                 ; exit
         lda  (txt),y                  ; get current char in buffer 
         beq  testeol1                 ; if it is 0 then exit
         cmp  #13                      ; compare it with cr to set zero flag
testeol1:rts

; -------------------------------------
getfromscreen:
         lda  (scr),y
         rts

         LDA #BANK1IO
         STA MMUCR

         lda  #$05
         sta  MMURCR

         lda  (scr),y

         pha
         lda  #$04
         sta  MMURCR
         pla

         rts

;getfromscreen:
;         clc
;         tya
;         adc  scr
;         sta  fromscr+1
;         lda  scr+1
;         adc  #0
;         sta  fromscr+2
;
;         jsr  fimm
;fromscr: .byte BANK0,$28,$04
;         rts

; -------------------------------------
putonscreen:
         sta  (scr),y
         rts

         pha
         lda  #$05
         sta  MMURCR
         pla
         pha
         sta  (scr),y
         lda  #$04
         sta  MMURCR
         pla
         rts

;putonscreen:
;         pha
;         clc
;         tya
;         adc  scr
;         sta  toscr+1
;         lda  scr+1
;         adc  #0
;         sta  toscr+2
;         pla
;         jsr  simm
;toscr:   .byte BANK0,$28,$04
;         rts

; -------------------------------------
initialize:
         lda  sob
         ldx  sob+1                    ; ax = start of buffer

;        clc
;        adc  #1                       ; ax += 2
;        bcc  init1
;        inx

init1:   sta  txt
         stx  txt+1                    ; txt = ax
         sta  top
         stx  top+1                    ; top = ax

         lda  end
         sta  ptr
         lda  end+1
         sta  ptr+1                    ; ptr = end

         ldy  #$00                     ; fill zeros
         tya
init2:   sta  (ptr),y                  ; *ptr = 0
         inc  ptr
         bne  init3
         inc  ptr+1                    ; ptr++
init3:   ldx  ptr+1
         cpx  eob+1
         bcc  init2
         ldx  ptr
         cpx  eob
         bcc  init2                    ; do while ptr < eob

; -------------------------------------
initscr:
         lda  #<screenbeg
         sta  scr
         lda  #>screenbeg
         sta  scr+1
         rts

; Statusline Format:
; C000/L00000 M00000 T00000/00000/00000
;   ^     ^      ^      ^     ^     ^ 
;  Col  Line MemFree Bottom Current Top Pointer
statusline:

         jsr  message
         .byte 19      ; (home)
;        .byte "COLUMN:"
         .byte 'C';+$80
;        .byte 'O'+$80
;        .byte 'L'+$80
;        .byte 'U'+$80
;        .byte 'M'+$80
;        .byte 'N'+$80
;        .byte ':'
         .byte 0
         lda  shift
         sec
         adc  col
         tax
         tya          ; =0
         JSR OutputXA
         jsr  message
;        .byte " LINE:"
;        .byte ' '
;        .byte 'L'+$80
;        .byte 'I'+$80
;        .byte 'N'+$80
;        .byte 'E'+$80
         .byte '/','L';+$80
;        .byte ':'
         .byte 0
         ldy  line+1
         ldx  line
         inx
         bne  f32
         iny
f32:     tya
         JSR OutputXA
         jsr  message
;        .byte " FREE:"
         .byte ' '
         .byte 'M';+$80
;        .byte 'R'+$80
;        .byte 'E'+$80
;        .byte 'E'+$80
;        .byte ':'
         .byte 0
         lda  eob+1
         sec
         sbc  end+1
         tay
         lda  eob
         sbc  end
         tax
         tya
         JSR OutputXA

         jsr  message
         .byte ' ','T',0;+$80,0
         LDX sob
         LDA sob+1
         JSR OutputXA

         jsr  message
         .byte '/',0
         LDX end
         LDA end+1
         JSR OutputXA

         jsr  message
         .byte '/',0
         LDX eob
         LDA eob+1
         JSR OutputXA

         jsr  message
         .byte "   "                   ; 3 spaces
         .byte 0                       

         rts

; -------------------------------------
OutputXATest:
        
         TAY

         LDA FARVEC
         PHA

         LDA #BANK1
         STA FARVEC

         lda ptr
         pha
         lda ptr+1
         pha

         TYA

         JSR OutputXA

;        jsr  message
;        .byte "BEEP!",$0d,0

         pla 
         sta ptr+1
         pla
         sta ptr

         PLA
         STA FARVEC

         rts

; -------------------------------------
OutputXA:
         JSR XA2STR
         JSR ShowBuf
         RTS

; -------------------------------------
; The MESSAGE routine can be used much the way Basic's PRINT command is.
; All ASCII text following JSR MESSAGE will be printed.
; This text must be followed by a zero byte;
; execution resumes on the byte following the zero.

; print in source messages
message:
         ldy  #0
         pla
         sta  ptr
         pla
         sta  ptr+1
b19:     inc  ptr
         bne  f33
         inc  ptr+1
f33:     lda  (ptr),y
         beq  f34
         JSK  print
         bne  b19
f34:     lda  ptr+1
         pha
         lda  ptr
         pha
         rts

; -------------------------------------
XA2STR:
;        LDX #$00
;        LDA #$8B
         STX VAL+0
         STA VAL+1

; -------------------------------------
; Convert an 16 bit binary value into a 24bit BCD value
;
; Another use for BCD is in the conversion of binary values to decimal
; ones. Some algorithms perform this conversion by counting the number of
; times that 10000's, 1000's, 100's, 10's and 1's can be subtracted from the
; binary value before it underflows, but I normally use a simple fixed loop
; that shifts the bits out of the binary value one at a time and adds it to
; an intermediate result that is being doubled (in BCD) on each iteration.

BIN2BCD:
         LDA #0                        ; Clear the result area
         STA RES+0
         STA RES+1
         STA RES+2
         LDX #16                       ; Setup the bit counter
         SED                           ; Enter decimal mode
_LOOP:   ASL VAL+0                     ; Shift a bit out of the binary
         ROL VAL+1                     ; ... value
         LDA RES+0                     ; And add it into the result, doubling
         ADC RES+0                     ; ... it at the same time
         STA RES+0
         LDA RES+1
         ADC RES+1
         STA RES+1
         LDA RES+2
         ADC RES+2
         STA RES+2
         DEX                           ; More bits to process?
         BNE _LOOP
         CLD                           ; Leave decimal mode

; -------------------------------------
BCD2STR:
         LDY #0                        ; least significant digit
         LDA RES+2
         AND #$F0
         LSR
         LSR
         LSR
         LSR
         CLC
         ADC #'0'
         JSR AddToBuf

         LDA RES+2
         AND #$0F
         CLC
         ADC #'0'
         JSR AddToBuf

         LDA RES+1
         AND #$F0
         LSR
         LSR
         LSR
         LSR
         CLC
         ADC #'0'
         JSR AddToBuf

         LDA RES+1
         AND #$0F
         CLC
         ADC #'0'
         JSR AddToBuf

         LDA RES
         AND #$F0
         LSR
         LSR
         LSR
         LSR
         CLC
         ADC #'0'
         JSR AddToBuf

         LDA RES                       ; most significant digit
         AND #$0F
         CLC
         ADC #'0'
         STA XABUF,Y

         INY                           ; end of string marker
         LDA #0
         STA XABUF,Y

         RTS

; -------------------------------------
; Add digit to buf and ignore leading zeros
AddToBuf:                              
         CMP #'0'
         BEQ AddToBuf2
AddToBuf1:
         STA XABUF,Y
         INY
         RTS
AddToBuf2:
         CPY #0
         BEQ AddToBuf3
         BNE AddToBuf1
AddToBuf3:
         RTS

ShowBuf:
         LDY #0
ShowBuf1:
         LDA XABUF,Y
         CMP #0
         BEQ ShowBuf2
         JSK print
         INY
         JMP ShowBuf1
ShowBuf2:
         RTS

VAL:     .BYTE 0,0
RES:     .BYTE 0,0,0
XABUF:   .BYTE 0,0,0,0,0,0,0

; -------------------------------------
; my ready
myready:
         lda  end
         sta  endpos
         lda  end+1
         sta  endpos+1

;        PLA
;        LDA #BANK0
; 	 STA FARVEC

         PLA                           ; drop return address to getkey
         PLA

         JSR RestoreZPArea

         PLA
         STA keychk                    ; restore pointer on keyput routine

         PLA
  	 STA FARVEC

         RTS

; -------------------------------------
SaveZPArea:
	 LDX #(ZPAreaSize-1)
SaveZPArea1:
	 LDA ZPAreaBeg,X
	 STA SavedZPArea,X
	 DEX
	 BPL SaveZPArea1
	 RTS

; -------------------------------------
RestoreZPArea:
      	 LDX #(ZPAreaSize-1)
RestoreZPArea1:
      	 LDA SavedZPArea,X
      	 STA ZPAreaBeg,X
      	 DEX
      	 BPL RestoreZPArea1
      	 RTS                       

; -------------------------------------
SavedZPArea:
         .RES ZPAreaSize

; -------------------------------------
;
; 1. The CRSR keys work pretty much the way you would expect. Use them to 
;    move (the cursor) or scroll (the window) up, down, left and right.
;    You cannot CRSR out of the range of entered text.
;
; 2. The INST/DEL key is used to insert and delete one character at
;    a time just the way it is in Basic.
;
; 3. The RETURN key is non-destructive and does nothing but advance
;    you to the next line.
;
; 4. F1 deletes whole lines to the right and can be used to join
;    two lines as well.
;
; 5. F2 inserts a line, ie. a CHR$(13). It can be used to split a
;    line in two as well as open up space.
;
; 6. F3 and F4 allow vertical page scrolling.
;    F3 takes you down and F4 up one screen at a time.
;
; 7. F5 and F6 control horizontal paging.
;    F5 moves you one screen to the right and F6 one
;    screen to the left.
;
; 8. RUN/STOP exits to Basic where any immediate commands may be
;    used, ED<RETURN> will put you back in the editor.

; command entries
commands:
         .byte 148                     ; inst
         .word insrtspc-1
         .byte 20                      ; del
         .word deletechr-1
         .byte 133                     ; f1
         .word deleteln-1
         .byte 137                     ; f2
         .word insertln-1
         .byte 134                     ; f3
         .word pagedown-1
         .byte 138                     ; f4
         .word pageup-1
         .byte 135                     ; f5
         .word pageright-1
         .byte 139                     ; f6
         .word pageleft-1
         .byte 3                       ; run stop
         .word myready-1
         .byte 17                      ; (cursor down)
         .word down-1
         .byte 145                     ; (cursor up)
         .word up-1
         .byte 157                     ; (cursor left)
         .word left-1
         .byte 29                      ; (cursor right)
         .word right-1
         .byte 167                     ; cbm-m
         .word monitor-1

commandnum = * - commands

endofprg:
	.byte 0
	.byte 0

endoftext:
	.byte 0
        .byte 0

        .reloc
        .global MRED_END
MRED_END:

.endif
