; vim:ts=9:sw=9:sta:et:sts=9:ai:syn=a65

.include "config.i"

.ifdef with_RaDi

.include "RaDi.i"
.include "m16.i"

CR       = 13

.ifdef C64                             ; C64 defines
ptr      = $FD                         ;
MEMSTART = ENDOFRADI                   ;
MEMEND   = $CFFF                       ;
.ifdef Dbg                             ;
marker   = $02                         ;
.endif                                 ;            

.else

.ifdef C128                            ; C128 defines
         .import sobpos,eobpos
ptr      = $FD                         ;
MEMSTART = sobpos                      ;
MEMEND   = eobpos-1                    ;
.ifdef Dbg                             ;
marker   = $07FF                       ;
.endif                                 ;

.endif
.endif

         .segment "RADI"

         .global RADI_START
RADI_START:

.ifdef C64
;        .word $C000
         .org $C000
.else
.ifdef C128
;        .word $1300
         .org RaDi_DestAddr
.endif
.endif
         .export InitRaDi
InitRaDi:
         LDA #<ICLOSE
         STA lICLOSE
         LDA #>ICLOSE
         STA lICLOSE+1

         LDA #<IOPEN
         STA lIOPEN
         LDA #>IOPEN
         STA lIOPEN+1

         LDA #<IGETIN
         STA lIGETIN
         LDA #>IGETIN
         STA lIGETIN+1

         LDA #<ICHKIN
         STA lICHKIN
         LDA #>ICHKIN
         STA lICHKIN+1

         LDA #<ICKOUT
         STA lICKOUT
         LDA #>ICKOUT
         STA lICKOUT+1

         LDA #<IBASIN
         STA lIBASIN
         LDA #>IBASIN
         STA lIBASIN+1

         LDA #<IBSOUT
         STA lIBSOUT
         LDA #>IBSOUT
         STA lIBSOUT+1

.ifdef WithSAVELOAD
         LDA #<ILOAD
         STA lILOAD
         LDA #>ILOAD
         STA lILOAD+1

         LDA #<ISAVE
         STA lISAVE
         LDA #>ISAVE
         STA lISAVE+1
.endif
         RTS

; =====================================
         .export MPTRTAB
MPTRTAB:                               ; memory pointer table
MFROM:   .word MEMSTART
MTO:     .word MEMEND
MPTR:    .word 0000
;MEOF:    .word 0000
         .import endpos
MEOF     = endpos ; $0B00
; =====================================
IncPTR:  INC16 MPTR
         RTS

SetMEOF: LDA MPTR
         STA MEOF
         LDA MPTR+1
         STA MEOF+1
         RTS

RewindM: LDA MFROM
         STA MPTR
         LDA MFROM+1
         STA MPTR+1
         RTS

MemFull: CMP16LT MTO,MPTR
         RTS

MemEOF:  CMP16LT MPTR,MEOF
         RTS
; =====================================
InitMem: LDA #<MEMSTART
         STA MFROM
         STA MPTR
         LDA #>MEMSTART
         STA MFROM+1
         STA MPTR+1

         LDA #<MEMEND
         STA MTO
         LDA #>MEMEND
         STA MTO+1

         JSR SetMEOF
         RTS
; =====================================
IOPEN:
         LDX zLA                       ; Logische Filenummer
.ifdef C64
         BNE IOPEN1                    ; ungleich null?
         JMP kERROR6                   ; nein -> 'not input file' Fehler
; -------------------------------------
IOPEN1:
.endif
         JSR kLOOKUP                   ; Find File in the Logical File Table
         BNE IOPEN2                    ; ungleich null?
         JMP kERROR2                   ; gef. -> 'file open' Fehler
; -------------------------------------
IOPEN2:
         LDX zLDTND                    ; Anzahl der offenen Files
         CPX #10                       ; mit 10 vergleichen
         BCC IOPEN3                    ; wenn kleiner, weiter
         JMP kERROR1                   ; sonst -> 'too many open files' Fehler
; -------------------------------------
IOPEN3:
         INC zLDTND                    ; Anzahl der offenen Files erhhen
         LDA zLA                       ; Logische Filenummer in die
         STA lLAT,X                    ; log. Dateinummerntabelle eintragen
         LDA zSA
         ORA #$60                      ; Print, Input, Get in
         STA zSA                       ; in Sekundradresse eintragen
         STA lSAT,X                    ; Sek.Adr. in SA Tabelle eintragen
         LDA zFA                       ; Device Number
         STA lFAT,X                    ; in Gerteadressentabelle eintrage
; -------------------------------------
IOPEN4:
         CMP #7                        ; unser Gert?
         BEQ IOPEN5                    ; ja, -> Sonderbehandlung
         JMP NOPEN1                    ; nein, weiter mit Betriebssystemroutine
; -------------------------------------
IOPEN5:

         LDA zSA                       ; Sekundradresse holen
         AND #$0F                      ; oberes Nibble (Bit 4-7) ausmaskieren
         BEQ IOPEN6                    ; -> to open for read

                                       ; === open for write ===
.ifdef Dbg
         JSR message
         .byte "OPENED FILE FOR WRITE",CR,0
.endif
         JSR InitMem

         LDA #0
         CLC
         RTS

IOPEN6:                                ; === open for read ===
         .ifdef Dbg
         JSR message
         .byte "OPENED FILE FOR READ",CR,0
         .endif

         JSR RewindM

         LDA #0
.ifdef Dbg         
         STA marker
.endif         
         CLC
         RTS

; =====================================
ICLOSE:
.ifdef C128
         ROR zSVXT                     ; Carry als Kz. in Z-Page Flag rotieren
.endif
         JSR kJLTLK                    ; sucht logische Filenummer
         BEQ ICLOSE1                   ; gefunden, schlieen ->
         CLC                           ; File nicht vorhanden,
         RTS                           ; dann fertig
; -------------------------------------
ICLOSE1:                               ; schlieen der Datei
         JSR kJZ100                    ; Fileparameter setzen
         TXA                           ; A = X (Index der gef. Eintrags)
         PHA                           ; auf Stapel merken
         LDA zFA                       ; Device Number
         CMP #7                        ; Unser gert?
         BEQ ICLOSE2                   ; ja, -> unsere Datei schlieen
         JMP NCLOSE1                   ; nein, Close des Betriebssystems
; -------------------------------------
ICLOSE2:                               ; unser Close durchfhren
         LDA zSA
         AND #1                        ; Schreibfile (SE==1)?
         BEQ ICLOSE3                   ; Nein, ->

.ifdef Dbg
         JSR message
         .BYTE "FLUSHING WRITE FILE",CR,0
.endif

ICLOSE3:
.ifdef Dbg
         JSR message
         .BYTE "CLOSING FILE",CR,0
.endif
         JMP NCLOSE2                   ; weiter mit Close des Betriebssystems
                                       ; zum Erniedrigen der Anzahl der off.
                                       ; Dateien und Zusammenschieben der
                                       ; Tabellen ab dem nun leeren Eintrag.

; =====================================
IBSOUT:
         PHA
         LDA zDFLTO                    ; if our device is the
         CMP #7                        ; default output device
         BEQ IBSOUT1                   ; then do special treatment ->
         PLA
         JMP NBSOUT                    ; else use normal kernal routine
; -------------------------------------
IBSOUT1:
         PLA
IBSOUTI:
         STA zPTR1                     ; pass data in zPTR1
         TXA
         PHA
         TYA
         PHA

.ifdef Dbg
         LDA zDFLTO
         PHA

         LDA #3
         STA zDFLTO

         JSR message
         .BYTE "IBSOUT: <",0
.endif

         JSR MemFull
         BCC IBSOUT3                   ; yes ->

         LDA MPTR
         STA IBSOUT2+1
         LDA MPTR+1
         STA IBSOUT2+2

         LDA FARVEC                    ; save current farvec
         PHA                           ; of/for user program

         LDA MMUCR
         STA FARVEC

         LDA zPTR1
.ifdef Dbg
         STA marker
.endif         
;        LDY #0
;IBSOUT2:
;        STA MPTR,Y
         JSR simm
IBSOUT2: .BYTE BANK1,$00,$00

         PLA                           ; restore saved 
         STA FARVEC                    ; farvec for user program

         JSR IncPTR
         JSR SetMEOF
IBSOUT3:

.ifdef Dbg
         LDA zPTR1
         JSR kBSOUT

         JSR message
         .BYTE ">",CR,0

         PLA
         STA zDFLTO
.endif

         CLC
         PLA
         TAY
         PLA
         TAX
         LDA zPTR1
         RTS

; =====================================
IBASIN:
         LDA zDFLTN                    ; if our device is the
         CMP #7                        ; default input device
         BEQ IBASIN1                   ; then do special treatment ->
         JMP NBASIN                    ; else use normal kernal routine
; -------------------------------------
IBASIN1:
         STX zXSAV                     ; save .X value

.ifdef Dbg
         JSR message
         .BYTE "IBASIN:",0
.endif

         JSR MemEOF
         BCS EOF2                      ; yes ->

         LDA MPTR
         STA IBASIN2+1
         LDA MPTR+1
         STA IBASIN2+2

;        LDY #0
;IBASIN2:
;        LDA MPTR,Y

         LDA FARVEC                    ; save current FARVEC for
         PHA                           ; of/for user program

         LDA MMUCR
         STA FARVEC

         JSR fimm
IBASIN2: .BYTE BANK1,$00,$00

         TAX

         PLA                           ; restore saved 
         STA FARVEC                    ; farvec for user program
         
         JSR IncPTR

         JSR MemEOF
         BCS EOF                       ; yes ->
         TXA
         JMP IBASIN3
                                       ; End Of File reached:
EOF2:
         LDA #$42                      ; End Of File status und Timeout
         JSR kUDST                     ; Update Status ($90)
         LDA #CR                       ; return CR in .A
         JMP IBASIN3
EOF:
         LDA #$40                      ; End Of File status
         JSR kUDST                     ; Update Status ($90)
         TXA

IBASIN3:
.ifdef Dbg
         PHA
         JSR kBSOUT
         LDA #CR
         JSR kBSOUT
         PLA
.endif
         CLC
         LDX zXSAV                     ; restore .X value
         RTS

; =====================================
ICKOUT:
         JSR kLOOKUP
         BEQ ICKOUT1
         JMP kERROR3                   ; "file not open" error
; -------------------------------------
ICKOUT1:
         JSR kJZ100                    ; routine to fetch table entries
         LDA zFA                       ; if Device Number
         CMP #7                        ; is our special device
         BEQ ICKOUT2                   ; then use special kernal entry
         JMP NCHKOUT1                  ; else use normal  kernal entry
; -------------------------------------
ICKOUT2:

.ifdef Dbg
         PHA
         JSR message
         .BYTE "ICKOUT",CR,0
         PLA
.endif

         JMP NCHKOUT2

; =====================================
ICHKIN:
         JSR kLOOKUP
         BEQ ICHKIN1
         JMP kERROR3                   ; "file not open" error
; -------------------------------------
ICHKIN1:
         JSR kJZ100                    ; routine to fetch table entries
         LDA zFA                       ; if Device Number
         CMP #7                        ; is our special device
         BEQ ICHKIN2                   ; then use special kernal entry
         JMP NCHKIN1                   ; else use normal  kernal entry
; -------------------------------------
ICHKIN2:

.ifdef Dbg
         PHA
         JSR message
         .BYTE "ICHKIN",CR,0
         PLA
.endif

         JMP NCHKIN2

; =====================================
JmpVec2: .WORD NGETIN
; -------------------------------------
IGETIN:
         LDA zDFLTN                    ; if the default input device
         CMP #7                        ; is out special device
         BEQ IGETIN2                   ; then use        kernal BASIN
         JMP (JmpVec2)                 ; else use normal kernal GETIN
; -------------------------------------
IGETIN2:

.ifdef Dbg
         PHA
         JSR message
         .BYTE "IGETIN",CR,0
         PLA
.endif

         JMP BASIN

.ifdef WithSAVELOAD
; =====================================
ILOAD:
         STA zVERCK
         LDA #0
         STA zSTATUS
         LDA zFA
         CMP #7
         BEQ ILOAD2
         SEC
         AND #$E
         BNE ILOAD1
         CLC
                                
ILOAD1:
         JMP NLOAD1
; ------------------------------
ILOAD2:
LD25:    LDX zSA                       ; save SA in .X
         JSR kLUKING                   ; tell user looking
         LDA #$60                      ; special load command
         JSR IOPEN5                    ; open the file

         JSR IBASIN1                   ; get first byte
         STA zEAL

         LDA zSTATUS                   ; test status for error
         LSR A
         LSR A
         BCS LD90                      ; ERROR 4 "file not found"

         JSR IBASIN1                   ; get second byte
         STA zEAH

         TXA                           ; find out old SA
         BNE LD30                      ; SA<>0 use disk address
                                       ; else load where user wants
         LDA zMEMUSS                   ; cassette load temps (2 bytes)
         STA zEAL                      ; end addr load/save low
         LDA zMEMUSS+1
         STA zEAH                      ; end addr load/save high

LD30:    JSR kLODING                   ; tell user "loading..." in direct mode

LD40:    LDA #$FD                      ; mask off timeout
         AND zSTATUS
         STA zSTATUS
         JSR kSTOP                     ; stop key?
         BNE LD45                      ; no...

         JMP kBREAK                    ; stop key pressed

LD45:    JSR IBASIN1                   ; get next byte
         LDY zVERCK                    ; performing verify?
         BEQ LD50                      ; no... load
         LDY #0
         CMP (zEAL),Y                  ; verify it
         BEQ LD60                      ; ok...
         LDA #$10                      ; no good... verify error (SPERR)
         JSR kUDST                     ; update status
         .BYTE $2C                     ; skip next store

LD50:    STA (zEAL),Y
LD60:    INC zEAL                      ; increment store addr
         BNE LD64
         INC zEAH
LD64:    BIT zSTATUS                   ; EOI/EOF?
         BVC LD40                      ; no... continue load

LD180:   JMP NLOAD2                    ; good exit
;        CLC
;        LDX zEAL
;        LDY zEAH
;        RTS

LD90:    JMP kERROR4                   ; ERROR 4 "file not found"

; =====================================
ISAVE:
         LDA zFA
         CMP #7
         BEQ ISAVE2
         SEC
         AND #$0E
         BNE ISAVE1
         CLC
ISAVE1:				  
         JMP NSAVE1
; --------------------------------------
ISAVE2:				  
         LDA #$61                      ; special save command
         STA zSA
SV25:    JSR IOPEN5                    ; open the file
         JSR kSAVING                   ; tell user "saving..." in direct mode

         LDY #0
         JSR kRD300                    ; restore starting address...
                                       ; (SAH,SAL) = (STAH,STAL)
                                       ; ($AD,$AC) =  ($C2,$C1)
         LDA zSAL
         JSR IBSOUTI                   ; output start address low byte
         LDA zSAH
         JSR IBSOUTI                   ; output start address high byte

SV30:    JSR kCMPSTE                   ; compare start to end
         BCS SV50                      ; have reached end
         LDA (zSAL),Y                  ; get byte to save
         JSR IBSOUTI                   ; output it
         JSR kSTOP                     ; stop key?
         BNE SV40                      ; no...

BREAK:   LDA #0                        ; break exit
         SEC                           ; set flag for "break" message
         RTS

SV40:    JSR kINCSAL                   ; increment current addr.
         BNE SV30

SV50:    CLC                           ; clean exit
         RTS

.endif

; =====================================

; The MESSAGE routine can be used much the way Basic's PRINT command is.
; All ASCII text following JSR MESSAGE will be printed.
; This text must be followed by a zero byte;
; execution resumes on the byte following the zero.

; print in source messages
message:
         ldy  #0
         pla
         sta  ptr
         pla
         sta  ptr+1
b19:     inc  ptr
         bne  f33
         inc  ptr+1
f33:     lda  (ptr),y
         beq  f34

         JSR  kBSOUT

         bne  b19
f34:     lda  ptr+1
         pha
         lda  ptr
         pha
         rts

;SaveRegs:
; 	 php
;	 txa
;	 pha
;	 tya
;	 pha
;	 rts

;RestoreRegs:
; 	 pla
;	 tay
;	 pla
;	 tax
; 	 pla
; 	 plp
;	 rts
ENDOFRADI:
         .reloc
         .global RADI_END
RADI_END:

.endif
