; vim:ts=9:sw=9:sta:et:sts=9:ai:syn=a65

        .list on
        .listbytes unlimited
        .debuginfo +

        .include "config.i"
        .include "lisp128.i"
        .include "m16.i"

BANK0    = $3F                         ; MMU Configuration Mask for Bank 0
BANK1    = $7F                         ; MMU Configuration Mask for Bank 1
BANK15   = $00                         ; MMU Configuration Mask for Bank 15
MMUCR    = $FF00

.ifdef Move_Stacks
STACKBANK= BANK1
.endif

;FREKZP1 = $FA
;FREKZP2 = $FC
;FREKZP3 = $FE
;SCREEN  = $0400

        .import jimm,simm,fimm,JSFRTS
        .segment "CODE"

        .org $1BFF

        .word BasicSYSLine
BasicSYSLine:
        .word Main
LineNumber = 128
        .byte <LineNumber
        .byte >LineNumber
        .byte $FE,$02,$20,$30,$3A      ; bank 0:
;       .byte $FE,$02,$31,$35,$3A      ; bank15:
        .byte $9E                      ; SYS Basic Token
        .byte "7186",0                 ; sys $1C12
        .byte 0,0

; ----------------------------------------------------------------------------

.macro JSK KernAddr
        JSR jimm
        .byte BANK15,>KernAddr,<KernAddr
.endmacro

.macro IOSTASH StoreAddr
        JSR simm
        .byte BANK15,<StoreAddr,>StoreAddr
.endmacro

.macro STF Bank,StoreAddr
        JSR simm
        .byte Bank,<StoreAddr,>StoreAddr
.endmacro

.macro IOFETCH FetchAddr
        JSR fimm
        .byte BANK15,<FetchAddr,>FetchAddr
.endmacro


;jimm     = $0d00
;simm     = $0d03
;fimm     = $0d06
JSRFAR    = $02CD
FARVEC    = JSRFAR+$11

;MMUCR     = $FF00

FETCH     = $02A2                      ; Fetch subroutine in RAM
FETVEC    = $02AA                      ; Vector patch location for FETCH
STASH     = $02AF                      ; Stash routine in RAM
STAVEC    = $02B9                      ; Vector patch location for STASH

;LoadAddress = $1c01
StrBuf    = $087d

; ---------------------------------------
;.segment "Lisp128"

Main:
        LDA #BANK0
        STA FARVEC
        STA MMUCR

        JSR Bootstrap

        LDA #BANK0
        STA FARVEC
        STA MMUCR

        JMP Init

CfgArea:
MaxLine: .WORD 00038
HighWaterMark: .WORD $0080
EndOfProgPtr: .WORD EndOfProg

FSLTOP: .WORD $FEF0 ;$DFF0
DSTACK: .WORD $E000
ASTACK: .WORD $FEFE ;ASTACK:  .WORD $FFFE
.ifdef Debug_Stacks
DSTACKMAX: .WORD $0000
ASTACKMIN: .WORD $0000
.endif
        .WORD HashBase
NodeListPtr: .WORD NodeList
HashBasePtr: .WORD HashBase
HashBase: .WORD nNIL
        .WORD HashBase
        .BYTE 'A'                      ; Type 1: LITERAL; Flag 4 set
HashTable: .WORD Chain00
; HashTable[$00]
HashTable_02: .WORD Chain01            ; HashTable[$02]
HashTable_04: .WORD Chain02            ; HashTable[$04]
HashTable_06: .WORD Chain03            ; ...
HashTable_08: .WORD Chain04
HashTable_0A: .WORD Chain05
HashTable_0C: .WORD Chain06
HashTable_0E: .WORD Chain07
HashTable_10: .WORD Chain08
HashTable_12: .WORD Chain09
HashTable_14: .WORD Chain0A
HashTable_16: .WORD Chain0B
HashTable_18: .WORD Chain0C
HashTable_1A: .WORD Chain0D
HashTable_1C: .WORD Chain0E
HashTable_1E: .WORD Chain0F
HashTable_20: .WORD Chain10
HashTable_22: .WORD Chain11
HashTable_24: .WORD Chain12
HashTable_26: .WORD Chain13
HashTable_28: .WORD Chain14
HashTable_2A: .WORD Chain15
HashTable_2C: .WORD Chain16
HashTable_2E: .WORD Chain17
HashTable_30: .WORD Chain18
HashTable_32: .WORD Chain19
HashTable_34: .WORD Chain1A
HashTable_36: .WORD Chain1B
HashTable_38: .WORD Chain1C
HashTable_3A: .WORD Chain1D
HashTable_3C: .WORD Chain1E
HashTable_3E: .WORD Chain1F            ; HashTable[3E] (last HashTable entry)
        .WORD HashBase
        .WORD HashBase
        .WORD HashBase

; =====================================
ShowCopyright:
        LDY #0
; =====================================
MsgB:
        LDA aCopyright,Y
        INY
        JSK kCHROUT
        CMP #0                         ; (A - M) -> NZC
        BNE MsgB                       ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
aCopyright: .BYTE $93
aCopyright2: .BYTE $D,"(C)PETER FELDTMANN, ROMAN BARTKE  ",$D,$D
        .BYTE 0
        .BYTE 0
.if C128

hNMI:   SEI
        PHA
        
        TXA
        PHA

        TYA
        PHA

        LDA MMUCR                      ; alte MMU Config
        LDA #BANK0
        PHA                            ; merken

        LDA #BANK15                    ; bank 15 setzen
        STA MMUCR                      ; setzen

        CLD                            ; Dezimale Betriebsart zurcksetzen
        LDA #$7F                       ; NMI Kennzeichen setzen
        STA CIA2ICR                    ; NMI Mglichkeit lschen

        LDY CIA2ICR                    ; Flags lesen und lschen
        BMI DoRS232NMI                 ; Prfe, ob RS-232 aktiv ist

;
; Check for STOP Key down
;
        JSR UD60                       ; SET STOP FLAG HERE 
                                     ; (Pos 60 of UpdateTime)

        JSR kSTOP                      ; STOP -- CHECK STOP KEY FLAG AND
                                     ; RETURN Z FLAG SET IF FLAG TRUE.
                                     ; ALSO CLOSES ACTIVE CHANNELS AND
                                     ; FLUSHES KEYBOARD QUEUE.
                                     ; ALSO RETURNS KEY DOWNS FROM LAST
                                     ; KEYBOARD ROW IN .A.

;       BNE DoRS232NMI                 ; NO STOP KEY...TEST FOR RS232

        PLA
        STA MMUCR

        PLA
        TAY
        
        PLA
        TAX
 
        PLA

        JMP hBRK
; -------------------------------------

DoRS232NMI:
        JSR NMI_RS232

        PLA
        STA MMUCR
        
        PLA
        TAY
        
        PLA
        TAX

        PLA
        RTI
; -------------------------------------
.endif
.if C128
;
; Where system goes on a BRK instruction
;
hBRK:
        LDA #$00
        STA MMUCR
        STA FARVEC

        JSR RESTOR                     ; RESTORE SYSTEM INDIRECTS
        JSR IOINIT                     ; RESTORE I/O FOR BASIC
;	JSR JPCINT                     ; RESTORE SCREEN FOR BASIC

        JSR kCLRCHN

        LDA #$FF
        JSR kCLOSE
        JSR kCLRCHN
        LDA #$40                       ; '@'      ; Kernal error messages only are on
        STA zNSGFLG

        LDA #BANK0
        STA MMUCR
        STA FARVEC

        CLI                            ; I <- 0

;	LDX #$FF
;	TXS

        JSR Init0
        JMP Interpreter
.endif

; =====================================
.ifdef Move_Stacks
SetReturnBank:
        PHA
        LDA #BANK0
        STA FARVEC
        PLA
        RTS
.endif
; =====================================
InitDStack:
        LDX DSTACK
        STX DStackPtrLo                ; Initialize StackPtrLo from DSTACK setting
        LDA DSTACK+1
        STA DStackPtrHi1+2             ; Initialize StackPtrHi-References from DSTACK+1 setting
        STA DStackPtrHi2+2
        STA DStackPtrHi3+2
        STA DStackPtrHi4+2
        JSR PushHashBase2DStack        ; PUSHD NIL
        JSR PushHashBase2DStack        ; PUSHD NIL
        JMP PopD2WA                    ; WA = POPD i.e. WA = NIL
; =====================================
InitAStack:
        LDX ASTACK
        STX AStackPtrLo
.ifdef Debug_Stacks
        STX ASTACKMIN
.endif
        LDA ASTACK+1
        STA AStackPtrHi1+2
        STA AStackPtrHi2+2
        STA AStackPtrHi3+2
        STA AStackPtrHi4+2
.ifdef Debug_Stacks
        STA ASTACKMIN+1
.endif
        RTS                            ; PC <- (Stack)
; -------------------------------------
PushDStack:
        LDX DStackPtrLo
        INX
        INX
        BNE PushDStack1                ; if Z=0, PC = PC + offset
        INC DStackPtrHi1+2
        INC DStackPtrHi2+2
        INC DStackPtrHi3+2
        INC DStackPtrHi4+2
PushDStack1:
        STX DStackPtrLo
.ifdef Move_Stacks
        JSR SetReturnBank
        STX DStackPtrHi1+1
        JSR simm
DStackPtrHi1:
        .BYTE STACKBANK,$00,$00
.else
DStackPtrHi1:
        STA StackPtrAbs,X
.endif
        TYA
        INX

.ifdef Move_Stacks
        STX DStackPtrHi2+1
        JSR simm
DStackPtrHi2:
        .BYTE STACKBANK,$00,$00
.else
DStackPtrHi2:
        STA StackPtrAbs,X
.endif

PushDStack2:
        LDA DStackPtrHi1+2
        CMP AStackPtrHi1+2             ; (A - M) -> NZC
        BCS StackOverFlowError         ; if C=1, PC = PC + offset
.ifdef Debug_Stacks
        JSR HoldMaxDStackSize
.endif
        RTS                            ; PC <- (Stack)
; -------------------------------------
StackOverFlowError:
        JSR HashBaseToNodePtr
        LDA #<aStackOverflow           ; "STACK OVERFLOW"
        LDY #>aStackOverflow           ; "STACK OVERFLOW"
        JMP ShowErrorMsg
; -------------------------------------
.ifdef Debug_Stacks
HoldMaxDStackSize:
        CMP16LT DSTACKMAX,DStackPtrHi1+1
        BCS HoldMaxDStackSize1
        LDA DStackPtrHi1+1
        STA DSTACKMAX
        LDA DStackPtrHi1+2
        STA DSTACKMAX+1
HoldMaxDStackSize1:
        RTS
; -------------------------------------
HoldMaxAStackSize:
        CMP16LT ASTACKMIN,AStackPtrHi1+1
        BCC HoldMaxAStackSize1
        LDA AStackPtrHi1+1
        STA ASTACKMIN
        LDA AStackPtrHi1+2
        STA ASTACKMIN+1
HoldMaxAStackSize1:
        RTS
.endif
; -------------------------------------
; AX = POP from DATA stack
PopDStack:
;       SEI                            ; I <- 1
;       LDX #$35                       ; '5'      ; BASIC, no KERNAL
;       STX zR6510                     ; we want to read stack values 
                                       ; from ram below kernal rom
        LDX DStackPtrLo
.ifdef Move_Stacks
        JSR SetReturnBank

        STX DStackPtrHi3+1
        JSR fimm
DStackPtrHi3:
        .BYTE STACKBANK,$00,$00
.else
DStackPtrHi3:
        LDA StackPtrAbs,X
.endif
        INX
.ifdef Move_Stacks
        PHA
        STX DStackPtrHi4+1
        JSR fimm
DStackPtrHi4:
        .BYTE STACKBANK,$00,$00
        TAY
        PLA
.else
DStackPtrHi4:
        LDY StackPtrAbs,X
.endif
        DEX
        BNE PopDStack1                 ; if Z=0, PC = PC + offset
        DEC DStackPtrHi1+2
        DEC DStackPtrHi2+2
        DEC DStackPtrHi3+2
        DEC DStackPtrHi4+2
PopDStack1:
        DEX
        DEX
        STX DStackPtrLo
;       LDX #$36                       ; '6'      ; no BASIC, KERNAL
;       STX zR6510
;       CLI                            ; I <- 0
        LDX DStackPtrHi1+2
        CPX DSTACK+1                   ; (X - M) -> NZC
        BCC EmptyStackError            ; if C=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
EmptyStackError:
        JSR HashBaseToNodePtr
        LDA #<aEmptyStack              ; "EMPTY STACK"
        LDY #>aEmptyStack              ; "EMPTY STACK"
        JMP ShowErrorMsg
; =====================================
DupDStack:
        JSR PopDStack
        PHA                            ; Stack <- (A)
        JSR PushDStack
        PLA
        JMP PushDStack
; -------------------------------------
PushYA2AStack:
        LDX AStackPtrLo
        BNE PushAStack1                ; if Z=0, PC = PC + offset
        DEC AStackPtrHi1+2
        DEC AStackPtrHi2+2
        DEC AStackPtrHi3+2
        DEC AStackPtrHi4+2
PushAStack1:
        DEX
        DEX
        STX AStackPtrLo
.ifdef Move_Stacks
        JSR SetReturnBank
        STX AStackPtrHi1+1
        JSR simm
AStackPtrHi1:
        .BYTE STACKBANK,$00,$00
.else
AStackPtrHi1:
        STA StackPtrAbs,X
.endif
        TYA
        INX

.ifdef Move_Stacks
        STX AStackPtrHi2+1
        JSR simm
AStackPtrHi2:
        .BYTE STACKBANK,$00,$00
.else
AStackPtrHi2:
        STA StackPtrAbs,X
.endif
.ifdef Debug_Stacks
        JSR HoldMaxAStackSize
.endif
        JMP PushDStack2
; -------------------------------------
PopAStack2YA:
;       SEI                            ; I <- 1
;       LDX #$35                       ; '5'      ; switch KERNAL ROM off
;       STX zR6510

        LDX AStackPtrLo

.ifdef Move_Stacks
        JSR SetReturnBank

        STX AStackPtrHi3+1
        JSR fimm
AStackPtrHi3:
        .BYTE STACKBANK
        .WORD $0000
.else
AStackPtrHi3:
        LDA StackPtrAbs,X
.endif
        INX
.ifdef Move_Stacks
        STX AStackPtrHi4+1
        PHA
        JSR fimm
AStackPtrHi4:
        .BYTE STACKBANK
        .WORD $0000
        TAY
        PLA
.else
AStackPtrHi4:
        LDY StackPtrAbs,X
.endif
        INX
        STX AStackPtrLo
        BNE PushAStack2                ; if Z=0, PC = PC + offset
        INC AStackPtrHi1+2
        INC AStackPtrHi2+2
        INC AStackPtrHi3+2
        INC AStackPtrHi4+2
PushAStack2:
;       LDX #$36                       ; '6'      ; switch BASIC ROM off
;       STX zR6510
;       CLI                            ; I <- 0
        RTS                            ; PC <- (Stack)
; =====================================
InputLine:
        LDA zDFLTN
        CMP #7
        BNE InputLine0

        LDA zSTATUS
        BEQ InputLine0
        
        LDX #0
        LDA #'N'
        STA lBUF,X
        INX
        LDA #'I'
        STA lBUF,X
        INX
        LDA #'L'
        STA lBUF,X
        INX
        LDA #$D
        STA lBUF,X
;       LDA #$D
;       JSK kCHROUT         
        JMP InputLine2

InputLine0:
        LDX #0

InputLine1:
        JSK kCHRIN
        CMP #$D                        ; (A - M) -> NZC
        BEQ InputLine2                 ; if Z=1, PC = PC + offset
        STA lBUF,X
        INX
        CPX #80                        ; (X - M) -> NZC
        BNE InputLine1                 ; if Z=0, PC = PC + offset

InputLine2:
        LDA #0
        STA lBUF,X
        STA lBUFIndex
        LDA zDFLTN
        BEQ PrintCR                    ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
PrintCR:
        LDA #0
        STA LinePos

        LDA #$D
        JSK kCHROUT
        RTS
; =====================================
GetNextFromLBuf:
        INC lBUFIndex
; =====================================
GetCurrentFromLBuf:
        LDX lBUFIndex
        LDA lBUF,X
        RTS                            ; PC <- (Stack)
;
; get char from line buffer skipping space, shift-space and comma
; =====================================
GetNextNoWhiteSpaceChar:
        JSR GetCurrentFromLBuf
GetNextNoWhiteSpaceChar1:
        BNE GetNextNoWhiteSpaceChar2   ; if Z=0, PC = PC + offset
        JSR InputLine
        JMP GetNextNoWhiteSpaceChar
; -------------------------------------
GetNextNoWhiteSpaceChar2:
        CMP #' '                       ; (A - M) -> NZC
        BEQ GetNextNoWhiteSpaceChar3   ; if Z=1, PC = PC + offset
        CMP #$A0                       ; ''      ; SHIFT-SPACE
        BEQ GetNextNoWhiteSpaceChar3   ; if Z=1, PC = PC + offset
        CMP #','                       ; (A - M) -> NZC
        BEQ GetNextNoWhiteSpaceChar3   ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
GetNextNoWhiteSpaceChar3:
        JSR GetNextFromLBuf
        JMP GetNextNoWhiteSpaceChar1
; ==========================================================
; (ACC32) = 0
; AX = (EndOfProgram)
; (FSLPtr) = AX
; do {
;   (NodePtr) = AX
;   (NextPtr) = AX
;   (NextPtr) += 5
;   (NodePtr)[0..1] = #HashBase ($0912)
;   (NodePtr)[2..3] = (NextPtr)[0..1]
;   (NodePtr)[4] = 0
;   (ACC32)++
;   AX = (NextPtr)
; } while ((NextPtr) < (FSLTOP))
;
; (NodePtr) = #$0912
;
; $3b09  12 09 0e 3b 00
;        |   | |   | ||
;        |   | |   | ++-- Flag
;        |   | +---+----- NextPtr
;        +---+----------- HashBase
; $3b0e  12 09 13 3b 00
; $3b13  12 09 18 3b 00
; $3b18  12 09 1d 3b 00
; $3b1d  12 09 22 3b 00
; $3b22  12 09 27 3b 00
; $3b27  12 09 2c 3b 00
; $3b2c  12 09 31 3b 00
; $3b31  12 09 36 3b 00
; $3b36  12 09 3b 3b 00
; $3b3b  12 09 40 3b 00
; $3b40  12 09 45 3b 00
; $3b45  12 09 4a 3b 00
; $3b4a  12 09 4f 3b 00
; $3b4f  12 09 54 3b 00
; $3b54  12 09 59 3b 00
; $3b59  12 09 5e 3b 00
; $3b5e  12 09 63 3b 00
; $3b63  12 09 68 3b 00
; ...
; $cfbf  12 09 c4 cf 00
; $cfc4  12 09 c9 cf 00
; $cfc9  12 09 ce cf 00
; $cfce  12 09 d3 cf 00
; $cfd3  12 09 d8 cf 00
; $cfd8  12 09 dd cf 00
; $cfdd  12 09 e2 cf 00
; $cfe2  12 09 e7 cf 00
; $cfe7  12 09 ec cf 00
; $cfec  12 09 12 09 00
;        |   | |   | ||
;        |   | |   | ++-- Flag ($CFF0 = FSLTOP)
;        |   | +---+----- NextPtr (=HashBase!)
;        +---+----------- HashBase
; $cff1  ff ff ff ff ff
; $cff6  ff ff ff ff ff
; $cffb  ff ff ff ff ff
; $d000  00 00 00 00 00
;
; =====================================
InitFSL:
        JSR MoveZero2ACC32
        LDA EndOfProgPtr
        LDX EndOfProgPtr+1
        STA FSLPtr
        STX FSLPtr+1
InitFSL1:
        STA NodePtr
        STX NodePtr+1
        STA NextPtr
        STX NextPtr+1
;
        LDA NextPtr
        CLC                            ; C <- 0
        ADC #5
        STA NextPtr
        LDA NextPtr+1
        ADC #0
        STA NextPtr+1
;
        LDY #0
        LDA #<HashBase
        STA (NodePtr),Y
        INY
        LDA #>HashBase
        STA (NodePtr),Y
        INY
        LDA NextPtr
        STA (NodePtr),Y
        INY
        LDA NextPtr+1
        STA (NodePtr),Y
        INY
        LDA #0
        STA (NodePtr),Y
        INC ACC32
        BNE InitFSL2                   ; if Z=0, PC = PC + offset
        INC ACC32+1
InitFSL2:
        LDA NextPtr
        LDX NextPtr+1
        CPX FSLTOP+1                   ; (X - M) -> NZC
        BNE InitFSL1                   ; if Z=0, PC = PC + offset
        CMP FSLTOP                     ; (A - M) -> NZC
        BCC InitFSL1                   ; if C=0, PC = PC + offset
;
InitLastNode:
        LDA #<HashBase
        LDY #2
        STA (NodePtr),Y
        INY
        LDA #>HashBase
        STA (NodePtr),Y
        RTS                            ; PC <- (Stack)
; ============================================================================
; NodePtr = FSLPtr
; We take a fresh node from Free Mem Table (FSL)
; and let NodePtr reference it.
;
; FSLPtr = FSLPtr->NextPtr
; Then we let Free Mem Table begin at it
; the successor Node.
;
; If NodePtr doesn't reference HashBase
; (NodePtr != HashBase),
; we got a free node, otherwise
; we need a garbage collection.
;
; On return we have NodePtr referencing a new
; allocated node.
; =====================================
AllocateNode:
        LDA FSLPtr
        LDX FSLPtr+1
        STA NodePtr
        STX NodePtr+1                  ; NodePtr = FSLPtr
        LDY #2
        LDA (FSLPtr),Y
        TAX
        INY
        LDA (FSLPtr),Y
        STA FSLPtr+1
        STX FSLPtr                     ; FSLPtr = FSLPtr->NextPtr
        JSR CmpNodePtrWithHashBase
        BEQ NeedGC                     ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
NeedGC:
        JSR DoGC
        LDA NodesFree+1
        BNE AllocateNode               ; if Z=0, PC = PC + offset
        LDA NodesFree
        CMP HighWaterMark              ; (A - M) -> NZC
        BCS AllocateNode               ; if C=1, PC = PC + offset
        LDA #<hRESET
        LDY #>hRESET
        JSR YA_JSR
        JMP hNMI
; =====================================
ReadNextLine:
        PHA                            ; Stack <- (A)
        JSR ClearReadBuf
        LDY #0
        STA Flag_1D
        PLA
        CMP #$22                       ; '"'      ; (A - M) -> NZC
        BEQ ReadNextLine4              ; if Z=1, PC = PC + offset
ReadNextLine2:
        STA READBUF+1,Y
        INY
        JSR GetNextFromLBuf
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #' '                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #$A0                       ; ''      ; SHIFT-SPACE
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #','                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #'.'                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #')'                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #'('                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #']'                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #'['                       ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #$27                       ; '''      ; (A - M) -> NZC
        BEQ ReadNextLine3              ; if Z=1, PC = PC + offset
        CMP #$22                       ; '"'      ; (A - M) -> NZC
        BNE ReadNextLine2              ; if Z=0, PC = PC + offset
ReadNextLine3:
        STY READBUF                    ; ; set length of BSTR
        RTS                            ; PC <- (Stack)
; -------------------------------------
; read double quoted string literal
ReadNextLine4:
        JSR GetNextFromLBuf
        BEQ ReadNextLine6              ; if Z=1, PC = PC + offset
        CMP #$22                       ; '"'      ; (A - M) -> NZC
        BEQ ReadNextLine5              ; if Z=1, PC = PC + offset
        STA READBUF+1,Y
        INY
        BNE ReadNextLine4              ; if Z=0, PC = PC + offset
ReadNextLine5:
        JSR GetNextFromLBuf
ReadNextLine6:
        LDA #$FF
        STA Flag_1D
        JMP ReadNextLine3
; =====================================
CmpStrBufWithREADBUF:
        LDY #0
        LDA READBUF
        CMP StrBuf                     ; compare length first
        BNE CmpStrBufWithREADBUF2      ; if Z=0, PC = PC + offset
        TAY                            ; if equal check for zero size
        BEQ CmpStrBufWithREADBUF2      ; if Z=1, PC = PC + offset
CmpStrBufWithREADBUF1:
        LDA StrBuf,Y                   ; compare contents
        CMP READBUF,Y                  ; (A - M) -> NZC
        BNE CmpStrBufWithREADBUF2      ; if Z=0, PC = PC + offset
        DEY
        BNE CmpStrBufWithREADBUF1      ; if Z=0, PC = PC + offset
CmpStrBufWithREADBUF2:
        RTS                            ; PC <- (Stack)
; =====================================
ClearReadBuf:
        LDX #$80
        LDA #0
ClearReadBuf1:
        STA READBUF,X
        DEX
        BPL ClearReadBuf1              ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; Omit processor stack in this JSR call and use ASTACK instead.
; So we have larger stack compared with the processor one.
; =====================================
YA_JSR:
        STA CallPtr
        STY CallPtr+1
        PLA
        CLC                            ; C <- 0
        ADC #1
        STA ReturnAddress
        PLA
        ADC #0
        TAY
        LDA ReturnAddress
        JSR PushYA2AStack
        JMP (CallPtr)
; -------------------------------------
YA_RTS:
        JSR PopAStack2YA
        STA ReturnAddress
        STY ReturnAddress+1
        JMP (ReturnAddress)
; =====================================
MakeString:
        JSR LinkStringCharacters       ; push ATOM for Data node
        JSR PushHashBase2DStack        ; push NIL to DSTACK for Next-Link
        JSR CALLhCONS
        JSR PopD2NodePtr               ; get constructed S-Expr
        LDY #4                         ; set type for Node 4=ATOM LITERAL
        LDA #1
        LDX Flag_1D
        BEQ MakeString1                ; if Z=1, PC = PC + offset
        LDA #8                         ; set type fo node 8=STRING LITERAL
MakeString1:
        STA (NodePtr),Y
        JMP PushNodePtrToDStack        ; push it again to DSTACK
; -------------------------------------
MkStringAndPutIntoHashTable:
        JSR MakeString
        JSR DupDStack
        JSR InitNextPtrWithHashTableIndex
        JSR PushNextPtr2DStack
        JSR CALLhCONS
        JSR PopD2NextPtr
        LDX HashTableIndex             ; get corresponding hash table bucket
        LDA NextPtr                    ; put new node at head of this hash table bucket's list
        STA HashTable,X
        LDA NextPtr+1
        STA HashTable+1,X
        RTS                            ; PC <- (Stack)
; ============================================================================
; search object in its associated HashLists chain
; C=0: not found
; C=1: found
; =====================================
SearchObject:
        JSR InitNextPtrWithHashTableIndex
SearchObject1:
        JSR CmpNextPtrWithHashBase
        BNE SearchObject2              ; if NextPtr == NIL (=HashBase), nothing to remove
        CLC                            ; equal object not found
        RTS                            ; PC <- (Stack)
; -------------------------------------
SearchObject2:
        JSR NextDataPtrToNodePtr
        JSR MoveNodePtr2StringPtr
        JSR StrPtrFormat
        JSR CmpStrBufWithREADBUF
        BEQ SearchObject3              ; found equal object
        JSR NextPtrNextToNextPtr       ; move to next object
        JMP SearchObject1
; -------------------------------------
SearchObject3:
        SEC                            ; equal object found
        RTS                            ; PC <- (Stack)
; -------------------------------------
DoReadRest3:
        JSR MakeHash
        JSR SearchObject               ; search object in its associated HashLists chain
        BCC DoMkStringAndPutIntoHashTable; not found, ->
        JMP PushNodePtrToDStack
; -------------------------------------
DoMkStringAndPutIntoHashTable:
        JMP MkStringAndPutIntoHashTable
; -------------------------------------
hREAD:
        LDA #0
        STA ARG32NegativeFlag
        LDA #<hRead1
        LDY #>hRead1
        JSR YA_JSR
        JMP YA_RTS
; -------------------------------------
hRead0:
        LDA #<hRead5
        LDY #>hRead5
        JSR YA_JSR
        LDA #0
        STA ARG32NegativeFlag
        JMP YA_RTS
.ifdef WithPROMPT        
Depth:  .BYTE 0
.endif
; -------------------------------------
hRead1:
        LDA #0
        STA ARG32NegativeFlag
        JSR GetNextNoWhiteSpaceChar
        CMP #'('                       ; (A - M) -> NZC
        BEQ hRead5                     ; if Z=1, PC = PC + offset
        CMP #$27                       ; '''      ; (A - M) -> NZC
        BNE hRead2                     ; if Z=0, PC = PC + offset
        JMP hReadQuoted
; -------------------------------------
hRead2:
        CMP #'['                       ; (A - M) -> NZC
        BEQ hRead0                     ; if Z=1, PC = PC + offset
        CMP #')'                       ; (A - M) -> NZC
        BEQ hRead3                     ; if Z=1, PC = PC + offset
        CMP #']'                       ; (A - M) -> NZC
        BEQ hRead3                     ; if Z=1, PC = PC + offset
        CMP #'.'                       ; (A - M) -> NZC
        BNE hRead4                     ; if Z=0, PC = PC + offset
hRead3:
        JMP hRead11
; -------------------------------------
hRead4:
        JSR CallReadRest
        JMP YA_RTS
; -------------------------------------
hRead5:
        LDA #0
        STA ARG32NegativeFlag
        JSR GetNextFromLBuf
hRead6:
        JSR GetNextNoWhiteSpaceChar
        CMP #']'                       ; (A - M) -> NZC
        BEQ hRead13                    ; if Z=1, PC = PC + offset
        CMP #')'                       ; (A - M) -> NZC
        BEQ hRead12                    ; if Z=1, PC = PC + offset
        LDA #<hRead1
        LDY #>hRead1
        JSR YA_JSR
        LDA ARG32NegativeFlag
        BEQ hRead7                     ; if Z=1, PC = PC + offset
        JSR PushHashBase2DStack
        JMP hCONS
; -------------------------------------
hRead7:
        JSR GetNextNoWhiteSpaceChar
        CMP #'.'                       ; (A - M) -> NZC
        BEQ hRead8                     ; if Z=1, PC = PC + offset
        LDA #<hRead6
        LDY #>hRead6
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hRead8:
        JSR GetNextFromLBuf
        LDA #<hRead1
        LDY #>hRead1
        JSR YA_JSR
        JSR CALLhCONS
        LDA ARG32NegativeFlag
        BEQ hRead9                     ; if Z=1, PC = PC + offset
        JMP YA_RTS
; -------------------------------------
hRead9:
        JSR GetNextNoWhiteSpaceChar
        CMP #')'                       ; (A - M) -> NZC
        BNE hRead10                    ; if Z=0, PC = PC + offset
        JSR GetNextFromLBuf
        JMP YA_RTS
; -------------------------------------
hRead10:
        CMP #']'                       ; (A - M) -> NZC
        BNE hRead11                    ; if Z=0, PC = PC + offset
        JSR GetNextFromLBuf
        LDA #$FF
        STA ARG32NegativeFlag
        JMP YA_RTS
; -------------------------------------
hRead11:
        JSR HashBaseToNodePtr
        LDA #<aReadError               ; "READ ERROR"
        LDY #>aReadError               ; "READ ERROR"
        JMP ShowErrorMsg
; -------------------------------------
hRead12:
        JSR PushHashBase2DStack
        JSR GetNextFromLBuf
        JMP YA_RTS
; -------------------------------------
hRead13:
        JSR PushHashBase2DStack
        JSR GetNextFromLBuf
        LDA #$FF
        STA ARG32NegativeFlag
        JMP YA_RTS
; =====================================
Msg:
        STA MsgPtr
        STY MsgPtr+1
Msg1:
        LDY #0
Msg2:
        LDA (MsgPtr),Y
        BEQ MsgDone                    ; if Z=1, PC = PC + offset
        JSR NextChrOut
        INY
        BNE Msg2                       ; if Z=0, PC = PC + offset
MsgDone:
        RTS                            ; PC <- (Stack)
; -------------------------------------
aReadError:      .BYTE "READ ERROR",0
aUndefinedFunct: .BYTE "UNDEFINED FUNCTION",0
aBreak:          .BYTE $D,"BREAK",$D,0
aStackOverflow:  .BYTE "STACK OVERFLOW",0
aEmptyStack:     .BYTE "EMPTY STACK",0
; -------------------------------------
; ============================================================================
; CONS:
; Node = AllocateNode
; Node.Next = PopD
; Node.Data = PopD
; PushD Node
hCONS:
        JSR AllocateNode
        JSR PopD2NextPtr
        JSR NextPtrToNodePtrNext
        JSR PopD2NextPtr
        JSR NextPtrToNodePtrData       ; NodePtr->Data = NextPtr
        JSR PushNodePtrToDStack
        JMP YA_RTS
; =====================================
CALLhCONS:
        LDA #<hCONS
        LDY #>hCONS
        JMP YA_JSR
; -------------------------------------
PrintLiteral:
        JSR StrPtrFormat
PrintLiteral1:
        LDY CRandWordWrapFlag
        BEQ ShowStrBuf                 ; if Z=1, PC = PC + offset
        LDA StrBuf                     ; get string size
        BEQ ShowStrBuf3                ; empty string
        JSR IfNecessaryWrapLine
; =====================================
ShowStrBuf:
        LDA StrBuf
        BEQ ShowStrBuf3                ; if Z=1, PC = PC + offset
        LDY #1
ShowStrBuf2:
        LDA StrBuf,Y
        BEQ ShowStrBuf3                ; if Z=1, PC = PC + offset
        JSR NextChrOut
        INY
        BPL ShowStrBuf2                ; if N=0, PC = PC + offset
ShowStrBuf3:
        RTS                            ; PC <- (Stack)
; =====================================
OutChar:
        PHA                            ; Stack <- (A)
        LDA CRandWordWrapFlag
        BEQ OutChar1                   ; if Z=1, PC = PC + offset
        LDA LinePos
        CMP MaxLine                    ; (A - M) -> NZC
        BCC OutChar1                   ; if C=0, PC = PC + offset
        JSR PrintCR
OutChar1:
        PLA
; =====================================
NextChrOut:
        INC LinePos
        JSK kCHROUT
        RTS
;
; Print SEXPR on DSTACK
; =====================================
PrintSExpr:
        LDA #<DoPrintSExpr
        LDY #>DoPrintSExpr
        JMP YA_JSR
; -------------------------------------
DoPrintSExpr:
        JSR ChkBreakAndPause
        JSR PopD2NextPtr
DoPrintSExpr2:
        JSR GetNextPtrTypeInfo
        BEQ DoPrintSExprList           ; if Z=1, PC = PC + offset
DoPrintSExpr3:
        CMP #2                         ; IsNumeric
        BNE DoPrintSExpr4              ; No, ->
        JSR ShowNextPtrNumeric         ; yes
        JMP YA_RTS
; -------------------------------------
DoPrintSExpr4:
        JSR MoveNextPtrToStringPtr
        JSR PrintStringOrLiteral
        JMP YA_RTS
; -------------------------------------
DoPrintSExprList:
        LDA #'('
        JSR OutChar
DoPrintSExprList2:
        JSR PushNextPtr2DStack
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR PrintSExpr
        JSR PopD2NextPtr
        JSR NextPtrNextToNextPtr
        JSR MoveNextPtrToNodePtr
        JSR CmpNextPtrWithHashBase
        BEQ DoPrintSExprList4          ; if Z=1, PC = PC + offset
        LDY #4
        LDA (NodePtr),Y
        AND #$F
        BNE DoPrintSExprList3          ; if Z=0, PC = PC + offset
        LDA #' '
        JSR OutChar
        JMP DoPrintSExprList2
; -------------------------------------
DoPrintSExprList3:
        LDA #'.'
        JSR OutChar
        JSR PushNodePtrToDStack
        JSR PrintSExpr
DoPrintSExprList4:
        LDA #')'
        JSR OutChar
        JMP YA_RTS
; -------------------------------------
hPRINT:
        JSR DupDStack
        LDA #7
        STA CRandWordWrapFlag
        LDA #0
        STA IndentColumn
        JSR PrintSExpr
        JSR PrintCR
hLISTQUOTE:
        JMP YA_RTS
; -------------------------------------
hReadQuoted:
        LDA #<nQUOTE
        LDY #>nQUOTE
        JSR PushDStack
        JSR GetNextFromLBuf
        LDA #<hRead1
        LDY #>hRead1
        JSR YA_JSR
        JSR PushHashBase2DStack
        JSR CALLhCONS
        JMP hCONS
; =====================================
LinkStringCharacters:
        LDA #0
        STA Counter
        LDX #0
LinkStringCharacters1:
        STX SavedXReg
        JSR AllocateNode
        LDX SavedXReg
        LDY #0
LinkStringCharacters2:
        INX
        LDA READBUF,X
        STA (NodePtr),Y
        INY
        CPY #4                         ; (Y - M) -> NZC
        BNE LinkStringCharacters2      ; if Z=0, PC = PC + offset
        LDA #4
        STA (NodePtr),Y                ; set node type = literal (4)
        STX SavedXReg
        JSR PushNodePtrToDStack
        LDX SavedXReg
        INC Counter                    ; count pushes
        LDA READBUF+1,X
        BNE LinkStringCharacters1      ; if Z=0, PC = PC + offset
        JSR PushHashBase2DStack        ; pushd NIL
LinkStringCharacters3:
        LDX Counter
        BEQ LinkStringCharacters4      ; if Z=1, PC = PC + offset
        JSR CALLhCONS
        DEC Counter                    ; more pushes left?
        JMP LinkStringCharacters3
; -------------------------------------
LinkStringCharacters4:
        RTS                            ; PC <- (Stack)
; =====================================
StrPtrFormat:
        LDY #$80
        LDA #0
ClearStrBuf:
        STA StrBuf,Y
        DEY
        BPL ClearStrBuf                ; if N=0, PC = PC + offset
;
        JSR GetStringPtrTypeInfo
        CMP #$F                        ; (A - M) -> NZC
        BNE StrPtrFormat4              ; dereference is needed ->
        LDY #6
        LDX #0
StrPtrFormat2:
        LDA (StringPtr),Y
        STA StrBuf+1,X
        BEQ StrPtrFormat3              ; if Z=1, PC = PC + offset
        INY
        INX
        BNE StrPtrFormat2              ; if Z=0, PC = PC + offset
StrPtrFormat3:
        STX StrBuf
        RTS                            ; PC <- (Stack)
; -------------------------------------
StrPtrFormat4:
        LDY #0
        LDA (StringPtr),Y
        STA StringPtr2
        INY
        LDA (StringPtr),Y
        STA StringPtr2+1
        LDX #0
StrPtrFormat5:
        LDY #0
        LDA (StringPtr2),Y
        STA StringPtr
        INY
        LDA (StringPtr2),Y
        STA StringPtr+1
        LDY #0
StrPtrFormat6:
        LDA (StringPtr),Y
        BEQ StrPtrFormat7              ; string terminator '\0'? -> return
        INC StrBuf
        LDX StrBuf
        STA StrBuf,X
        INY
        CPY #4                         ; (Y - M) -> NZC
        BNE StrPtrFormat6              ; if Z=0, PC = PC + offset
        LDY #2
        LDA (StringPtr2),Y
        TAX
        INY
        LDA (StringPtr2),Y
        STA StringPtr2+1
        STX StringPtr2
        CMP #>HashBase                 ; (A - M) -> NZC
        BNE StrPtrFormat5              ; if Z=0, PC = PC + offset
        CPX #<HashBase                 ; (X - M) -> NZC
        BNE StrPtrFormat5              ; if Z=0, PC = PC + offset
StrPtrFormat7:
        RTS                            ; PC <- (Stack)
; =====================================
Init0:
;       LDA #$36                       ; switch BASIC ROM off
;       STA zR6510

        LDA #0
;       STA lSAREG
;       STA lSXREG
;       STA lSYREG
        STA lSPREG                     ; initialize status register for hCALL

        LDA #<hBRK
        LDY #>hBRK
        STA lCNBINV                    ; set BRK vector
        STY lCNBINV+1
        LDA #<hNMI
        LDY #>hNMI
        STA lNMINV                     ; set NMI vector
        STY lNMINV+1
        LDA #$40                       ; Kernal error messages only are on
        STA zNSGFLG

        JSR SetColors
        JSR ShowBanner

.ifdef SuperTape
        JSR InitSuperTape
.endif
.ifdef with_RaDi
        .import InitRaDi

        SEI
        JSR InitRaDi
        CLI
.endif
        RTS

; -------------------------------------
ShowBanner:
        SEI

        LDA #<aBanner                  ; " *** LISP ... ***\r\r"
        LDY #>aBanner
        JSR Msg

; C128 VIC sprite programming

; When an IRQ occurs, one of the things that the BASIC part of the IRQ routine
; does is to read the values of the shadow registers $11D6-$11E6 and write
; them into the actual registers $D000-$D010 that control the positions of
; sprites on the screen. This means that it is no use to write to the
; registers $D000-$D010 to set sprite positions since the written values will
; be overwritten at the next IRQ. Instead, the shadow registers at $11D6-$11E6
; should be written to in order to set the positions of sprites. If you want
; to write to the actual registers directly, this can be done by setting bit 0
; of the system variable at address $0A04 equal to 0. This tells the Kernal
; that BASIC has not been initialised and therefore the BASIC part of the IRQ
; routine will not be executed. Because of this, some BASIC commands, e.g.
; SOUND, PLAY and SPRITE, will not work.

; The sprite pointers are on the C128 just as on the C64 normally placed at
; $07F8-$07FF. However, on the C128, these pointers have pre-defined values
; pointing to a block of memory at $0E00-$0FFF that is reserved for sprite
; data. Since Power Assembler uses a part of this memory ($0F00-$1000), you
; will probably want to change the values of the sprite pointers. The C64 does
; not have pre-defined values for the sprite pointers.

; The sprite pointers are in the C64 and the C128 always located as the last
; 8 bytes of the 1 kB chunk of screen memory. If you on the C128 use the BASIC
; command GRAPHIC to go to one of the graphics modes, the screen memory uses
; addresses $1C00-$1FFF instead of $0400-$07FF. This means that addresses
; $1FF8-$1FFF are used for sprite pointers instead of addresses $07F8-$07FF. 


VIC      = $D000                       ;VIC Base
WSKSP0   = $07F8                       ;PTR. SPRITE 0
WSKSP1   = $07F9                       ;PTR. SPRITE 1
WSKSP2   = $07FA                       ;PTR. SPRITE 2

BLOCK13  = $0E00                       ;PTR. BEG. OF BLOCK 13
BLOCK14  = $0E40                       ;PTR. BEG. OF BLOCK 14
BLOCK15  = $0E80                       ;SEE ABOVE - BLOCK 15

        LDA #BANK15
        STA MMUCR                    ; bank 15

        LDA $D506
        STA SAVECFG

        LDA #%00000110               ; 4K common RAM, in lower RAM of Bank 0
        STA $D506

        JSR START

        LDA #1
        LDX #40
whiteline:
        STA $D800+80,X
        DEX
        BPL whiteline

        LDA #(0+24)                   ; X-Pos
        STA XPOS

        LDA #<aVersion
        LDY #>aVersion
        STA MsgPtr
        STY MsgPtr+1

        LDY #0
ShowVersion:
        LDA (MsgPtr),Y
        BEQ ShowVersionDone

        JSR kCHROUT
        JSR ShowChr

        INY
        BNE ShowVersion

ShowVersionDone:

        LDA #%00000000
        STA VIC+21                    ; Sprites off

        LDA SAVECFG
        STA $D506

        LDA #$0D
        JSR kCHROUT
        JSR kCHROUT
        JSR kCHROUT
        LDA #YELLOW
        JSR kCHROUT

        LDA #BANK0
        STA FARVEC
        STA MMUCR
        CLI

        RTS

ShowChr:          
        PHA
        TYA
        PHA
        TXA
        PHA

        LDY #8
ShowCh1:LDA XPOS
        STA VIC+0                     ; X-Pos Sprite 0
        STA VIC+2                     ; X-Pos Sprite 1
        JSR Pause
        CLC
        ADC #1
        STA XPOS
        DEY
        BNE ShowCh1

        LDY #2
ShowCh2:JSR Pause
        DEY
        BNE ShowCh2

        PLA
        TAX
        PLA
        TAY
        PLA

        RTS

XPOS:   .BYTE 00
SAVECFG:.BYTE 00

BAR:    .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000
        .BYTE %01000000,%00000000,%00000000

HIDECHR: 
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %00000000,%00000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000
        .BYTE %10111111,%10000000,%00000000


START:
        LDX #$00                      ; clear sprite memory
        LDA #$00
STLOOP: STA BLOCK13,X
        STA BLOCK14,X
        STA BLOCK15,X
        INX
        CPX #63
        BEQ ST0
        JMP STLOOP

ST0:    LDX #$00                      ; init sprite memory
ST0L:   LDA BAR,X
        STA BLOCK13,X
        LDA HIDECHR,X
        STA BLOCK14,X
        INX
        CPX #63
        BEQ ST1
        JMP ST0L

ST1:    LDX #56
        STX WSKSP0                    ; init sprite pointers
        INX
        STX WSKSP1
        INX
        STX WSKSP2
COLOR:  LDA #1                        ; White
        STA VIC+39                    ; Color Sprite 0
        LDA #0                        ; Black
        LDA 53281
        STA VIC+40                    ; Color Sprite 1
        STA VIC+41                    ; Color Sprite 2

        LDA #%00000000
        STA VIC+16                    ; X-Pos > 255?

        LDA #(0+24)                   ; X-Pos
        LDX #(7+(8*2)+30)             ; Y-Pos

        STA VIC                       ; X-Pos Sprite 0
        STX VIC+1                     ; Y-Pos Sprite 0

        STA VIC+2                     ; X-Pos Sprite 1
        STX VIC+3                     ; Y-Pos Sprite 1

        ADC #48                       ; X-Pos (256 + 48 = 304)
        STA VIC+4                     ; X-Pos Sprite 2
        STX VIC+5                     ; Y-Pos Sprite 2

        LDA #%00000000                ; no double size
        STA VIC+23                    ; Double Size Y
        STA VIC+29                    ; Double Size X

        LDA #%00000000                ; no Multicolor
        STA VIC+28                    ; Sprite Multicolor

        LDA #%00000011
        STA VIC+21                    ; Sprite on

        LDA VIC+27                    ; Sprite 0-1 vor dem Hintergrund
        AND #%11111100
        STA VIC+27

        RTS

Pause:
        PHA
        TYA
        PHA
        TXA
        PHA

        LDY #0
PAUSE1: LDX #0
PAUSE2: INX
        CPX #$FE
        BNE PAUSE2
        INY
        CPY #$02
        BNE PAUSE1

        PLA
        TAX
        PLA
        TAY
        PLA

        RTS

; -------------------------------------
BLACK   = 144
WHITE   = 5
RED     = 28
ORANGE  = 129
YELLOW  = 158
GREEN   = 30
CYAN    = 159
BLUE    = 31
PURPLE  = 156

RVSON   = 18
RVSOFF  = 146

UBAR1   = 163
UBAR2   = 183
UBAR3   = 184

LBAR1   = 164
LBAR2   = 175
LBAR3   = 185
LBAR4   = 162

UP      = 145
CLRHOME = 147
; -------------------------------------
aVersion: 
        .BYTE version,0
aBanner: 
        .BYTE CLRHOME,BLACK
        .RES 24,LBAR1
        .BYTE $D
        .BYTE RED,RVSON,LBAR1,LBAR1,RVSOFF
        .BYTE ORANGE,RVSON,LBAR2,LBAR2,RVSOFF
        .BYTE YELLOW,RVSON,LBAR3,LBAR3,RVSOFF
        .BYTE GREEN,UBAR3,UBAR3
        .BYTE CYAN,UBAR2,UBAR2
        .BYTE BLUE,UBAR1,UBAR1
        .BYTE PURPLE,UBAR1,UBAR1,$D,$D
        .RES 10,' '
        .BYTE RED,LBAR1,LBAR1
        .BYTE ORANGE,LBAR1,LBAR1
        .BYTE YELLOW,LBAR2,LBAR2
        .BYTE GREEN,LBAR3,LBAR3
        .BYTE CYAN,LBAR4,LBAR4
        .BYTE BLUE,RVSON,UBAR3,UBAR3,RVSOFF
        .BYTE PURPLE,RVSON,UBAR1,UBAR1,RVSOFF,$D
        .BYTE BLACK
        .RES 24,UBAR1
        .BYTE $D
        .BYTE WHITE,UP,UP,UP,0
aNodesFree: .BYTE " NODES FREE",$D,$D,0
; -------------------------------------
SetStarColor:
        PHA
        LDA #<IndicatorStarColorPos    ; Zeiger auf 
        STA MEMVEC                     ; Farbram
        LDA #>IndicatorStarColorPos    ; setzen
        STA MEMVEC+1
        LDY #0                         ; Write-Offset

        LDA #MEMVEC                    ; Zeiger fr
        STA STAVEC                     ; Stash

        PLA
        LDX #00                        ; MEM-Config (Bank 15)
        JSR STASH
        RTS

GetStarColor:
        LDA #<IndicatorStarColorPos    ; Zeiger auf 
        STA MEMVEC                     ; Farbram
        LDA #>IndicatorStarColorPos    ; setzen
        STA MEMVEC+1
        LDY #0                         ; Write-Offset

        LDA #MEMVEC                    ; Zeiger fr
        STA FETVEC                     ; Stash

        LDX #00                        ; MEM-Config (Bank 15)
        JSR FETCH
        RTS
; -------------------------------------
.ifdef WithPROMPT
Prompt: 
        JSR MoveZero2ACC32
        LDA Depth
        STA ACC32
;       JSR ShowACC32

        LDA #<aPrompt
        LDY #>aPrompt
        JSR Msg
        RTS

aPrompt: .BYTE "> ",0 ; .BYTE 0 ; "> ",0
.endif
; -------------------------------------
Init:
        LDA #BANK0                     ; init FARVEC with
        STA FARVEC                     ; bank 0
        STA MMUCR                      ; in MMU config

        LDY #0                         ; Initialize DSTACK
        LDA DSTACK                     
        STA DStackPtrLo 

        LDA DSTACK+1
        STA DStackPtrLo+1

.ifdef Move_Stacks
        JSR SetReturnBank

        LDA DStackPtrLo
        STA InitTopDStack
        STA InitTopDStack2
        LDA DStackPtrLo+1
        STA InitTopDStack+1
        STA InitTopDStack2+1

        LDA #$FF
        JSR simm
        .BYTE STACKBANK
InitTopDStack:
        .WORD $0000
        INC16 InitTopDStack2

        LDA #$FF
        JSR simm
        .BYTE STACKBANK
InitTopDStack2:
        .WORD $0000
.else
        LDA #$FF
        STA (DStackPtrLo),Y            ; (DSTACK) = $FF
        INY
        STA (DStackPtrLo),Y            ; (DSTACK+1) = $FF
.endif

        JSR Init0
        JSR ResetHashTable
        JSR InitFSL                    ; FreeNodeCount is in ACC32 after call
        JSR ShowACC32
        LDA #<aNodesFree               ; " NODES FREE\r\r"
        LDY #>aNodesFree               ; " NODES FREE\r\r"
        JSR Msg
        LDA #<HashBase
        LDY #>HashBase
        STA HashBasePtr
        STY HashBasePtr+1              ; (word_0910)=#$0912

Interpreter:
        LDA #0
        STA IndentColumn
        STA LinePos

.ifdef WithPROMPT        
        STA Depth
        JSR Prompt
.endif

.ifdef with_AUTOSTART

        LDA InitDone                   ; if AUTOSTART init not done
        BEQ DoInit                     ; do it
        JMP NoInitFound                ; else ignore it
DoInit:
        LDA #1                         ; mark init
        STA InitDone                   ; as already done
        JSR CheckInit                  ; check for autostart file
        BCC LoadInit                   ; found, load it
        JMP NoInitFound                ; else skip it

; -------------------------------------
aLoadInit:
        .BYTE "(LOAD 8 ",34,"INIT",32,32,32,32,32,32,32,32,".LSP",34,")",$D,0
; -------------------------------------
LoadInit:
        LDY #0
LoadInit1:
        LDA aLoadInit,Y                ; output load command on screen ...
        INY
        JSK kCHROUT
        CMP #0
        BNE LoadInit1
                                       ; ... and execute it with emulated
                                       ; key presses:
                                       ; put <CRSP-UP> <RETURN>
                                       ;     <CRSR-UP> <RETURN>
                                       ; into key buffer
        LDA #UP                        ; CRSR-UP
        STA KEYD
        STA KEYD+2
        LDA #$D                        ; RETURN
        STA KEYD+1
        STA KEYD+3
        LDA #4                         ; 4 key presses in keybuf
        STA zNDX

        JMP InterpreterLoop

; -------------------------------------
InitFN: .BYTE "INIT        .LSP,U,R"
InitFNLen = * - InitFN
InitDone: .BYTE 0
; -------------------------------------
CheckInit:
        LDA #BANK15
        JSK kSETBNK

        LDA #InitFNLen
        LDX #<InitFN
        LDY #>InitFN
        JSK kSETNAM

        LDA #2
        LDX #8
        LDY #2
        JSK kSETLFS

        JSK kOPEN                     ; OPEN 2,8,2,"INIT        .LSP,U,R"

        BCC CheckInit1
        SEC                           ; Error in kOPEN
        RTS

CheckInit1:
        JSK kREADST
        BEQ CheckInit2                ; OK ->
        LDA #2
        JSK kCLOSE                    ; CLOSE 2
;       JSK kCLRCHN
        SEC                           ; "INIT        .LSP,U" NOT FOUND
        RTS

CheckInit2:
        LDA #2
        JSK kCLOSE                    ; CLOSE 2
;       JSK kCLRCHN
        CLC                           ; "INIT        .LSP,U" FOUND
        RTS


NoInitFound:
        JSR InputLine
.else         
        JSR InputLine
.endif
InterpreterLoop:
        JSR InitAStack
        JSR InitDStack
        JSR Push0toAStack
        LDA #<HashBase
        LDY #>HashBase
        STA CurrentNode
        STY CurrentNode+1
.ifdef WithPROMPT        
        JSR Prompt
.endif        
        LDA #<hREAD
        LDY #>hREAD
        JSR YA_JSR
        JSR CallEval                   ; after the Eval Call
                                     ; the evaluated value
                                     ; is left on DSTACK
        LDA #<hPRINT                   ; which is printed here
        LDY #>hPRINT
        JSR YA_JSR
        JMP InterpreterLoop
; =====================================
PushNodePtrToDStack:
        LDA NodePtr
        LDY NodePtr+1
        JMP PushDStack
; =====================================
PushNodePtrToAStack:
        LDA NodePtr
        LDY NodePtr+1
        JMP PushYA2AStack
; =====================================
PushWAtoDStack:
        LDA WA
        LDY WA+1
        JMP PushDStack
; =====================================
PushNextPtr2DStack:
        LDA NextPtr
        LDY NextPtr+1
        JMP PushDStack
; =====================================
PushNextPtr2AStack:
        LDA NextPtr
        LDY NextPtr+1
        JMP PushYA2AStack
; =====================================
Push0toAStack:
        LDA #0
        LDY #0
        JMP PushYA2AStack
; =====================================
PushStringPtr2DStack:
        LDA StringPtr
        LDY StringPtr+1
        JMP PushDStack
; =====================================
PushStringPtr2AStack:
        LDA StringPtr
        LDY StringPtr+1
        JMP PushYA2AStack
; ============================================================================
; PUSHD NIL
; =====================================
PushHashBase2DStack:
        LDA #<HashBase
        LDY #>HashBase
        JMP PushDStack
; =====================================
PushNValue2DStack:
        LDA #<nVALUE
        LDY #>nVALUE
        JMP PushDStack
; =====================================
PopD2NodePtr:
        JSR PopDStack
        STA NodePtr
        STY NodePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
PopA2NodePtr:
        JSR PopAStack2YA
        STA NodePtr
        STY NodePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
PopD2WA:
        JSR PopDStack
        STA WA
        STY WA+1
        RTS                            ; PC <- (Stack)
; =====================================
PopD2NextPtr:
        JSR PopDStack
        STA NextPtr
        STY NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
PopA2NextPtr:
        JSR PopAStack2YA
        STA NextPtr
        STY NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
PopD2StringPtr:
        JSR PopDStack
        STA StringPtr
        STY StringPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
PopA2StringPtr:
        JSR PopAStack2YA
        STA StringPtr
        STY StringPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
MoveNodePtr2NextPtr:
        LDA NodePtr
        LDY NodePtr+1
        STA NextPtr
        STY NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
MoveNodePtr2StringPtr:
        LDA NodePtr
        LDY NodePtr+1
        STA StringPtr
        STY StringPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
MoveStringPtrToNodePtr:
        LDA StringPtr
        LDY StringPtr+1
        STA NodePtr
        STY NodePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
MoveNextPtrToNodePtr:
        LDA NextPtr
        LDY NextPtr+1
        STA NodePtr
        STY NodePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
MoveStringPtr2NextPtr:
        LDA StringPtr
        LDY StringPtr+1
        STA NextPtr
        STY NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
MoveNextPtrToStringPtr:
        LDA NextPtr
        LDY NextPtr+1
        STA StringPtr
        STY StringPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
NextPtrDataToNextPtr:
        LDY #0
        BEQ NextPtr_ToNextPtr          ; if Z=1, PC = PC + offset
; =====================================
NextPtrNextToNextPtr:
        LDY #2
NextPtr_ToNextPtr:
        LDA (NextPtr),Y
        TAX
        INY
        LDA (NextPtr),Y
        STA NextPtr+1
        STX NextPtr
        RTS                            ; PC <- (Stack)
; =====================================
NodePtrDataToNodePtr:
        LDY #0
        BEQ NodePtr_ToNodePtr          ; if Z=1, PC = PC + offset
; =====================================
NodePtrNextToNodePtr:
        LDY #2
NodePtr_ToNodePtr:
        LDA (NodePtr),Y
        TAX
        INY
        LDA (NodePtr),Y
        STA NodePtr+1
        STX NodePtr
        RTS                            ; PC <- (Stack)
; =====================================
StringNodeDataToStringPtr:
        LDY #0
        BEQ StringNode_ToStringPtr     ; if Z=1, PC = PC + offset
; =====================================
StringNodePtrNextToStringPtr:
        LDY #2
StringNode_ToStringPtr:
        LDA (StringPtr),Y
        TAX
        INY
        LDA (StringPtr),Y
        STA StringPtr+1
        STX StringPtr
        RTS                            ; PC <- (Stack)
; =====================================
NodePtrDataToStringPtr:
        LDY #0
        BEQ NodePtr_ToStringPtr        ; if Z=1, PC = PC + offset
; =====================================
NodePtrNextToStringPtr:
        LDY #2
NodePtr_ToStringPtr:
        LDA (NodePtr),Y
        STA StringPtr
        INY
        LDA (NodePtr),Y
        STA StringPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
NextPtrToNodePtrData:
        LDY #0
        BEQ NextPtrToNodePtrOffset     ; if Z=1, PC = PC + offset
; =====================================
NextPtrToNodePtrNext:
        LDY #2
NextPtrToNodePtrOffset:
        LDA NextPtr
        STA (NodePtr),Y
        INY
        LDA NextPtr+1
        STA (NodePtr),Y
        RTS                            ; PC <- (Stack)
; =====================================
NodePtrToNextPtrData:
        LDY #0
        BEQ NodePtrToNextPtrOffset     ; if Z=1, PC = PC + offset
; =====================================
NodePtrToNextPtrNext:
        LDY #2
NodePtrToNextPtrOffset:
        LDA NodePtr
        STA (NextPtr),Y
        INY
        LDA NodePtr+1
        STA (NextPtr),Y
        RTS                            ; PC <- (Stack)
; =====================================
CmpNodePtrWithStringPtr:
        LDA NodePtr+1
        CMP StringPtr+1                ; (A - M) -> NZC
        BNE CmpNodePtrWithStringPtr1   ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP StringPtr                  ; (A - M) -> NZC
CmpNodePtrWithStringPtr1:
        RTS                            ; PC <- (Stack)
; =====================================
AreNodeAndNextEqual:
        LDA NodePtr+1
        CMP NextPtr+1                  ; (A - M) -> NZC
        BNE AreNodeAndNextEqual1       ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP NextPtr                    ; (A - M) -> NZC
AreNodeAndNextEqual1:
        RTS                            ; PC <- (Stack)
; =====================================
CmpStringPtrWithNextPtr:
        LDA StringPtr+1
        CMP NextPtr+1                  ; (A - M) -> NZC
        BNE CmpStringPtrWithNextPtr1   ; if Z=0, PC = PC + offset
        LDA StringPtr
        CMP NextPtr                    ; (A - M) -> NZC
CmpStringPtrWithNextPtr1:
        RTS                            ; PC <- (Stack)
;
;
; Z=1 if NextPtr == HashBase
; =====================================
CmpNextPtrWithHashBase:
        LDA NextPtr+1
        CMP #>HashBase                 ; (A - M) -> NZC
        BNE CmpWithHashBase1           ; if Z=0, PC = PC + offset
        LDA NextPtr
        CMP #<HashBase                 ; (A - M) -> NZC
CmpWithHashBase1:
        RTS                            ; PC <- (Stack)
; -------------------------------------
        LDA NextPtr
        ORA NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
CmpNodePtrWithHashBase:
        LDA NodePtr+1
        CMP #>HashBase                 ; (A - M) -> NZC
        BNE CmpNodePtrWithHashBase1    ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP #<HashBase                 ; (A - M) -> NZC
CmpNodePtrWithHashBase1:
        RTS                            ; PC <- (Stack)
; =====================================
CmpStringPtrWithHashBase:
        LDA StringPtr+1
        CMP #>HashBase                 ; (A - M) -> NZC
        BNE CmpStringPtrWithHashBase1  ; if Z=0, PC = PC + offset
        LDA StringPtr
        CMP #<HashBase                 ; (A - M) -> NZC
CmpStringPtrWithHashBase1:
        RTS                            ; PC <- (Stack)
; =====================================
HashBaseToNodePtr:
        LDA #<HashBase
        LDY #>HashBase
        STA NodePtr
        STY NodePtr+1
        RTS                            ; PC <- (Stack)
; -------------------------------------
return_nil:
        JSR PushHashBase2DStack
        JMP YA_RTS
; -------------------------------------
PushNextPtr2DStack_RTS_OnFlgsEq4:
        JSR PushNextPtr2DStack
        JMP YA_RTS
; =====================================
NextDataPtrToNodePtr:
        LDY #0
        BEQ Next_PtrToNodePtr          ; if Z=1, PC = PC + offset
NextNextPtrToNodePtr:
        LDY #2
Next_PtrToNodePtr:
        LDA (NextPtr),Y
        STA NodePtr
        INY
        LDA (NextPtr),Y
        STA NodePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
.ifdef Move_Stacks
NextDataPtrToNodePtrGC:

        JSR SetReturnBank

        LDA NextPtr
        STA NextDataPtrToNodePtrGC1
        LDA NextPtr+1
        STA NextDataPtrToNodePtrGC1+1

        JSR fimm
        .BYTE STACKBANK
NextDataPtrToNodePtrGC1:
        .WORD $0000
        STA NodePtr

        LDA NextDataPtrToNodePtrGC1
        STA NextDataPtrToNodePtrGC2
        LDA NextDataPtrToNodePtrGC1+1
        STA NextDataPtrToNodePtrGC2+1

        INC16 NextDataPtrToNodePtrGC2
        JSR fimm
        .BYTE STACKBANK
NextDataPtrToNodePtrGC2:
        .WORD $0000
        STA NodePtr+1

        RTS                            ; PC <- (Stack)
.endif
; =====================================
NodeDataPtrToNextPtr:
        LDY #0
        BEQ Node_PtrToNextPtr          ; if Z=1, PC = PC + offset
NodeNextPtrToNextPtr:
        LDY #2
Node_PtrToNextPtr:
        LDA (NodePtr),Y
        STA NextPtr
        INY
        LDA (NodePtr),Y
        STA NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
NextDataPtrToStringPtr:
        LDY #0
        BEQ Next_PtrToStringPtr        ; if Z=1, PC = PC + offset
NextNextPtrToStringPtr:
        LDY #2
Next_PtrToStringPtr:
        LDA (NextPtr),Y
        STA StringPtr
        INY
        LDA (NextPtr),Y
        STA StringPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
StringDataPtrToNodePtr:
        LDY #0
        BEQ String_PtrToNodePtr        ; if Z=1, PC = PC + offset
StringNextPtrToNodePtr:
        LDY #2
String_PtrToNodePtr:
        LDA (StringPtr),Y
        STA NodePtr
        INY
        LDA (StringPtr),Y
        STA NodePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
StringDataPtrToNextPtr:
        LDY #0
        BEQ String_PtrToNextPtr        ; if Z=1, PC = PC + offset
; =====================================
StringNextPtrToNextPtr:
        LDY #2
String_PtrToNextPtr:
        LDA (StringPtr),Y
        STA NextPtr
        INY
        LDA (StringPtr),Y
        STA NextPtr+1
        RTS                            ; PC <- (Stack)
; =====================================
CallEval:
        LDA #<hEVAL
        LDY #>hEVAL
        JMP YA_JSR
;
; X==1 -> nLAMBDA
; X==2 -> nNLAMBDA
; X==3 -> nEXPR
; X==4 -> nMACRO
; X==5 -> nEXPR
; X==6 -> nFEXPR
; X==7 -> nVALUE
; =====================================
IsNoHandlerNode:
        LDX #7
IsNoHandlerNode1:
        TXA
        ASL A
        TAY
        LDA NodePtr+1
        CMP NoHandlerNodesTable+1,Y    ; (A - M) -> NZC
        BNE IsNoHandlerNode2           ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP NoHandlerNodesTable,Y      ; (A - M) -> NZC
        BNE IsNoHandlerNode2           ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
IsNoHandlerNode2:
        DEX
        BNE IsNoHandlerNode1           ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
NoHandlerNodesTable:
        .WORD 0
        .WORD nLAMBDA
        .WORD nNLAMBDA
        .WORD nEXPR
        .WORD nMACRO
        .WORD nEXPR
        .WORD nFEXPR
        .WORD nVALUE
; -------------------------------------
hEVAL:
        JSR ChkBreakAndPause
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BEQ hEVAL_OnList               ; if Z=1, PC = PC + offset
        LSR A
        BCS hEval_OnSystemFunction     ; if C=1, PC = PC + offset
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hEval_OnSystemFunction:
        JMP EVALSystemFunction
; -------------------------------------
UndefinedFunctError:
        LDA NodesFree
        LDX NodesFree+1
        STA NodePtr
        STX NodePtr+1
        LDA #<aUndefinedFunct          ; "UNDEFINED FUNCTION"
        LDY #>aUndefinedFunct          ; "UNDEFINED FUNCTION"
        JMP ShowErrorMsg
; =====================================
PushNodesDataPtr2AStack:
        LDY #0
        LDA (NodePtr),Y
        STA ReturnAddress
        INY
        LDA (NodePtr),Y
        TAY
        LDA ReturnAddress
        JMP PushYA2AStack
; -------------------------------------
hEVAL_OnList:
        LDA NextPtr
        LDY NextPtr+1
        STA SavedNextPtr
        STY SavedNextPtr+1
        JSR NextDataPtrToNodePtr       ; NodePtr = NextPtr.Data
        JSR NextPtrNextToNextPtr       ; NextPtr = NextPtr.Next
hEVAL_OnList1:
        JSR GetNodePtrTypeInfo
        BNE hEVAL_OnList5              ; if Z=0, PC = PC + offset
hEVAL_OnList2:
        JSR PushNodePtrToDStack
        JSR NodePtrDataToNodePtr
        LDA NodePtr
        LDX NodePtr+1
        CPX #>nNLAMBDA                 ; (X - M) -> NZC
        BNE hEVAL_OnList3              ; if Z=0, PC = PC + offset
        CMP #<nNLAMBDA                 ; (A - M) -> NZC
        BEQ hEVAL_OnList4              ; if Z=1, PC = PC + offset
hEVAL_OnList3:
        LDA #<OnFlagsEq2
        LDY #>OnFlagsEq2
        JSR YA_JSR
        JMP hAPPLY
; -------------------------------------
hEVAL_OnList4:
        JMP hAPPLY1
; -------------------------------------
hEVAL_OnList5:
        LDX NodePtr
        LDY NodePtr+1
        STX NodesFree
        STY NodesFree+1
        CMP #$F                        ; (A - M) -> NZC
        BEQ hEVAL_OnList5a             ; if Z=1, PC = PC + offset
        CMP #1                         ; (A - M) -> NZC
        BNE UndefinedFunctError        ; if Z=0, PC = PC + offset
        JSR NodePtrNextToStringPtr
hEVAL_OnList6:
        JSR CmpStringPtrWithHashBase
        BEQ UndefinedFunctError        ; if Z=1, PC = PC + offset
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR IsNoHandlerNode
        CPX #4                         ; (X - M) -> NZC
        BCS hEVAL_OnList6a             ; if C=1, PC = PC + offset
        JSR StringNodePtrNextToStringPtr
        JMP hEVAL_OnList6
; -------------------------------------
OnFlagsNotEq7:
        CPX #1                         ; (X - M) -> NZC
        BNE FlagsNot1                  ; if Z=0, PC = PC + offset
        JMP OnFlagsEq1
; -------------------------------------
FlagsNot1:
        CPX #2                         ; (X - M) -> NZC
        BNE FlagsNot2                  ; if Z=0, PC = PC + offset
        JMP OnFlagsEq2
; -------------------------------------
FlagsNot2:
        CPX #3                         ; (X - M) -> NZC
        BNE FlagsNot3                  ; if Z=0, PC = PC + offset
        JMP hAPPLY15_OnFlagsEq3
; -------------------------------------
FlagsNot3:
        CPX #4                         ; (X - M) -> NZC
        BNE FlagsNot4                  ; if Z=0, PC = PC + offset
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
FlagsNot4:
        JMP FlagsGtEq7
; -------------------------------------
hEVAL_OnList6a:
        CPX #7                         ; (X - M) -> NZC
        BCS FlagsGtEq7                 ; if C=1, PC = PC + offset
        LDA NodesFree
        LDY NodesFree+1
        STA CurrentNode
        STY CurrentNode+1
        CPX #4                         ; (X - M) -> NZC
        BEQ DoEval8_5                  ; if Z=1, PC = PC + offset
        JSR StringDataPtrToNodePtr
        JMP hEVAL_OnList2
; -------------------------------------
DoEval8_5:
        JMP Do_Eval8_5_
; -------------------------------------
hEVAL_OnList5a:
        JSR PushNodesDataPtr2AStack
        LDY #5
        LDA (NodePtr),Y
        TAX
        CPX #7                         ; (X - M) -> NZC
        BNE DoEval8_6                  ; if Z=0, PC = PC + offset
        JMP UndefinedFunctError
; -------------------------------------
DoEval8_6:
        JMP OnFlagsNotEq7
; -------------------------------------
FlagsGtEq7:
        JSR StringDataPtrToNodePtr     ; push Node handler's address to AStack
        LDX NodePtr+1
        CPX NodesFree+1                ; (X - M) -> NZC
        BNE DoEval8_7                  ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP NodesFree                  ; (A - M) -> NZC
        BNE DoEval8_7                  ; if Z=0, PC = PC + offset
        JMP UndefinedFunctError
; -------------------------------------
DoEval8_7:
        JMP hEVAL_OnList1
; -------------------------------------
hMAPCAR:
        JSR PopD2NextPtr
        JSR PopD2StringPtr
hMAPCAR1:
        JSR GetNextPtrTypeInfo
        BEQ hMAPCAR2                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMAPCAR2:
        JSR PushStringPtr2AStack
        JSR PushNextPtr2AStack
        JSR PushStringPtr2DStack
        JSR NextPtrDataToNextPtr
        JSR PushNextPtr2DStack
        JSR PushHashBase2DStack
        JSR CALLhCONS
        LDA #<hAPPLY
        LDY #>hAPPLY
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2StringPtr
        JSR NextPtrNextToNextPtr
        LDA #<hMAPCAR1
        LDY #>hMAPCAR1
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hAPPLY:
        JSR PopD2NextPtr
hAPPLY1:
        JSR PopD2StringPtr
hAPPLY2:
        LDA StringPtr
        LDX StringPtr+1
        STA NodesFree
        STX NodesFree+1
        JSR GetStringPtrTypeInfo
        BNE hAPPLY7                    ; if Z=0, PC = PC + offset
        JSR StringDataPtrToNodePtr
        LDA NodePtr+1
        LDX NodePtr
        CMP #>nLAMBDA                  ; '3'      ; (A - M) -> NZC
        BNE hAPPLY4                    ; if Z=0, PC = PC + offset
        CPX #<nLAMBDA                  ; (X - M) -> NZC
        BNE hAPPLY4                    ; if Z=0, PC = PC + offset
hAPPLY3:
        JMP Do_hAPPLY3
; -------------------------------------
hAPPLY4:
        CMP #>nNLAMBDA                 ; '3'      ; (A - M) -> NZC
        BNE hAPPLY5                    ; if Z=0, PC = PC + offset
        CPX #<nNLAMBDA                 ; (X - M) -> NZC
        BEQ hAPPLY3                    ; if Z=1, PC = PC + offset
hAPPLY5:
        CMP #>nLABEL                   ; '6'      ; (A - M) -> NZC
        BNE hAPPLY6                    ; if Z=0, PC = PC + offset
        CPX #<nLABEL                   ; ''      ; (X - M) -> NZC
        BNE hAPPLY6                    ; if Z=0, PC = PC + offset
        JMP hApply17
; -------------------------------------
hAPPLY6:
        JSR PushNextPtr2AStack
        JSR PushStringPtr2DStack
        JSR CallEval
        JSR PopA2NextPtr
        JMP hAPPLY1
; -------------------------------------
hAPPLY7:
        CMP #$F                        ; (A - M) -> NZC
        BEQ hAPPLY10                   ; if Z=1, PC = PC + offset
        JSR MoveStringPtrToNodePtr
hAPPLY8:
        JSR StringNodePtrNextToStringPtr
        JSR CmpStringPtrWithHashBase
        BNE hAPPLY9                    ; if Z=0, PC = PC + offset
        JMP UndefinedFunctError
; -------------------------------------
hAPPLY9:
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR IsNoHandlerNode
        CPX #5                         ; nLAMBDA or nNLAMBDA or nEXPR or nMACRO  ?
        BCC hAPPLY8                    ; yes ->
        JSR StringNodeDataToStringPtr
        JMP hAPPLY2
; -------------------------------------
hAPPLY11:
        CPX #1                         ; (X - M) -> NZC
        BEQ hAPPLY14                   ; if Z=1, PC = PC + offset
        CPX #3                         ; (X - M) -> NZC
        BEQ hAPPLY14                   ; if Z=1, PC = PC + offset
        CPX #7                         ; (X - M) -> NZC
        BNE hAPPLY12                   ; if Z=0, PC = PC + offset
        JMP UndefinedFunctError
; -------------------------------------
hAPPLY12:
        JSR PushNextPtr2DStack
hAPPLY13:
        JSR MoveStringPtrToNodePtr
        JSR PushNodesDataPtr2AStack
        JMP YA_RTS
; -------------------------------------
hAPPLY14:
        LDA #<hAPPLY15_OnFlagsEq3
        LDY #>hAPPLY15_OnFlagsEq3
        JSR YA_JSR
        JMP hAPPLY13
; -------------------------------------
hAPPLY10:
        LDY #5
        LDA (StringPtr),Y
        TAX
        JMP hAPPLY11
; -------------------------------------
hAPPLY15_OnFlagsEq3:
        JSR GetNextPtrTypeInfo
        BEQ hAPPLY16_OnList            ; if Z=1, PC = PC + offset
        JMP YA_RTS
; -------------------------------------
hAPPLY16_OnList:
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack        ; PushD CAR(Next)
        JSR NextPtrNextToNextPtr       ; Next = CDR(Next)
        JMP hAPPLY15_OnFlagsEq3
; -------------------------------------
OnFlagsEq2:
        JSR GetNextPtrTypeInfo
        BEQ OnFlagsEq2_1               ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
OnFlagsEq2_1:
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR NextPtrNextToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ OnFlagsEq2_2               ; if Z=1, PC = PC + offset
        JSR CallEval
        JSR PushHashBase2DStack
        JMP hCONS
; -------------------------------------
OnFlagsEq2_2:
        JSR PushNextPtr2AStack
        JSR CallEval
        JSR PopA2NextPtr
        LDA #<OnFlagsEq2
        LDY #>OnFlagsEq2
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hRPLACA:
        JSR PopD2NodePtr
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BNE hRPLACA1                   ; if Z=0, PC = PC + offset
        JSR NodePtrToNextPtrData
hRPLACA1:
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hRPLACD:
        JSR PopD2NodePtr
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BNE hRPLACD1                   ; if Z=0, PC = PC + offset
        JSR NodePtrToNextPtrNext
hRPLACD1:
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hSET:
        JSR PopD2StringPtr
        JSR PushNValue2DStack
        JSR PushStringPtr2DStack
        JMP hPUTPROP
; -------------------------------------
hSETQ:
        JSR CallEval
        JMP hSET
; =====================================
CallSET:
        LDA #<hSET
        LDY #>hSET
        JMP YA_JSR
; -------------------------------------
hEQ:
        JSR Compare
hEQ1:
        BEQ return_true                ; if Z=1, PC = PC + offset
do_ret_nil:
        JMP return_nil
; -------------------------------------
return_true:
        LDA #<nT
        LDY #>nT
        JSR PushDStack
        JMP YA_RTS
; -------------------------------------
hATOM:
        JSR PopD2NodePtr
        LDY #4
        LDA (NodePtr),Y
        AND #$F
        BEQ do_ret_nil                 ; if Z=1, PC = PC + offset
        BNE return_true                ; if Z=0, PC = PC + offset
hNULL:
        JSR PushHashBase2DStack
        JMP hEQ
; -------------------------------------
hNOT:
        JSR PopD2NextPtr
        JSR CmpNextPtrWithHashBase
        BEQ return_true                ; if Z=1, PC = PC + offset
        BNE do_ret_nil                 ; if Z=0, PC = PC + offset
hNCONC:
        JSR PopD2NextPtr
hNCONC_1:
        JSR GetNextPtrTypeInfo
        BEQ hNCONC_2                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hNCONC_2:
        JSR NextDataPtrToStringPtr
        JSR GetStringPtrTypeInfo
        BEQ hNCONC_3                   ; if Z=1, PC = PC + offset
        JSR NextPtrNextToNextPtr
        JMP hNCONC_1
; -------------------------------------
hNCONC_3:
        JSR PushStringPtr2DStack
hNCONC_4:
        JSR NextPtrNextToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hNCONC_5                   ; if Z=1, PC = PC + offset
        JMP YA_RTS
; -------------------------------------
hNCONC_5:
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
hNCONC_6:
        JSR MoveStringPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR GetStringPtrTypeInfo
        BEQ hNCONC_6                   ; if Z=1, PC = PC + offset
        JSR NextPtrToNodePtrNext
        JSR PopA2NextPtr
        JSR MoveNodePtr2StringPtr
        JMP hNCONC_4
; =====================================
sub_1418:
        JSR PushNodePtrToDStack
        JSR PushNodePtrToAStack
        JSR NodePtrNextToStringPtr
loc_1421:
        JSR CmpStringPtrWithHashBase
        BEQ loc_1444                   ; if Z=1, PC = PC + offset
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        LDA NodePtr+1
        CMP #>nVALUE                   ; '3'      ; (A - M) -> NZC
        BNE loc_143E                   ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP #<nVALUE                   ; ')'      ; (A - M) -> NZC
        BNE loc_143E                   ; if Z=0, PC = PC + offset
        JSR StringNodeDataToStringPtr
        JMP loc_1444
; -------------------------------------
loc_143E:
        JSR StringNodePtrNextToStringPtr
        JMP loc_1421
; -------------------------------------
loc_1444:
        JSR PopA2NodePtr
        JSR PushStringPtr2DStack
        JSR PushNodePtrToDStack
        JSR PushNextPtr2DStack
        JSR CallSET
        JSR PopDStack
        JSR CALLhCONS
        RTS                            ; PC <- (Stack)
; -------------------------------------
S145A:
        JSR CmpNodePtrWithHashBase
        BNE loc_1462                   ; if Z=0, PC = PC + offset
        JMP return_nil
; -------------------------------------
loc_1462:
        JSR GetNodePtrTypeInfo
        BEQ loc_1470                   ; if Z=1, PC = PC + offset
        JSR sub_1418
        JSR PushHashBase2DStack
        JMP loc_149A
; -------------------------------------
loc_1470:
        JSR PushNodePtrToAStack
        JSR NodePtrDataToNodePtr
        JSR MoveNextPtrToStringPtr
        JSR GetNextPtrTypeInfo
        BNE loc_1484                   ; if Z=0, PC = PC + offset
        JSR NextNextPtrToStringPtr
        JSR NextPtrDataToNextPtr
loc_1484:
        JSR PushStringPtr2AStack
        JSR sub_1418
        JSR PopA2NextPtr
        JSR PopA2NodePtr
        JSR NodePtrNextToNodePtr
        LDA #<S145A
        LDY #>S145A
        JSR YA_JSR
loc_149A:
        JMP hCONS
; =====================================
sub_149D:
        LDA #<S145A
        LDY #>S145A
        JSR YA_JSR
        LDA HashBasePtr
        LDY HashBasePtr+1
        JSR PushDStack
        JSR CALLhCONS
        JSR PopDStack
        STA HashBasePtr
        STY HashBasePtr+1
        RTS                            ; PC <- (Stack)
; =====================================
DoUNBIND:
        LDA HashBasePtr
        LDY HashBasePtr+1
        STA NextPtr
        STY NextPtr+1
        JSR NextNextPtrToStringPtr
        JSR CmpNextPtrWithHashBase
        BNE DoUNBIND1                  ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
DoUNBIND1:
        LDA StringPtr
        LDY StringPtr+1
        STA HashBasePtr
        STY HashBasePtr+1
        JSR NextDataPtrToStringPtr
DoUNBIND2:
        JSR CmpStringPtrWithHashBase
        BNE DoUNBIND3                  ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
DoUNBIND3:
        JSR PushStringPtr2AStack
        JSR StringNodeDataToStringPtr
        JSR StringDataPtrToNodePtr
        JSR StringNextPtrToNextPtr
        JSR PushNodePtrToDStack
        JSR PushNValue2DStack
        JSR PushNextPtr2DStack
        LDA #<hPUTPROP
        LDY #>hPUTPROP
        JSR YA_JSR
        JSR PopDStack
        JSR PopA2StringPtr
        JSR StringNodePtrNextToStringPtr
        JMP DoUNBIND2
; -------------------------------------
Do_hAPPLY3:
        JSR StringNodePtrNextToStringPtr
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR PushStringPtr2AStack
        JSR sub_149D
        JSR PushHashBase2DStack
        JSR PopA2StringPtr
Do_hAPPLY3_1:
        JSR GetStringPtrTypeInfo
        BNE Do_hAPPLY3_3               ; if Z=0, PC = PC + offset
        JSR PopDStack
        JSR StringDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR StringNodePtrNextToStringPtr
        JSR GetStringPtrTypeInfo
        BEQ Do_hAPPLY3_2               ; if Z=1, PC = PC + offset
        JSR CallEval
        JMP Do_hAPPLY3_3
; -------------------------------------
Do_hAPPLY3_2:
        JSR PushStringPtr2AStack
        JSR CallEval
        JSR PopA2StringPtr
        JMP Do_hAPPLY3_1
; -------------------------------------
Do_hAPPLY3_3:
        JSR DoUNBIND
        JMP YA_RTS
; -------------------------------------
hDE:
        JSR PopD2NextPtr
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR PushNodePtrToDStack
        LDA #<nEXPR
        LDY #>nEXPR
        JSR PushDStack
        LDA #<nLAMBDA
        LDY #>nLAMBDA
hDE_1:
        JSR PushDStack
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2DStack
        JSR CALLhCONS
        LDA #<hPUTPROP
        LDY #>hPUTPROP
        JSR YA_JSR
        JSR PopDStack
        JMP YA_RTS
; -------------------------------------
hCOND:
        JSR PopD2StringPtr
hCOND1:
        JSR GetStringPtrTypeInfo
        BEQ hCOND3                     ; if Z=1, PC = PC + offset
hCOND2:
        JMP return_nil
; -------------------------------------
hCOND3:
        JSR StringDataPtrToNextPtr
        JSR GetNextPtrTypeInfo
        BNE hCOND2                     ; if Z=0, PC = PC + offset
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR PushStringPtr2AStack
        JSR PushNextPtr2AStack
        JSR CallEval
        JSR PopA2NextPtr
        JSR PopA2StringPtr
        JSR DupDStack
        JSR PopD2NodePtr
        JSR CmpNodePtrWithHashBase
        BNE hCOND4                     ; if Z=0, PC = PC + offset
        JSR PopDStack
        JSR StringNodePtrNextToStringPtr
        JMP hCOND1
; -------------------------------------
hCOND4:
        JSR PopD2StringPtr
        JSR NextPtrNextToNextPtr
hCOND5:
        JSR GetNextPtrTypeInfo
        BEQ hCOND6                     ; if Z=1, PC = PC + offset
        JSR PushStringPtr2DStack
        JMP YA_RTS
; -------------------------------------
hCOND6:
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR NextPtrNextToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hCOND7                     ; if Z=1, PC = PC + offset
        JSR CallEval
        JMP YA_RTS
; -------------------------------------
hCOND7:
        JSR PushNextPtr2AStack
        JSR CallEval
        JSR PopA2NextPtr
        JSR PopD2StringPtr
        JMP hCOND5
; -------------------------------------
hApply17:
        JSR PushNextPtr2AStack
        JSR StringNodePtrNextToStringPtr
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR StringDataPtrToNextPtr
        JSR PushNextPtr2AStack
        JSR sub_149D
        JSR PopA2StringPtr
        JSR PopA2NextPtr
        LDA #<Do_hAPPLY3
        LDY #>Do_hAPPLY3
        JSR YA_JSR
        JSR DoUNBIND
        JMP YA_RTS
; -------------------------------------
ShowErrorMsg:
        STA MsgPtr
        STY MsgPtr+1
        JSR CloseAll
        LDA #0
        STA LinePos
        JSR Msg1
        JSR InitAStack
        JSR InitDStack
        LDA NodePtr+1
;       CMP #9                         ; (A - M) -> NZC
        CMP #>HashBase
; CHECKME and FIXME!
        BEQ ShowErrorMsg1              ; if Z=1, PC = PC + offset
        LDA #':'
        JSR NextChrOut
        LDA #' '
        JSR NextChrOut
        JSR PushNodePtrToDStack
        LDA #<hPRINT
        LDY #>hPRINT
        JSR YA_JSR
        JMP ShowErrorMsg2
; -------------------------------------
ShowErrorMsg1:
        JSR PrintCR
ShowErrorMsg2:
        LDA CurrentNode+1
;       CMP #9                         ; (A - M) -> NZC
        CMP #>HashBase
; CHECKME and FIXME!
        BEQ ShowErrorMsg3              ; if Z=1, PC = PC + offset
        LDA #<aIn                      ; "IN "
        LDY #>aIn                      ; "IN "
        JSR Msg
        LDA CurrentNode
        LDX CurrentNode+1
        STA NodePtr
        STX NodePtr+1
        JSR PushNodePtrToDStack
        LDA #<hPRINT
        LDY #>hPRINT
        JSR YA_JSR
ShowErrorMsg3:
        JMP Interpreter
; -------------------------------------
aIn:  .BYTE "IN ",0
; -------------------------------------
EVALSystemFunction:
        JSR CmpNextPtrWithHashBase
        BEQ EVALSystemFunction3        ; if Z=1, PC = PC + offset
        JSR NextNextPtrToStringPtr
EVALSystemFunction1:
        JSR GetStringPtrTypeInfo
        BNE EVALSystemFunction4        ; if Z=0, PC = PC + offset
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR IsNoHandlerNode
        CPX #7                         ; nVALUE?
        BEQ EVALSystemFunction2        ; yes ->
        JSR StringNodePtrNextToStringPtr
        JMP EVALSystemFunction1
; -------------------------------------
EVALSystemFunction2:
        JSR StringDataPtrToNextPtr
EVALSystemFunction3:
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
EVALSystemFunction4:
        JSR MoveNextPtrToNodePtr
        LDA #<aUnboundVar              ; "UNBOUND VARIABLE"
        LDY #>aUnboundVar              ; "UNBOUND VARIABLE"
        JMP ShowErrorMsg
; -------------------------------------
aUnboundVar: .BYTE "UNBOUND VARIABLE",0
; -------------------------------------
Do_Eval8_5_:
        LDA SavedNextPtr
        LDY SavedNextPtr+1
        STA NextPtr
        STY NextPtr+1
        JSR StringNodeDataToStringPtr
        LDA #<Do_hAPPLY3
        LDY #>Do_hAPPLY3
        JSR YA_JSR
        JMP hEVAL
; -------------------------------------
hDM:
        JSR PopD2NextPtr
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR PushNodePtrToDStack
        LDA #<nMACRO
        LDY #>nMACRO
        JSR PushDStack
        LDA #<nNLAMBDA
        LDY #>nNLAMBDA
        JMP hDE_1
; -------------------------------------
hPROGN:
        JSR PopD2NextPtr
        LDA #<HashBase
        LDY #>HashBase
        STA StringPtr
        STY StringPtr+1
        JMP hCOND5
; -------------------------------------
hCARPROG1:
        JSR PopD2NextPtr
hCARPROG2:
        JSR GetNextPtrTypeInfo
        BNE hCARPROG3                  ; if Z=0, PC = PC + offset
        JSR NextPtrDataToNextPtr
hCARPROG3:
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hCDRGETPROPLIST:
        JSR PopD2NextPtr
hCDRGETPROPLIST1:
        JSR GetNextPtrTypeInfo
        BEQ hCDRGETPROPLIST2           ; if Z=1, PC = PC + offset
        CMP #1                         ; (A - M) -> NZC
        BNE hCDRGETPROPLIST3           ; if Z=0, PC = PC + offset
hCDRGETPROPLIST2:
        JSR NextPtrNextToNextPtr
hCDRGETPROPLIST3:
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hPRIN1:
        LDA #0
        STA CRandWordWrapFlag
        STA IndentColumn
        JSR PrintSExpr
        JMP return_nil
; -------------------------------------
hTERPRI:
        JSR PrintCR
        JMP return_nil
; -------------------------------------
hEXIT:
;       JMP RESET
        JSK RESET
; -------------------------------------
hCOMMENT:
        JSR PopDStack
        JMP return_nil
; -------------------------------------
hDF:
        JSR PopD2NextPtr
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR PushNodePtrToDStack
        LDA #<nFEXPR
        LDY #>nFEXPR
        JSR PushDStack
        LDA #<nNLAMBDA
        LDY #>nNLAMBDA
        JMP hDE_1
; -------------------------------------
hAPPLYSTAR:
        JSR PopD2NextPtr
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR PushNextPtr2AStack
        JSR CallEval
        JSR PopA2NextPtr
        JSR NextNextPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR CALLhCONS
        JMP hEVAL
; -------------------------------------
hPROG:
        JSR PopD2NextPtr
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2AStack
        JSR PushNextPtr2AStack
        LDA #<HashBase
        LDY #>HashBase
        STA NextPtr
        STY NextPtr+1
        JSR sub_149D
        JSR sub_1797
        JSR PopA2NextPtr
hPROG_1:
        LDA #<S17C3
        LDY #>S17C3
        JSR YA_JSR
        LDA IndentColumn
        CMP #2                         ; (A - M) -> NZC
        BEQ hPROG_1                    ; if Z=1, PC = PC + offset
        JSR DoUNBIND
        JSR PopD2NextPtr
        JSR PopD2NodePtr
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; =====================================
sub_1797:
        JSR PopA2NextPtr
loc_179A:
        JSR GetNextPtrTypeInfo
        BEQ loc_17A2                   ; if Z=1, PC = PC + offset
        JMP PushHashBase2DStack
; -------------------------------------
loc_17A2:
        JSR NextDataPtrToStringPtr
        JSR NextPtrNextToNextPtr
        JSR GetStringPtrTypeInfo
        BEQ loc_179A                   ; if Z=1, PC = PC + offset
        JSR PushStringPtr2DStack
        JSR PushNextPtr2DStack
        JSR PushNextPtr2AStack
        JSR CALLhCONS
        JSR sub_1797
        JSR CALLhCONS
        RTS                            ; PC <- (Stack)
; -------------------------------------
loc_17C0:
        JSR PopA2NextPtr
S17C3:
        JSR GetNextPtrTypeInfo
        BEQ loc_17CF                   ; if Z=1, PC = PC + offset
        LDA #0
        STA IndentColumn
        JMP return_nil
; -------------------------------------
loc_17CF:
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR GetNodePtrTypeInfo
        BNE S17C3                      ; if Z=0, PC = PC + offset
        JSR PushNextPtr2AStack
        JSR NodePtrDataToStringPtr     ; StringPtr = NodePtr->Data
        LDA StringPtr
        LDX StringPtr+1
        CPX #>nRETURN                  ; (X - M) -> NZC
        BNE loc_1802                   ; if Z=0, PC = PC + offset
        CMP #<nRETURN                  ; (A - M) -> NZC
        BNE loc_1802                   ; if Z=0, PC = PC + offset
        JSR NodePtrNextToNodePtr
        JSR NodePtrDataToNodePtr
        JSR PushNodePtrToDStack
        JSR PopA2NextPtr
        JSR CallEval
        LDA #1
        STA IndentColumn
        JMP YA_RTS
; -------------------------------------
loc_1802:
        CPX #>nGO                      ; (X - M) -> NZC
        BNE loc_1842                   ; if Z=0, PC = PC + offset
        CMP #<nGO                      ; (A - M) -> NZC
        BNE loc_1842                   ; if Z=0, PC = PC + offset
        JSR NodePtrNextToNodePtr
        JSR NodePtrDataToNodePtr
        JSR DupDStack
        JSR PopD2NextPtr
loc_1816:
        JSR CmpNextPtrWithHashBase
        BNE loc_1821                   ; if Z=0, PC = PC + offset
        JSR MoveStringPtrToNodePtr
        JMP UndefinedFunctError
; -------------------------------------
loc_1821:
        JSR NextDataPtrToStringPtr
        JSR StringNodeDataToStringPtr
        JSR CmpNodePtrWithStringPtr
        BEQ loc_1832                   ; if Z=1, PC = PC + offset
        JSR NextPtrNextToNextPtr
        JMP loc_1816
; -------------------------------------
loc_1832:
        JSR PopAStack2YA
        JSR NextPtrDataToNextPtr
        JSR NextPtrNextToNextPtr
        LDA #2
        STA IndentColumn
        JMP YA_RTS
; -------------------------------------
loc_1842:
        CPX #>nCOND                    ; '2'      ; (X - M) -> NZC
        BEQ loc_1852                   ; if Z=1, PC = PC + offset
loc_1846:
        JSR PushNodePtrToDStack
        JSR CallEval
        JSR PopDStack
        JMP loc_17C0
; -------------------------------------
loc_1852:
        CMP #<nCOND                    ; ''      ; (A - M) -> NZC
        BNE loc_1846                   ; if Z=0, PC = PC + offset
        JSR NodeNextPtrToNextPtr
loc_1859:
        JSR GetNextPtrTypeInfo
        BEQ loc_1861                   ; if Z=1, PC = PC + offset
        JMP loc_17C0
; -------------------------------------
loc_1861:
        JSR NextDataPtrToNodePtr
        JSR GetNodePtrTypeInfo
        BEQ loc_186C                   ; if Z=1, PC = PC + offset
        JMP loc_17C0
; -------------------------------------
loc_186C:
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2AStack
        JSR PushNodePtrToAStack
        JSR NodePtrDataToNodePtr
        JSR PushNodePtrToDStack
        JSR CallEval
        JSR PopD2StringPtr
        JSR PopA2NodePtr
        JSR PopA2NextPtr
        JSR CmpStringPtrWithHashBase
        BEQ loc_1859                   ; if Z=1, PC = PC + offset
        JSR NodeNextPtrToNextPtr
        LDA #<S17C3
        LDY #>S17C3
        JSR YA_JSR
        LDA IndentColumn
        BNE loc_18A0                   ; if Z=0, PC = PC + offset
        JSR PopD2NodePtr
        JMP loc_17C0
; -------------------------------------
loc_18A0:
        CMP #1                         ; (A - M) -> NZC
        BNE loc_18AA                   ; if Z=0, PC = PC + offset
        JSR PopAStack2YA
        JMP YA_RTS
; -------------------------------------
loc_18AA:
        JSR PopAStack2YA
        JMP S17C3
; -------------------------------------
hMAP2CAR:
        JSR PopD2NodePtr
        JSR PopD2NextPtr
        JSR PopD2StringPtr
hMAP2CAR1:
        JSR GetNextPtrTypeInfo
        STA Counter
        LDA (NodePtr),Y
        AND #$F
        ORA Counter
        BEQ hMAP2CAR2                  ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMAP2CAR2:
        JSR PushStringPtr2DStack
        JSR PushStringPtr2AStack
        JSR PushNodePtrToAStack
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
        JSR PushNextPtr2DStack
        JSR NodePtrDataToNodePtr
        JSR PushNodePtrToDStack
        JSR PushHashBase2DStack
        JSR CALLhCONS
        JSR CALLhCONS
        LDA #<hAPPLY
        LDY #>hAPPLY
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2NodePtr
        JSR PopA2StringPtr
        JSR NextPtrNextToNextPtr
        JSR NodePtrNextToNodePtr
        LDA #<hMAP2CAR1
        LDY #>hMAP2CAR1
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hASSOC:
        JMP hSASSOC
; -------------------------------------
hGETPROP:
        JSR PopD2StringPtr
        JSR PopD2NodePtr
        JSR NodePtrCheck4NIL
hGETPROP1:
        JSR NodeNextPtrToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hGETPROP2                  ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hGETPROP2:
        JSR NextPtrDataToNextPtr
        JSR CmpStringPtrWithNextPtr
        BNE hGETPROP3                  ; if Z=0, PC = PC + offset
        JSR NodeNextPtrToNextPtr
        JSR NextPtrNextToNextPtr
        JSR NextPtrDataToNextPtr
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hGETPROP3:
        JSR NodeNextPtrToNextPtr
        JSR NextNextPtrToNodePtr
        JMP hGETPROP1
; -------------------------------------
hPUTPROP:
        JSR PopD2WA
        JSR PopD2StringPtr
        JSR PopD2NodePtr
        JSR NodePtrCheck4NIL
        JSR PushWAtoDStack
hPUTPROP1:
        JSR NodeNextPtrToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hPUTPROP2                  ; if Z=1, PC = PC + offset
        JSR PushNodePtrToDStack
        JSR PushStringPtr2DStack
        JSR PushWAtoDStack
        JSR PushHashBase2DStack
        JSR CALLhCONS
        JSR CALLhCONS
        JSR PopD2NodePtr
        JSR PopD2NextPtr
        JSR NodePtrToNextPtrNext
        JMP YA_RTS
; -------------------------------------
hPUTPROP2:
        JSR NextPtrDataToNextPtr
        JSR CmpStringPtrWithNextPtr
        BEQ hPUTPROP3                  ; if Z=1, PC = PC + offset
        JSR NodeNextPtrToNextPtr
        JSR NextNextPtrToNodePtr
        JMP hPUTPROP1
; -------------------------------------
hPUTPROP3:
        JSR NodeNextPtrToNextPtr
        JSR NextPtrNextToNextPtr
        LDA WA
        LDY WA+1
        STA NodePtr
        STY NodePtr+1
        JSR NodePtrToNextPtrData
        JMP YA_RTS
; -------------------------------------
hREMPROP:
        JSR PopD2StringPtr
        JSR PopD2NodePtr
        JSR NodePtrCheck4NIL
hREMPROP1:
        JSR NodeNextPtrToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hREMPROP2                  ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hREMPROP2:
        JSR NextPtrDataToNextPtr
        JSR CmpStringPtrWithNextPtr
        BEQ hREMPROP3                  ; if Z=1, PC = PC + offset
        JSR NodeNextPtrToNextPtr
        JSR NextNextPtrToNodePtr
        JMP hREMPROP1
; -------------------------------------
hREMPROP3:
        JSR PushStringPtr2DStack
        JSR NodeNextPtrToNextPtr
        JSR NextPtrNextToNextPtr
        JSR NextPtrNextToNextPtr
        JSR NextPtrToNodePtrNext
        JMP YA_RTS
; =====================================
NodePtrCheck4NIL:
        JSR CmpNodePtrWithHashBase     ; NotePtr == NIL?
        BEQ NodePtrCheck4NIL_1         ; yes, ->
        JSR GetNodePtrTypeInfo         ; no:
        CMP #1                         ; LITERAL?
        BNE NodePtrCheck4NIL_1         ; no, ->
        RTS                            ; yes
; -------------------------------------
NodePtrCheck4NIL_1:
        PLA
        PLA
        JMP return_nil
; =====================================
GetNodePtrTypeInfo:
        LDY #4
        LDA (NodePtr),Y
        AND #$F
        RTS                            ; PC <- (Stack)
; +-- TypeInfo
; |
; $HL
; ||
; |+-- 0 list
; | 1 system-functions and literals
; | 2 number
; | 4 literal
; | 8 string
; |
; +--- mask out
; =====================================
GetNextPtrTypeInfo:
        LDY #4
        LDA (NextPtr),Y
        AND #$F
        RTS                            ; PC <- (Stack)
; =====================================
GetStringPtrTypeInfo:
        LDY #4
        LDA (StringPtr),Y
        AND #$F
        RTS                            ; PC <- (Stack)
; =====================================
InvertARG32:
        LDA ARG32
        EOR #$FF
        STA ARG32
        LDA ARG32+1
        EOR #$FF
        STA ARG32+1
        LDA ARG32+2
        EOR #$FF
        STA ARG32+2
        LDA ARG32+3
        EOR #$FF
        STA ARG32+3
        INC ARG32
        BNE InvertARG32_ret            ; if Z=0, PC = PC + offset
        INC ARG32+1
        BNE InvertARG32_ret            ; if Z=0, PC = PC + offset
        INC ARG32+2
        BNE InvertARG32_ret            ; if Z=0, PC = PC + offset
        INC ARG32+3
InvertARG32_ret:
        RTS                            ; PC <- (Stack)
; =====================================
PrintStringOrLiteral:
        JSR GetNextPtrTypeInfo
        CMP #8                         ; (A - M) -> NZC
        BEQ PrintString                ; if Z=1, PC = PC + offset
DoPrintLiteral:
        JMP PrintLiteral
; -------------------------------------
PrintString:
        LDA CRandWordWrapFlag
        BEQ DoPrintLiteral             ; if Z=1, PC = PC + offset
        JSR StrPtrFormat
        LDX StrBuf
        INX
        INX
        TXA
        JSR IfNecessaryWrapLine
        LDA #$22
        JSR NextChrOut
        JSR ShowStrBuf
        LDA #$22
        JMP NextChrOut
; check if string fits into line, wrap otherwise
; A = length of string to be printed
; =====================================
IfNecessaryWrapLine:
        CLC                            ; C <- 0
        ADC LinePos
        CMP MaxLine                    ; (A - M) -> NZC
        BCC MaxLineCheck1              ; if C=0, PC = PC + offset
        JSR PrintCRandIndent
MaxLineCheck1:
        RTS                            ; PC <- (Stack)
; -------------------------------------
hCADR:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BNE hCADR1                     ; if Z=0, PC = PC + offset
        JSR NextPtrNextToNextPtr
hCADR1:
        JMP hCARPROG2
; -------------------------------------
hCDDR:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BNE hCDDR1                     ; if Z=0, PC = PC + offset
        JSR NextPtrNextToNextPtr
hCDDR1:
        JMP hCDRGETPROPLIST1
; =====================================
MoveW4toW6Indirect:
        LDY #0
MoveW4toW6Indirect_1:
        LDA (NextPtr),Y
        STA (NodePtr),Y
        INY
        LDA (NextPtr),Y
        STA (NodePtr),Y
        RTS                            ; PC <- (Stack)
; -------------------------------------
        LDY #2
        BNE MoveW4toW6Indirect_1       ; if Z=0, PC = PC + offset
; =====================================
sub_1A7C:
        LDY #2
loc_1A7E:
        LDA StringPtr
        STA (NodePtr),Y
        INY
        LDA StringPtr+1
        STA (NodePtr),Y
        RTS                            ; PC <- (Stack)
; -------------------------------------
        LDY #0
        BEQ loc_1A7E                   ; if Z=1, PC = PC + offset
OnFlagsEq1:
        JSR GetNextPtrTypeInfo
        BEQ OnFlagsEq1_1               ; if Z=1, PC = PC + offset
        JMP YA_RTS
; -------------------------------------
OnFlagsEq1_1:
        JSR NextDataPtrToNodePtr
        JSR PushNextPtr2AStack
        JSR PushNodePtrToDStack
        JSR CallEval
        JSR PopA2NextPtr
        JSR NextPtrNextToNextPtr
        JMP OnFlagsEq1
;
; HashTableIndex = 2 * ((4*(Buf[1] & 7) + (Buf[LastChar] & 3) + Len(Buf)-1) & 0x1F)
; =====================================
MakeHash:
        LDA READBUF+1
        AND #7
        ASL A
        ASL A
        STA HashTableIndex
        LDX READBUF
        LDA READBUF,X
        AND #3
        CLC                            ; C <- 0
        ADC HashTableIndex
        STA HashTableIndex
        DEX
        TXA
        CLC                            ; C <- 0
        ADC HashTableIndex
        AND #$1F
        ASL A
        STA HashTableIndex
        RTS                            ; PC <- (Stack)
; =====================================
InitNextPtrWithHashTableIndex:
        LDX HashTableIndex
        LDA HashTable,X
        STA NextPtr
        LDA HashTable+1,X
        STA NextPtr+1
        RTS                            ; PC <- (Stack)
; copy $3210-$324F
; to   $0917-$0956
; =====================================
ResetHashTable:
        LDX #$3F
ResetHashTable1:
        LDA HashTableCopy,X
        STA HashTable,X
        DEX
        BPL ResetHashTable1            ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
ChkBreakAndPause:
        JSK kSTOP
        BNE ChkF5Key                   ; if Z=0, PC = PC + offset
DoBreak:
        LDA #<aBreak                   ; "\rBREAK\r"
        LDY #>aBreak                   ; "\rBREAK\r"
        JSR Msg
        JMP Interpreter
; -------------------------------------
ChkF5Key:
        LDA zSFDX                      ; Matrix Coordinate of Current Key Pressed
        CMP #6                         ; F5 pressed
        BEQ ChkF7Key                   ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
ChkF7Key:
        JSK kSTOP
        BEQ DoBreak                    ; if Z=1, PC = PC + offset
        LDA zSFDX
        CMP #3                         ; F7 pressed
        BNE ChkF7Key                   ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
; *** mark *** part of the mark-and-sweep-algorithm
GCX:
        LDA NodePtr
        LDX NodePtr+1
        STA NextPtr                    ; (NextPtr) = (NodePtr)
        STX NextPtr+1
        STA FSLPtr                     ; (FSLPtr) = (NodePtr)
        STX FSLPtr+1
        LDA #<HashBase
        LDY #>HashBase
        STA StringPtr                  ; (StringPtr) = HashBase
        STY StringPtr+1
        JMP GCX1
; =====================================
DoGC:
        SEI                            ; I <- 1
;       LDA #$35                       ; '5'      ; switch KERNAL ROM off
;       STA zR6510
        JSR PushNextPtr2AStack
        JSR PushStringPtr2AStack
IndicatorStar:
.ifdef IndicatorPatch
        LDA IndicatorStarPos
        STA SavedIndicatorChar
        JSR GetStarColor
        STA SavedIndicatorColor
        LDA #1
        JSR SetStarColor
.endif
        LDA #'*'
        STA IndicatorStarPos           ; last col in line 1
        LDA NodeListPtr
        LDX NodeListPtr+1              ; NodePtr = NodeListPtr
        STA NodePtr
        STX NodePtr+1
; *** Mark-and-Sweep-Algorithmus ***
;
; Bei diesem Verfahren der Speicherbereinigung wird von bekanntermaen noch
; benutzten Objekten ausgehend allen Verweisen auf andere Objekte gefolgt.
; Jedes so erreichte Objekt wird markiert. Anschlieend werden alle nicht
; markierten Objekte zur Wiederverwendung freigegeben.
;
; Die Freigabe kann zur Speicherfragmentierung fhren. Das Problem ist
; hierbei jedoch etwas geringer als bei manueller Speicherverwaltung. Whrend
; bei manueller Speicherverwaltung die Deallozierung immer sofort erfolgt,
; werden bei Mark-and-Sweep fast immer mehrere Objekte auf einmal beseitigt,
; wodurch grere zusammenhngende Speicherbereiche frei werden knnen.
;
; clear all node flags
DoGC1:
        LDY #4
        LDA (NodePtr),Y                ; for each NodePtr in NodeList
        AND #$F
        STA (NodePtr),Y                ; clear hi-nibble of flags byte
        JSR NodePtr_MoveToNext         ; NodePtr = NodePtr->NextPtr
        LDX NodePtr+1
        CPX FSLTOP+1                   ; (X - M) -> NZC
        BNE DoGC1                      ; if Z=0, PC = PC + offset
; *** iterate and mark all chain lists ***
        LDX #0
DoGC2:
        STX HashTableIndex
        LDA HashTable,X
        STA NodePtr
        LDA HashTable+1,X
        STA NodePtr+1                  ; NodePtr = HashTable[HashTableIndex]
        JSR GCX                        ; *** mark *** part of the mark-and-sweep-algorithm
        LDX HashTableIndex             ; HashTableIndex++ (i.e. += 2)
        INX
        INX
        CPX #$40                       ; '@'      ; (X - M) -> NZC
        BNE DoGC2                      ; if Z=0, PC = PC + offset
;
; *** iterate and mark WA list ***
        LDA WA
        LDX WA+1
        STA NodePtr
        STX NodePtr+1
        JSR GCX                        ; *** mark *** part of the mark-and-sweep-algorithm
;
; *** iterate and mark HashBasePtr list ***
        LDA HashBasePtr
        LDX HashBasePtr+1
        STA NodePtr
        STX NodePtr+1
        JSR GCX                        ; *** mark *** part of the mark-and-sweep-algorithm

;
; *** iterate DSTACK and mark its elements ***
        LDA DSTACK
        LDX DSTACK+1                   ; init MsgPtr with DSTACK top

        CLC                            ; (AX) += 2
        ADC #2
        BCC DoGC2b
        INX
DoGC2b:
        
DoGC3:
        STA MsgPtr                     ; MsgPtr = current DSTACK ptr
        STX MsgPtr+1
        STA NextPtr                    ; NextPtr = current DSTACK ptr
        STX NextPtr+1
.ifdef Move_Stacks
        JSR NextDataPtrToNodePtrGC     ; NodePtr = NextPtr->Data
.else
        JSR NextDataPtrToNodePtr       ; NodePtr = NextPtr->Data
.endif
        JSR GCX                        ; mark data stack element
        LDA MsgPtr
        LDX MsgPtr+1
        CPX DStackPtrHi1+2             ; MsgPtr == DStackPtr?
        BNE DoGC4                      ; if Z=0, PC = PC + offset
        CMP DStackPtrLo                ; (A - M) -> NZC
        BEQ DoGC6                      ; yes, ->
DoGC4:
        LDA MsgPtr                     ; no, increment MsgPtr:
        CLC                            ; MsgPtr += 2
        ADC #2
        BCC DoGC5                      ; if C=0, PC = PC + offset
        INC MsgPtr+1
DoGC5:
        LDX MsgPtr+1
        JMP DoGC3                      ; do next DSTACK element
; -------------------------------------
; *** iterate and mark ASTACK elements ***
DoGC6:
        JSR MarkAStackElements

;
; *** sweep part ***
; of the mark and sweep algorithm
;
        LDA #<HashBase
        LDY #>HashBase
        STA NextPtr
        STY NextPtr+1                  ; NextPtr = &HashBase
        LDA #0
        STA NodesFree
        STA NodesFree+1                ; NodesFree = 0
        LDA EndOfProgPtr
        LDX EndOfProgPtr+1             ; NodePtr = EndOfProgPtr
DoGC_SweepLoop:
        STA NodePtr
        STX NodePtr+1
        LDY #4
        LDA (NodePtr),Y
        BMI DoGC_IsMarked              ; Marker set? ->
;
        LDA #0                         ; append unreferenced (i.e. unused) node to FSL
        STA (NodePtr),Y                ; clear marker flags and node type
        JSR NextPtrToNodePtrNext       ; NodePtr->Next = NextPtr
        JSR MoveNodePtr2NextPtr        ; NextPtr = NodePtr
;
        INC NodesFree                  ; NodesFree++
        BNE DoGC_IsMarked              ; if Z=0, PC = PC + offset
        INC NodesFree+1
DoGC_IsMarked:
        LDY #4                         ; RemoveMarker
        LDA (NodePtr),Y
        AND #$F
        STA (NodePtr),Y
;
        JSR NodePtr_MoveToNext
        LDA NodePtr
        LDX NodePtr+1
        CPX FSLTOP+1                   ; (X - M) -> NZC
        BNE DoGC_SweepLoop             ; if Z=0, PC = PC + offset
        CMP FSLTOP                     ; (A - M) -> NZC
        BCC DoGC_SweepLoop             ; if C=0, PC = PC + offset
;
        JSR MoveNextPtrToNodePtr
        LDA NextPtr
        LDX NextPtr+1
        STA FSLPtr
        STX FSLPtr+1
        JSR PopA2StringPtr
        JSR PopA2NextPtr

RemoveIndicatorStar:
.ifdef IndicatorPatch
        LDA SavedIndicatorColor
        JSR SetStarColor
        LDA SavedIndicatorChar
        STA IndicatorStarPos
.else
        LDA #' '
        STA IndicatorStarPos
.endif
;       LDA #$36                       ; '6'      ; no BASIC, KERNAL
;       STA zR6510
        CLI                            ; I <- 0
        RTS                            ; PC <- (Stack)
; =====================================
.ifdef IndicatorPatch
SavedIndicatorColor:
        .BYTE 0
SavedIndicatorChar:
        .BYTE 0
.endif
; =====================================
NodePtr_MoveToNext:
        LDA NodePtr
        CLC                            ; C <- 0
        ADC #5
        STA NodePtr
        BCC NodePtr_MoveToNext_1       ; if C=0, PC = PC + offset
        INC NodePtr+1
NodePtr_MoveToNext_1:
        RTS                            ; PC <- (Stack)
; =====================================
MarkAStackElements:
        LDA ASTACK
        LDX ASTACK+1                   ; init MsgPtr with ASTACK top
MAStE_MarkNext:
        STA MsgPtr
        STX MsgPtr+1                   ; MsgPtr = current ASTACK ptr
        STA NextPtr
        STX NextPtr+1                  ; NextPtr = current ASTACK ptr
.ifdef Move_Stacks
        JSR NextDataPtrToNodePtrGC     ; NodePtr = NextPtr->Data
.else
        JSR NextDataPtrToNodePtr       ; NodePtr = NextPtr->Data
.endif
        LDA NodePtr                    ; Compare ASTACK element's data ptr with EndOfProgPtr and FSLTOP
        LDX NodePtr+1                  ; to determine whether sweep must be done on it:
        CPX EndOfProgPtr+1             ; Call GCX only if address of elemnt's data is
; between EndOfProgPtr and FSLTOP:
; EndOfProgPtr <= NodePtr < FSLTOP
        BEQ MAStE_CmpEndOfProgPtrLo    ; IF X = EndOfProgPtrHi THEN GOTO CmpEndOfProgPtrLo
        BCC MAStE_CheckNext            ; IF X < EndOfProgPtrHi THEN GOTO CheckNext
        BCS MAStE_CmpFSLTopHi          ; IF X >= EndOfProgPtrHi THEN GOTO CmpFSLTopHi
MAStE_CmpEndOfProgPtrLo:
        CMP EndOfProgPtr               ; (A - M) -> NZC
        BCC MAStE_CheckNext            ; IF A < EndOfProgPtrLo THEN GOTO CheckNext
MAStE_CmpFSLTopHi:
        CPX FSLTOP+1                   ; (X - M) -> NZC
        BEQ MAStE_CmpFSLTopLo          ; IF X = FSLTopHi THEN GOTO CmpFSLTopLo
        BCS MAStE_CheckNext            ; IF X >= FSLTopHi THEN GOTO CheckNext
        BCC MAStE_MarkNodeList         ; IF X < FSLTopHi THEN GOTO MarkNodeList
MAStE_CmpFSLTopLo:
        CMP FSLTOP                     ; (A - M) -> NZC
        BCS MAStE_CheckNext            ; IF A >= FSLTopLo THEN GOTO CheckNext
MAStE_MarkNodeList:
        JSR GCX                        ; *** mark *** part of the mark-and-sweep-algorithm
MAStE_CheckNext:
        LDA MsgPtr                     ; MsgPtr == AStackPtr?
        LDX MsgPtr+1
        CPX AStackPtrHi1+2             ; (X - M) -> NZC
        BNE MAStE_DecMsgPtr            ; no, decrement MsgPtr and do next element
        CMP AStackPtrLo                ; (A - M) -> NZC
        BNE MAStE_DecMsgPtr            ; no, decrement MsgPtr and do next element
        RTS                            ; yes, -> return
; -------------------------------------
MAStE_DecMsgPtr:
        LDA MsgPtr                     ; no, decrement AStackPtr:
        SEC                            ; AStackPtr -= 2
        SBC #2
        BCS MAStE_DecMsgPtr_1          ; if C=1, PC = PC + offset
        DEC MsgPtr+1
MAStE_DecMsgPtr_1:
        LDX MsgPtr+1
        JMP MAStE_MarkNext             ; do next ASTACK element
; -------------------------------------
adr_1c6f:
        JMP YA_RTS
; =====================================
; IF (Marker Flag 4 or 8 are set) or Type IN (INTERNAL,NUMBER,ATOM) THEN
; RETURN Z = 1
;
CheckNodeType:
        LDY #4
        LDA (NodePtr),Y
        AND #$C0
        BNE CheckNodeType_2            ; skip if Marker Flag 4 or 8 are set
        LDA (NodePtr),Y
        AND #$F
        CMP #$F                        ; Type = INTERNAL
        BEQ CheckNodeType_1            ; yes, ->
        CMP #2                         ; Type = NUMBER
        BEQ CheckNodeType_1            ; yes, ->
        CMP #4                         ; Type = ATOM
CheckNodeType_1:
        RTS                            ; PC <- (Stack)
; -------------------------------------
CheckNodeType_2:
        LDA #0
        RTS                            ; PC <- (Stack)
; =====================================
GetNodePtrMarkerFlag2:
        LDY #4
        LDA (NodePtr),Y
        AND #$20
        RTS                            ; PC <- (Stack)
; =====================================
SetNodePtrMarkerFlag2:
        LDY #4
        LDA (NodePtr),Y
        ORA #$20
        STA (NodePtr),Y
        RTS                            ; PC <- (Stack)
; =====================================
SetNodePtrMarkerFlag8:
        LDY #4
        LDA (NodePtr),Y
        ORA #$80
        STA (NodePtr),Y
        RTS                            ; PC <- (Stack)
; =====================================
CmpNodePtrWithFSLPtr:
        LDA NodePtr+1
        CMP FSLPtr+1                   ; (A - M) -> NZC
        BNE CmpNodePtrWithFSLPtr1      ; if Z=0, PC = PC + offset
        LDA NodePtr
        CMP FSLPtr                     ; (A - M) -> NZC
CmpNodePtrWithFSLPtr1:
        RTS                            ; PC <- (Stack)
; =====================================
GCX2a:
        JSR NodePtrDataToStringPtr     ; StringPtr = NodePtr->Data
        JSR NextPtrToNodePtrData       ; NodePtr->Data = NextPtr
DoSetMarkerFlag8_and_DoNext:
        JSR SetNodePtrMarkerFlag8
GCX2b:
        JSR MoveNodePtr2NextPtr        ; NextPtr = NodePtr
        JMP MoveStringPtrToNodePtr     ; NodePtr = StringPtr
; =====================================
GCX3a:
        JSR MoveNextPtrToStringPtr
        JMP DoSetMarkerFlag8_and_DoNext
; =====================================
GCX4a:
        JSR NodePtrNextToStringPtr
;
        LDY #0                         ; NodePtr->Next = NodePtr->Data
        LDA (NodePtr),Y
        TAX
        INY
        LDA (NodePtr),Y
        LDY #3
        STA (NodePtr),Y
        TXA
        DEY
        STA (NodePtr),Y
;
        JSR NextPtrToNodePtrData       ; NodePtr->Data = NextPtr
        JSR SetNodePtrMarkerFlag2
        JMP GCX2b
; =====================================
GCX4b:
        JSR NodePtrNextToStringPtr
        JSR NextPtrToNodePtrNext
        JMP GCX2b
; -------------------------------------
GCX1:
        JSR CheckNodeType              ; (Marker Flag 4 or 8 are set) or Type IN (INTERNAL,NUMBER,ATOM) ?
        BEQ DoSetMarkerFlag8_and_DoNext; Yes, ->
GCX2:
        JSR GCX2a                      ; No, ->
GCX_Loop1:
        JSR CheckNodeType              ; (Marker Flag 4 or 8 are set) or Type IN (INTERNAL,NUMBER,ATOM) ?
        BNE GCX2                       ; No, ->
        JSR GCX3a                      ; Yes, ->
GCX_Loop2:
        JSR GetNodePtrMarkerFlag2
        BEQ GCX5                       ; if Z=1, PC = PC + offset
        JSR CmpNodePtrWithFSLPtr
        BEQ GCX7                       ; if Z=1, PC = PC + offset
        BNE GCX6                       ; if Z=0, PC = PC + offset
GCX5:
        JSR GCX4a
        JMP GCX_Loop1
; -------------------------------------
GCX6:
        JSR GCX4b
        JMP GCX_Loop2
; -------------------------------------
GCX7:
        JMP GCX4b
; =====================================
Compare:
        JSR PopD2NodePtr
        JSR PopD2NextPtr
; =====================================
Compare1:
        JSR GetNodePtrTypeInfo
        CMP #$F                        ; (A - M) -> NZC
        BEQ DoListCompare              ; if Z=1, PC = PC + offset
        CMP #2                         ; (A - M) -> NZC
        BCS Compare3                   ; if C=1, PC = PC + offset
DoListCompare:
        JMP AreNodeAndNextEqual
; -------------------------------------
Compare3:
        BNE Compare7                   ; if Z=0, PC = PC + offset
        DEY
Compare4:
        LDA (NodePtr),Y
        CMP (NextPtr),Y                ; (A - M) -> NZC
        BNE Compare6                   ; if Z=0, PC = PC + offset
        DEY
        BPL Compare4                   ; if N=0, PC = PC + offset
Compare5:
        LDA #0
        RTS                            ; PC <- (Stack)
; -------------------------------------
Compare6:
        LDA #$FF
        RTS                            ; PC <- (Stack)
; -------------------------------------
Compare7:
        JSR MoveNodePtr2StringPtr
        JSR StrPtrFormat
        LDX #$64
Compare8:
        LDA StrBuf,X
        STA READBUF,X
        DEX
        BPL Compare8                   ; if N=0, PC = PC + offset
        JSR MoveNextPtrToStringPtr
        JSR StrPtrFormat
        JSR CmpStrBufWithREADBUF
        BEQ Compare5                   ; if Z=1, PC = PC + offset
        BNE Compare6                   ; if Z=0, PC = PC + offset
; if 0x30 <= A <= 0x39: C=0
; else:   C=1
IsDigit:
        CMP #$3A                       ; ':'      ; (A - M) -> NZC
        BCS IsDigit1                   ; if C=1, PC = PC + offset
        SEC                            ; C <- 1
        SBC #$30
        SEC                            ; C <- 1
        SBC #$D0
IsDigit1:
        RTS                            ; PC <- (Stack)
; =====================================
CallReadRest:
        JSR ReadNextLine
; =====================================
CallReadRest1:
        LDA Flag_1D
        BEQ CallReadRest2              ; if Z=1, PC = PC + offset
        JMP MakeString
; -------------------------------------
CallReadRest2:
        LDX #1
        LDA READBUF,X
        STX ReadBufIndex
        LDY #0
        STY ACC32NegativeFlag
        CMP #$2B                       ; '+'      ; (A - M) -> NZC
        BEQ CallReadRest5              ; if Z=1, PC = PC + offset
        CMP #$2D                       ; '-'      ; (A - M) -> NZC
        BEQ CallReadRest4              ; if Z=1, PC = PC + offset
        JSR IsDigit
        BCC OnDigit                    ; if C=0, PC = PC + offset
NoDigit:
        JMP DoReadRest3
; -------------------------------------
; read negative integer
CallReadRest4:
        LDY #$FF
        STY ACC32NegativeFlag
; read positive integer
CallReadRest5:
        INC ReadBufIndex
        LDA READBUF
        CMP #1                         ; (A - M) -> NZC
        BEQ NoDigit                    ; if Z=1, PC = PC + offset
OnDigit:
        JSR MoveZero2ACC32
CallReadRest7:
        LDX ReadBufIndex
        LDA READBUF,X
        BEQ CallReadRest9              ; if Z=1, PC = PC + offset
        JSR IsDigit
        BCS NoDigit                    ; if C=1, PC = PC + offset
        PHA                            ; Stack <- (A)
        JSR MoveACC32_to_ARG32
        JSR ACC32_Times_2
        JSR ACC32_Times_2
        JSR ADD32
        JSR ACC32_Times_2
        PLA
        AND #$F
        CLC                            ; C <- 0
        ADC ACC32
        STA ACC32
        BCC CallReadRest8              ; if C=0, PC = PC + offset
        JSR ACC32INC1
CallReadRest8:
        INC ReadBufIndex
        JMP CallReadRest7
; -------------------------------------
CallReadRest9:
        LDA ACC32NegativeFlag
        BEQ CallReadRest10             ; if Z=1, PC = PC + offset
        JSR ComplementACC32
CallReadRest10:
        JMP MoveACC32_to_NodePtr_set_Flag2
; =====================================
ACC32_to_BCD:
        LDX #7
        LDA #0
ACC32_to_BCD_2:
        STA StrBuf,X
        STA ARG32,X
        DEX
        BPL ACC32_to_BCD_2             ; if N=0, PC = PC + offset
        JSR sub_1E0D
        SED                            ; D <- 1
        LDY #' '
ACC32_to_BCD_3:
        JSR sub_1DF3
        JSR ACC32_Times_2
        BCC ACC32_to_BCD_4             ; if C=0, PC = PC + offset
        JSR sub_1E00
ACC32_to_BCD_4:
        DEY
        BNE ACC32_to_BCD_3             ; if Z=0, PC = PC + offset
        CLD                            ; D <- 0
        RTS                            ; PC <- (Stack)
; =====================================
sub_1DF3:
        LDX #5
        CLC                            ; C <- 0
loc_1DF6:
        LDA ARG32,X
        ADC ARG32,X
        STA ARG32,X
        DEX
        BNE loc_1DF6                   ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
sub_1E00:
        LDX #5
        SEC                            ; C <- 1
loc_1E03:
        LDA #0
        ADC ARG32,X
        STA ARG32,X
        DEX
        BNE loc_1E03                   ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
sub_1E0D:
        LDA #0
        STA StrBuf
        BIT ACC32+3                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BMI loc_1E17                   ; if N=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
loc_1E17:
        LDA #$FF
        STA StrBuf
        JMP ComplementACC32
; =====================================
NegACC32:
        LDA ACC32
        EOR #$FF
        STA ACC32
        LDA ACC32+1
        EOR #$FF
        STA ACC32+1
        LDA ACC32+2
        EOR #$FF
        STA ACC32+2
        LDA ACC32+3
        EOR #$FF
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; =====================================
ComplementACC32:
        JSR NegACC32
ACC32INC:
        INC ACC32
        BNE DoneACC32INC               ; if Z=0, PC = PC + offset
ACC32INC1:
        INC ACC32+1
        BNE DoneACC32INC               ; if Z=0, PC = PC + offset
        INC ACC32+2
        BNE DoneACC32INC               ; if Z=0, PC = PC + offset
        INC ACC32+3
DoneACC32INC:
        RTS                            ; PC <- (Stack)
; =====================================
ShowNextPtrNumeric:
        JSR PopNextPtr2ACC32_1
; =====================================
ShowACC32:
        JSR ACC32_to_BSTR
        JMP PrintLiteral1
; =====================================
ACC32_to_BSTR:
        JSR ACC32_to_BCD
        LDY #0
        LDA StrBuf
        BEQ ACC32_to_BSTR_2            ; if Z=1, PC = PC + offset
        LDA #'-'
        JSR Acc2Buf_IncY
ACC32_to_BSTR_2:
        LDX #1
ACC32_to_BSTR_3:
        LDA ARG32,X
        BNE ACC32_to_BSTR_4            ; if Z=0, PC = PC + offset
        INX
        CPX #6                         ; (X - M) -> NZC
        BNE ACC32_to_BSTR_3            ; if Z=0, PC = PC + offset
        LDA #'0'
        JMP Acc2Buf_IncY
; -------------------------------------
ACC32_to_BSTR_4:
        AND #$F0
        BEQ ACC32_to_BSTR_6            ; if Z=1, PC = PC + offset
ACC32_to_BSTR_5:
        LSR A
        LSR A
        LSR A
        LSR A
        ORA #'0'
        JSR Acc2Buf_IncY
ACC32_to_BSTR_6:
        LDA ARG32,X
        AND #$0F
        ORA #'0'
        JSR Acc2Buf_IncY
        INX
        LDA ARG32,X
        CPX #6                         ; (X - M) -> NZC
        BNE ACC32_to_BSTR_5            ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
Acc2Buf_IncY:
        INY
        STA StrBuf,Y
        STY StrBuf
        RTS                            ; PC <- (Stack)
; =====================================
ACC32_Times_2:
        ASL ACC32
        ROL ACC32+1
        ROL ACC32+2
        ROL ACC32+3
        RTS                            ; PC <- (Stack)
; =====================================
ADD32:
        CLC                            ; C <- 0
        LDA ACC32
        ADC ARG32
        STA ACC32
        LDA ACC32+1
        ADC ARG32+1
        STA ACC32+1
        LDA ACC32+2
        ADC ARG32+2
        STA ACC32+2
        LDA ACC32+3
        ADC ARG32+3
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; =====================================
SUB32:
        SEC                            ; C <- 1
        LDA ACC32
        SBC ARG32
        STA ACC32
        LDA ACC32+1
        SBC ARG32+1
        STA ACC32+1
        LDA ACC32+2
        SBC ARG32+2
        STA ACC32+2
        LDA ACC32+3
        SBC ARG32+3
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; =====================================
MoveACC32_to_ARG32:
        LDX #3
MoveACC32_to_ARG32_1:
        LDA ACC32,X
        STA ARG32,X
        DEX
        BPL MoveACC32_to_ARG32_1       ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
MoveACC32_to_REG32:
        LDX #3
MoveACC32_to_REG32_1:
        LDA ACC32,X
        STA REG32,X
        DEX
        BPL MoveACC32_to_REG32_1       ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
MoveREG32_to_ACC32:
        LDX #3
MoveREG32_to_ACC32_1:
        LDA REG32,X
        STA ACC32,X
        DEX
        BPL MoveREG32_to_ACC32_1       ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
MoveZero2ACC32:
        LDA #0
        BEQ MoveA2ACC32                ; if Z=1, PC = PC + offset
        LDA #1
MoveA2ACC32:
        STA ACC32
        LDA #0
        STA ACC32+1
        STA ACC32+2
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; =====================================
MoveACC32_to_NodePtr_set_Flag2:
        JSR AllocateNode
        LDY #4
        LDA #2
        STA (NodePtr),Y
        DEY
MoveACC32_to_NodePtr_set_Flag2_1:
        LDA ACC32,Y
        STA (NodePtr),Y
        DEY
        BPL MoveACC32_to_NodePtr_set_Flag2_1; if N=0, PC = PC + offset
        JMP PushNodePtrToDStack
; -------------------------------------
hDONE:
        JSR MoveACC32_to_NodePtr_set_Flag2
        JMP YA_RTS
; =====================================
PopNextPtr2ACC32:
        JSR PopD2NextPtr
PopNextPtr2ACC32_1:
        JSR MoveNextPtrToNodePtr
        LDY #4
        LDA (NextPtr),Y
        JSR IsNumericArg
        LDY #3
PopNextPtr2ACC32_2:
        LDA (NextPtr),Y
        STA ACC32,Y
        DEY
        BPL PopNextPtr2ACC32_2         ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
IsNumericArg:
        AND #$F
        CMP #2                         ; (A - M) -> NZC
        BNE IsNumericArg1              ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
IsNumericArg1:
        LDA #<aNonNumericArg           ; "NON-NUMERIC ARGUMENT"
        LDY #>aNonNumericArg           ; "NON-NUMERIC ARGUMENT"
        JMP ShowErrorMsg
; -------------------------------------
aNonNumericArg: .BYTE "NON-NUMERIC ARGUMENT",0
; =====================================
PopStringPtr2ARG32:
        JSR PopD2StringPtr
PopStringPtr2ARG32_2:
        JSR MoveStringPtrToNodePtr
        LDY #4
        LDA (StringPtr),Y
        JSR IsNumericArg
        LDY #3
PopStringPtr2ARG32_3:
        LDA (StringPtr),Y
        STA ARG32,Y
        DEY
        BPL PopStringPtr2ARG32_3       ; if N=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
Pop_Str2ARG_and_Next2ACC:
        JSR PopStringPtr2ARG32
        JMP PopNextPtr2ACC32
; =====================================
ACC32DEC:
        SEC                            ; C <- 1
        LDA ACC32
        SBC #1
        STA ACC32
        LDA ACC32+1
        SBC #0
        STA ACC32+1
        LDA ACC32+2
        SBC #0
        STA ACC32+2
        LDA ACC32+3
        SBC #0
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; =====================================
XCmpACCwithARG:
        LDA ACC32+3
        EOR ARG32+3
        BPL XCmpACCwithARG1            ; if N=0, PC = PC + offset
        LDA ACC32+3
        AND #$80
        ORA #1
        RTS                            ; PC <- (Stack)
; -------------------------------------
XCmpACCwithARG1:
        LDA ACC32+3
        CMP ARG32+3                    ; (A - M) -> NZC
        BNE XCmpACCwithARG2            ; if Z=0, PC = PC + offset
        LDA ACC32+2
        CMP ARG32+2                    ; (A - M) -> NZC
        BNE XCmpACCwithARG2            ; if Z=0, PC = PC + offset
        LDA ACC32+1
        CMP ARG32+1                    ; (A - M) -> NZC
        BNE XCmpACCwithARG2            ; if Z=0, PC = PC + offset
        LDA ACC32
        CMP ARG32                      ; (A - M) -> NZC
        BNE XCmpACCwithARG2            ; if Z=0, PC = PC + offset
        LDA #0
        RTS                            ; PC <- (Stack)
; -------------------------------------
XCmpACCwithARG2:
        LDA #1
        BCS XCmpACCwithARG3            ; if C=1, PC = PC + offset
        LDA #$FF
XCmpACCwithARG3:
        RTS                            ; PC <- (Stack)
; -------------------------------------
hTIMES1:
        JSR MoveACC32_to_REG32
        JSR MoveZero2ACC32
        LDY #$20
hTIMES2:
        JSR ACC32_Times_2
        ASL REG32
        ROL REG32+1
        ROL REG32+2
        ROL REG32+3
        BCC hTIMES3                    ; if C=0, PC = PC + offset
        JSR ADD32
hTIMES3:
        DEY
        BNE hTIMES2                    ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
Divide:
        LDA #0
        STA REG32
        STA REG32+1
        STA REG32+2
        STA REG32+3
        LDA ACC32+3
        AND #$80
        STA ACC32NegativeFlag
        BPL Divide1                    ; if N=0, PC = PC + offset
        JSR ComplementACC32
Divide1:
        LDA ARG32+3
        AND #$80
        STA ARG32NegativeFlag
        BPL Divide2                    ; if N=0, PC = PC + offset
        JSR InvertARG32
Divide2:
        LDY #$20
Divide3:
        ASL ACC32
        ROL ACC32+1
        ROL ACC32+2
        ROL ACC32+3
        ROL REG32
        ROL REG32+1
        ROL REG32+2
        ROL REG32+3
        SEC                            ; C <- 1
        LDA REG32
        SBC ARG32
        LDA REG32+1
        SBC ARG32+1
        LDA REG32+2
        SBC ARG32+2
        LDA REG32+3
        SBC ARG32+3
        BCC Divide4                    ; if C=0, PC = PC + offset
        STA REG32+3
        LDA REG32
        SBC ARG32
        STA REG32
        LDA REG32+1
        SBC ARG32+1
        STA REG32+1
        LDA REG32+2
        SBC ARG32+2
        STA REG32+2
        LDA REG32+3
        SBC ARG32+3
        STA REG32+3
        INC ACC32
Divide4:
        DEY
        BNE Divide3                    ; if Z=0, PC = PC + offset
        LDA ACC32NegativeFlag
        EOR ARG32NegativeFlag
        BPL Divide5                    ; if N=0, PC = PC + offset
        JSR ComplementACC32
Divide5:
        RTS                            ; PC <- (Stack)
; -------------------------------------
hTAB:
        JSR PopNextPtr2ACC32
        SEC                            ; C <- 1
        JSK kPLOT
        STY ARG32
        LDA ACC32
        SEC                            ; C <- 1
        SBC ARG32
        BCC hTAB2                      ; if C=0, PC = PC + offset
        TAX
        INX
hTAB1:
        DEX
        BEQ hTAB2                      ; if Z=1, PC = PC + offset
        LDA #$1D
        JSR OutChar
        BNE hTAB1                      ; if Z=0, PC = PC + offset
hTAB2:
        JMP return_true
; =====================================
RND16_To_ACC32:
;       JSK kIOBASE                    ; YX = $DC00 (CIA #1)
        IOFETCH CIA1TBLO               ; A = CIA1[6] (Timer B Lo-Byte)
;       STX ReturnAddress
;       STY ReturnAddress+1            ; (ReturnAddress) = $DC00
;       LDY #4
;       LDA (ReturnAddress),Y          ; A = CIA1[4] (Timer A Lo-Byte)
        TAX                            ; X = A = CIA1[4]
;       LDY #$14
;       LDA (ReturnAddress),Y          ; A = *(CIA1+20) = CIA2[4] (Timer A Lo-Byte)
        IOFETCH CIA1TBLO               ; A = CIA1[6] (Timer B Lo-Byte)
MoveAX2ACC32:
        STX ACC32
        STA ACC32+1
        LDA #0
        STA ACC32+2
        STA ACC32+3                    ; ACC32 = (0,0,random byte1,random byte2)
        RTS                            ; PC <- (Stack)
; -------------------------------------
hADD1:
        JSR PopNextPtr2ACC32
        JSR ACC32INC
        JMP hDONE
; -------------------------------------
hSUB1:
        JSR PopNextPtr2ACC32
        JSR ACC32DEC
        JMP hDONE
; -------------------------------------
hCONSP:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BEQ ret_true                   ; if Z=1, PC = PC + offset
        BNE ret_nil                    ; if Z=0, PC = PC + offset
hMINUSP:
        JSR PopNextPtr2ACC32
        LDA ACC32+3
        BMI ret_true                   ; if N=1, PC = PC + offset
        BPL ret_nil                    ; if N=0, PC = PC + offset
hZEROP:
        JSR PopNextPtr2ACC32
        LDA ACC32
        ORA ACC32+1
        ORA ACC32+2
        ORA ACC32+3
        BEQ ret_true                   ; if Z=1, PC = PC + offset
ret_nil:
        JMP return_nil
; -------------------------------------
ret_true:
        JMP return_true
; -------------------------------------
hLESSP:
        JSR Pop_Str2ARG_and_Next2ACC
        JSR XCmpACCwithARG
        BMI ret_true                   ; if N=1, PC = PC + offset
        BPL ret_nil                    ; if N=0, PC = PC + offset
hGREATERP:
        JSR Pop_Str2ARG_and_Next2ACC
        JSR XCmpACCwithARG
        BEQ ret_nil                    ; if Z=1, PC = PC + offset
        BMI ret_nil                    ; if N=1, PC = PC + offset
        BPL ret_true                   ; if N=0, PC = PC + offset
hNUMBERP:
        LDA #2
        BNE hSTRINGP1                  ; if Z=0, PC = PC + offset
hSTRINGP:
        LDA #8
hSTRINGP1:
        STA ARG32NegativeFlag
        JSR PopD2NextPtr
        LDY #4
        LDA (NextPtr),Y
        AND #$F
        CMP ARG32NegativeFlag          ; (A - M) -> NZC
        BNE ret_nil                    ; if Z=0, PC = PC + offset
        BEQ ret_true                   ; if Z=1, PC = PC + offset
hSTRINGP2:
        LDA #$4C                       ; 'L'      ; JMP
        STA JMPBuf
        STX JMPBuf+1
        STY JMPBuf+2
        JSR PopD2StringPtr
        JSR GetStringPtrTypeInfo
        BEQ hSTRINGP3                  ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hSTRINGP3:
        JSR StringDataPtrToNextPtr
        JSR PopNextPtr2ACC32_1
        JSR StringNextPtrToNextPtr
hSTRINGP4:
        JSR GetNextPtrTypeInfo
        BEQ hSTRINGP5                  ; if Z=1, PC = PC + offset
        JMP hDONE
; -------------------------------------
hSTRINGP5:
        JSR NextDataPtrToStringPtr
        JSR PopStringPtr2ARG32_2
        JSR JMPBuf
        JSR NextPtrNextToNextPtr
        JMP hSTRINGP4
; -------------------------------------
hPLUS:
        LDX #<ADD32
        LDY #>ADD32
        JMP hSTRINGP2
; -------------------------------------
hDIFFERENCE:
        LDX #<SUB32
        LDY #>SUB32
        JMP hSTRINGP2
; -------------------------------------
hMINUS:
        JSR PopNextPtr2ACC32
        JSR ComplementACC32
        JMP hDONE
; -------------------------------------
hLENGTH:
        JSR MoveZero2ACC32
        JSR PopD2NextPtr
hLENGTH1:
        JSR GetNextPtrTypeInfo
        BEQ hLENGTH2                   ; if Z=1, PC = PC + offset
        JMP hDONE
; -------------------------------------
hLENGTH2:
        JSR ACC32INC
        JSR NextPtrNextToNextPtr
        JMP hLENGTH1
; -------------------------------------
hPOKE:
        JSR Pop_Str2ARG_and_Next2ACC
;       LDY #0
;       LDA ARG32
;       STA (ACC32),Y

        LDA MMUCR                      ; set return config
        STA FARVEC

        LDA CURCFG                     ; set bank for store immediate
        STA hPOKE1+0

        LDA ACC32                      ; set store address for simm
        STA hPOKE1+1                   ; lo, hi
        LDA ACC32+1
        STA hPOKE1+2

        LDA ARG32                      ; byte to store
        JSR simm                       ; store immadiate in random bank
hPOKE1:	.byte $00,$00,$00

        JSR PushStringPtr2DStack
        JMP YA_RTS
; -------------------------------------
hPEEK:
        JSR PopNextPtr2ACC32
;       LDY #0
;       LDA (ACC32),Y

        LDA MMUCR                      ; set return config
        STA FARVEC

        LDA CURCFG                     ; set bank for fetch immediate
        STA hPEEK1+0

        LDA ACC32                      ; set store address for fimm
        STA hPEEK1+1                   ; lo, hi
        LDA ACC32+1
        STA hPEEK1+2

        JSR fimm
hPEEK1:	.byte $00,$00,$00

        JSR MoveA2ACC32
        JMP hDONE
; -------------------------------------
hCFGAREA:                              ; returns memory position
        LDX #<CfgArea                  ; of configuration area
        LDA #>CfgArea
        JSR MoveAX2ACC32
        JMP hDONE
; -------------------------------------
.ifdef with_bank
hBANK:
        JSR PopNextPtr2ACC32
        LDA ACC32
        CMP #$10                       ; if BANK >= 16
        BCS hBANK1                     ; -> return nil
        STA CURBNK                     ; else set it
        TAX
        JSK GETCFG                     ; A = GetConfigAtIndex(X)
        STA CURCFG                     ; set mmu config for peek and poke
        JSR MoveA2ACC32
        JMP hDONE
hBANK1:
        JMP return_nil
.endif
; -------------------------------------
hAND:
        JSR HashBaseToNodePtr
        JSR PopD2NextPtr
hAND1:
        JSR GetNextPtrTypeInfo
        BEQ hAND2                      ; if Z=1, PC = PC + offset
        JSR PushNodePtrToDStack
        JMP YA_RTS
; -------------------------------------
hAND2:
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR PushNodePtrToDStack
        JSR PushNextPtr2AStack
        JSR CallEval
        JSR PopA2NextPtr
        JSR PopD2NodePtr
        JSR CmpNodePtrWithHashBase
        BNE hAND1                      ; if Z=0, PC = PC + offset
        JMP return_nil
; -------------------------------------
hOR:
        JSR PopD2NextPtr
hOR1:
        JSR GetNextPtrTypeInfo
        BEQ hOR2                       ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hOR2:
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR PushNodePtrToDStack
        JSR PushNextPtr2AStack
        JSR CallEval
        JSR PopA2NextPtr
        JSR PopD2NodePtr
        JSR CmpNodePtrWithHashBase
        BEQ hOR1                       ; if Z=1, PC = PC + offset
        JSR PushNodePtrToDStack
        JMP YA_RTS
; -------------------------------------
hLOGAND:
        LDX #<hLOGAND1
        LDY #>hLOGAND1
        JMP hSTRINGP2
; -------------------------------------
hLOGAND1:
        LDA ACC32
        AND ARG32
        STA ACC32
        LDA ACC32+1
        AND ARG32+1
        STA ACC32+1
        LDA ACC32+2
        AND ARG32+2
        STA ACC32+2
        LDA ACC32+3
        AND ARG32+3
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; -------------------------------------
hLOGOR:
        LDX #<hLOGOR1
        LDY #>hLOGOR1
        JMP hSTRINGP2
; -------------------------------------
hLOGOR1:
        LDA ACC32
        ORA ARG32
        STA ACC32
        LDA ACC32+1
        ORA ARG32+1
        STA ACC32+1
        LDA ACC32+2
        ORA ARG32+2
        STA ACC32+2
        LDA ACC32+3
        ORA ARG32+3
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; -------------------------------------
hLOGXOR:
        LDX #<hLOGXOR1
        LDY #>hLOGXOR1
        JMP hSTRINGP2
; -------------------------------------
hLOGXOR1:
        LDA ACC32
        EOR ARG32
        STA ACC32
        LDA ACC32+1
        EOR ARG32+1
        STA ACC32+1
        LDA ACC32+2
        EOR ARG32+2
        STA ACC32+2
        LDA ACC32+3
        EOR ARG32+3
        STA ACC32+3
        RTS                            ; PC <- (Stack)
; -------------------------------------
hCOPY:
        JSR PopD2NextPtr
hCOPY1:
        JSR GetNextPtrTypeInfo
        BEQ hCOPY2                     ; if Z=1, PC = PC + offset
        JSR PushNextPtr2DStack
        JMP YA_RTS
; -------------------------------------
hCOPY2:
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2AStack
        JSR MoveNodePtr2NextPtr        ; NextPtr = NodePtr
        LDA #<hCOPY1
        LDY #>hCOPY1
        JSR YA_JSR
        JSR PopA2NextPtr
        LDA #<hCOPY1
        LDY #>hCOPY1
        JSR YA_JSR
        JSR CALLhCONS
        JMP YA_RTS
; -------------------------------------
hAPPEND3:
        JSR PopD2NodePtr
        JSR PopD2StringPtr
        JSR GetStringPtrTypeInfo
        BEQ hAPPEND4                   ; if Z=1, PC = PC + offset
        JSR PushNodePtrToDStack
        JMP YA_RTS
; -------------------------------------
hAPPEND4:
        JSR PushNodePtrToAStack
        JSR AllocateNode
        JSR PushNodePtrToDStack
hAPPEND5:
        JSR StringDataPtrToNextPtr
        JSR NextPtrToNodePtrData       ; NodePtr->Data = NextPtr
        JSR StringNodePtrNextToStringPtr
        JSR GetStringPtrTypeInfo
        BEQ hAPPEND6                   ; if Z=1, PC = PC + offset
        JSR PopA2NextPtr
        JSR NextPtrToNodePtrNext
        JMP YA_RTS
; -------------------------------------
hAPPEND6:
        JSR MoveNodePtr2NextPtr        ; NextPtr = NodePtr
        JSR AllocateNode
        JSR NodePtrToNextPtrNext
        JMP hAPPEND5
; -------------------------------------
hAPPEND:
        JSR PopD2NextPtr
hAPPEND1:
        JSR GetNextPtrTypeInfo
        BEQ hAPPEND2                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hAPPEND2:
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR NextPtrNextToNextPtr
        LDA #<hAPPEND1
        LDY #>hAPPEND1
        JSR YA_JSR
        JMP hAPPEND3
; -------------------------------------
hTIMES:
        LDX #<hTIMES1
        LDY #>hTIMES1
        JMP hSTRINGP2
; -------------------------------------
hQUOTIENT:
        LDX #<Divide
        LDY #>Divide
        JMP hSTRINGP2
; -------------------------------------
hREMAINDER:
        JSR Pop_Str2ARG_and_Next2ACC
hREMAINDER1:
        JSR Divide
        JSR MoveREG32_to_ACC32
        LDA ACC32NegativeFlag
        BPL hREMAINDER2                ; if N=0, PC = PC + offset
        JSR ComplementACC32
hREMAINDER2:
        JMP hDONE
; -------------------------------------
hEQUAL:
        JSR PopD2NextPtr
        JSR PopD2NodePtr
hEQUAL1:
        JSR AreNodeAndNextEqual
        BNE hEQUAL2                    ; if Z=0, PC = PC + offset
        JMP return_true
; -------------------------------------
hEQUAL2:
        JSR GetNodePtrTypeInfo
        BEQ hEQUAL3                    ; if Z=1, PC = PC + offset
        JSR Compare1
        JMP hEQ1
; -------------------------------------
hEQUAL3:
        JSR GetNextPtrTypeInfo
        BEQ hEQUAL4                    ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hEQUAL4:
        JSR NodePtrDataToStringPtr     ; StringPtr = NodePtr->Data
        JSR NodePtrNextToNodePtr
        JSR PushNodePtrToAStack
        JSR MoveStringPtrToNodePtr
        JSR NextDataPtrToStringPtr
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2AStack
        JSR MoveStringPtr2NextPtr
        LDA #<hEQUAL1
        LDY #>hEQUAL1
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2NodePtr
        JSR PopD2StringPtr
        JSR CmpStringPtrWithHashBase
        BNE hEQUAL5                    ; if Z=0, PC = PC + offset
        JMP return_nil
; -------------------------------------
hEQUAL5:
        JMP hEQUAL1
; -------------------------------------
hSASSOC:
        JSR PopD2NextPtr
        JSR PopD2NodePtr
hSASSOC1:
        JSR GetNextPtrTypeInfo
        BEQ hSASSOC2                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hSASSOC2:
        JSR PushNodePtrToAStack
        JSR NextDataPtrToStringPtr
        JSR PushNextPtr2AStack
        JSR StringNodeDataToStringPtr
        JSR PushNodePtrToDStack
        JSR PushStringPtr2DStack
        LDA #<hEQUAL
        LDY #>hEQUAL
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2NodePtr
        JSR PopD2StringPtr
        JSR CmpStringPtrWithHashBase
        BEQ hSASSOC3                   ; if Z=1, PC = PC + offset
        JSR NextPtrDataToNextPtr
        JSR PushNextPtr2DStack
        JMP YA_RTS
; -------------------------------------
hSASSOC3:
        JSR NextPtrNextToNextPtr
        JMP hSASSOC1
; -------------------------------------
hRANDOM:
        JSR PopNextPtr2ACC32
        JSR DupDStack
        JSR PopStringPtr2ARG32
        JSR SUB32
        JSR MoveACC32_to_ARG32
        JSR RND16_To_ACC32
        LDA #<hREMAINDER1
        LDY #>hREMAINDER1
        JSR YA_JSR
        JSR Pop_Str2ARG_and_Next2ACC
        JSR ADD32
        JMP hDONE
; -------------------------------------
hABS:
        JSR PopNextPtr2ACC32
        LDA ACC32+3
        BPL hABS1                      ; if N=0, PC = PC + offset
        JSR ComplementACC32
hABS1:
        JMP hDONE
; -------------------------------------
hPP:
        LDA #0
        STA IndentColumn
        STA LinePos
        LDA #7
        STA CRandWordWrapFlag
        JSR PopD2NextPtr
        JSR CALLhPP1
        JMP hTERPRI
; -------------------------------------
hPP1:
        JSR GetNextPtrTypeInfo
        BEQ hPP2                       ; if Z=1, PC = PC + offset
        JSR ChkBreakAndPause
        LDA #<DoPrintSExpr2
        LDY #>DoPrintSExpr2
        JSR YA_JSR
        LDA IndentColumn
        CLC                            ; C <- 0
        ADC #2
        STA IndentColumn
        JMP YA_RTS
; -------------------------------------
hPP2:
        JSR NextDataPtrToNodePtr
        LDA NodePtr
        LDY NodePtr+1
        CPY #>nQUOTE                   ; '2'      ; (Y - M) -> NZC
        BNE hPP4                       ; if Z=0, PC = PC + offset
        CMP #<nQUOTE                   ; 's'      ; (A - M) -> NZC
        BNE hPP4                       ; if Z=0, PC = PC + offset
        LDA #$20
        JSR OutChar
        LDA #$27
        JSR OutChar
        JSR NextPtrNextToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hPP3                       ; if Z=1, PC = PC + offset
        JMP YA_RTS
; -------------------------------------
hPP3:
        JSR NextPtrDataToNextPtr
        JMP hPP1
; -------------------------------------
hPP4:
        LDA #$28
        JSR OutChar
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
        JSR CALLhPP1
        JSR PopA2NextPtr
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR CmpNextPtrWithHashBase
        BEQ hPP6                       ; if Z=1, PC = PC + offset
        JSR GetNextPtrTypeInfo
        BEQ hPP7                       ; if Z=1, PC = PC + offset
hPP5:
        LDA #$2E
        JSR OutChar
        JSR CALLhPP1
hPP6:
        LDA #$29
        JSR OutChar
        JMP YA_RTS
; -------------------------------------
hPP7:
        JSR GetNodePtrTypeInfo
        BNE hPP8                       ; if Z=0, PC = PC + offset
        JSR PrintCRandIndent
hPP8:
        LDA IndentColumn
        JSR PushDStack
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hPP10                      ; if Z=1, PC = PC + offset
hPP9:
        LDA #$20
        JSR OutChar
hPP10:
        JSR CALLhPP1
        JSR PopDStack
        STA IndentColumn
        JSR PopA2NextPtr
        JSR NextPtrNextToNextPtr
        JSR CmpNextPtrWithHashBase
        BEQ hPP6                       ; if Z=1, PC = PC + offset
        JSR GetNextPtrTypeInfo
        BNE hPP5                       ; if Z=0, PC = PC + offset
        LDA IndentColumn
        JSR PushDStack
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
        JSR GetNextPtrTypeInfo
        BNE hPP9                       ; if Z=0, PC = PC + offset
        JSR PrintCRandIndent
        JMP hPP10
; =====================================
CALLhPP1:
        LDA #<hPP1
        LDY #>hPP1
        JMP YA_JSR
; -------------------------------------
hSPACES:
        JSR PopNextPtr2ACC32
        LDX ACC32
        JSR hSPACES1
        JMP return_true
; =====================================
hSPACES1:
        INX
hSPACES2:
        DEX
        BEQ hSPACES3                   ; if Z=1, PC = PC + offset
        LDA #$20
        JSR OutChar
        JMP hSPACES2
; -------------------------------------
hSPACES3:
        RTS                            ; PC <- (Stack)
; =====================================
PrintCRandIndent:
        JSR PrintCR
        LDX IndentColumn
        JMP hSPACES1
; -------------------------------------
hPDEF:
        LDA #<hGETDEF
        LDY #>hGETDEF
        JSR YA_JSR
        JMP hPP
; -------------------------------------
hMEMBER:
        JSR PopD2NextPtr
        JSR PopD2NodePtr
hMEMBER1:
        JSR GetNextPtrTypeInfo
        BEQ hMEMBER2                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMEMBER2:
        JSR PushNodePtrToAStack
        JSR PushNextPtr2AStack
        JSR PushNodePtrToDStack
        JSR NextPtrDataToNextPtr
        JSR PushNextPtr2DStack
        LDA #<hEQUAL
        LDY #>hEQUAL
        JSR YA_JSR
        JSR PopD2StringPtr
        JSR PopA2NextPtr
        JSR PopA2NodePtr
        JSR CmpStringPtrWithHashBase
        BEQ hMEMBER3                   ; if Z=1, PC = PC + offset
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hMEMBER3:
        JSR NextPtrNextToNextPtr
        JMP hMEMBER1
; -------------------------------------
hGC:
        JSR DoGC
        LDX NodesFree
        LDA NodesFree+1
        JSR MoveAX2ACC32
        JMP hDONE
; -------------------------------------
hERROR:
        JSR CloseAll
        LDA #<hPRINT
        LDY #>hPRINT
        JSR YA_JSR
        JSR InitAStack
        JSR InitDStack
        JMP ShowErrorMsg2
; -------------------------------------
hCAAR:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BNE hCAAR1                     ; if Z=0, PC = PC + offset
        JSR NextPtrDataToNextPtr
hCAAR1:
        JMP hCARPROG2
; -------------------------------------
hCDAR:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BNE hCDAR1                     ; if Z=0, PC = PC + offset
        JSR NextPtrDataToNextPtr
hCDAR1:
        JMP hCDRGETPROPLIST1
; -------------------------------------
hNCONC1:
        JSR PushHashBase2DStack
        JSR CALLhCONS
        JSR PushHashBase2DStack
        JSR CALLhCONS
        JSR CALLhCONS
        JMP hNCONC
; -------------------------------------
hCOMPL:
        JSR PopNextPtr2ACC32
        JSR NegACC32
        JMP hDONE
; -------------------------------------
hASC:
        JSR PopD2StringPtr
        JSR StrPtrFormat
        LDX StrBuf+1
        LDA #0
        JSR MoveAX2ACC32
        JMP hDONE
; -------------------------------------
.ifdef with_dec
hDEC:
        JSR PopD2StringPtr
        JSR StrPtrFormat

        LDA StrBuf                     ; String Lnge
        STA HexLen
        PHA
        
        LDY #0                         ; Zeiger
        STY HexNdx                     ; Stellen-Zhler

        JSR MoveZero2ACC32             ; Ergebnis

        STY ACC32                      ; Low-Byte Ergebnis
        STY ACC32+1                    ; High-Byte Ergebnis

; nchstes Byte aus String

hDEC1:	CPY HexLen                     ; String Lnge
        BEQ hDEC5                      ; Ergebnis nach GK in ACC

        LDA StrBuf+1,Y                 ; Byte aus String holen
        INY

        CMP #' '
        BEQ hDEC1                      ; nchstes Byte aus String

        INC HexNdx                     ; Stellen Zhler erhhen
        LDX HexNdx                     ; und nach X holen

        CPX #5
        BEQ ArgumentError              ; "INVALID ARGUMENT" Error

        CMP #'0'
        BCC ArgumentError              ; "INVALID ARGUMENT" Error

        CMP #$3A                       ; ASCII(9) +1
        BCC hDEC3                      ; Low-Nibble nach high, ins Ergebnis

        CMP #'A'
        BCC ArgumentError              ; "INVALID ARGUMENT" Error

        CMP #'G'                       ; ASCII(f) +1
        BCS ArgumentError              ; "INVALID ARGUMENT" Error

; Korrektur a-f

        SBC #7

; Low-Nibble nach high, ins Ergebnis

hDEC3:  SBC #$2F                       ; ASCII(0) -1

        ASL A
        ASL A
        ASL A
        ASL A

        LDX #4                         ; Bit Zhler Nibble

hDEC4:  ASL A
        ROL ACC32                      ; Low-Byte Ergebnis
        ROL ACC32+1                    ; High-Byte Ergebnis

        DEX                            ; Bit-Zhler
        BNE hDEC4
        BEQ hDEC1                      ; immer

; Ergebnis in ACC32

hDEC5:  JMP hDONE

; -------------------------------------
ArgumentError:
        LDA #<aArgumentError           ; "INVALID ARGUMENT"
        LDY #>aArgumentError           ; "INVALID ARGUMENT"
        JMP ShowErrorMsg

HexLen:	.BYTE 0
HexNdx: .BYTE 0

aArgumentError: .BYTE "INVALID ARGUMENT",0
; -------------------------------------
.endif

.ifdef with_hex
; -------------------------------------
hHEX:
        JSR PopNextPtr2ACC32
        JSR ClearReadBuf

        LDA ACC32+2
        BNE hHEX1

        LDA ACC32+3
        BEQ hHEX2

hHEX1:	JMP ArgumentError
hHEX2:
        LDA #4
        STA READBUF                    ; length of string

        LDY #1                         ; Zeiger in Ausgabestring
        LDA ACC32+1                    ; High-Byte
        JSR HexIt                      ; ac nach HEX in Ausgabestring

        LDA ACC32                      ; Low-Byte
        JSR HexIt                      ; ac nach HEX in Ausgabestring

        LDA #$FF
        STA Flag_1D
        JSR MakeString
        JMP YA_RTS

; ac nach HEX in Ausgabestring
HexIt:	PHA                            ; High-/Low-Nibble

        LSR A                          ; High-Nibble nach Low
        LSR A
        LSR A
        LSR A

        JSR DoHex

        PLA                            ; High-/Low-Nibble

DoHex:  AND #%00001111
        CMP #10
        BCC DoHex1

        ADC #6                         ; Korrektur a-f
DoHex1:	ADC #'0'                       ; HEX nach ASCII
        STA READBUF,Y                  ; Ausgabestring

        INY                            ; Zeiger in Ausgabestring erhhen
        RTS
;	
; -------------------------------------
.endif

.ifdef with_MrEd
; -------------------------------------
        .import endpos
        .import OutputXA
        .import OutputXATest

hInfo:

        LDA #<aInfoMsg
        LDY #>aInfoMsg
        JSR Msg

        JSR fimm
        .byte BANK1,<endpos,>endpos
        TAX
        JSR fimm
        .byte BANK1,<(endpos+1),>(endpos+1)

;	LDX #<$07B0
;       LDA #>$07B0

  	JSR jimm
        .byte BANK1,>OutputXATest,<OutputXATest

        LDA #$0D
        JSK kCHROUT

        JMP return_nil

aInfoMsg: .BYTE "INFO:",$0D,0
.endif

; -------------------------------------
hPRINC:
        LDA #0
        BEQ hPRINL1                    ; if Z=1, PC = PC + offset
hPRINL:
        LDA #7
hPRINL1:
        STA CRandWordWrapFlag
        LDA #0
        STA IndentColumn
        JSR PopD2NextPtr
        LDA #<hPRINL3
        LDY #>hPRINL3
        JSR YA_JSR
        LDA CRandWordWrapFlag
        BEQ hPRINL2                    ; if Z=1, PC = PC + offset
        JMP hTERPRI
; -------------------------------------
hPRINL2:
        JMP return_nil
; -------------------------------------
hPRINL3:
        JSR GetNextPtrTypeInfo
        BEQ hPRINL4                    ; if Z=1, PC = PC + offset
        JMP DoPrintSExpr3
; -------------------------------------
hPRINL4:
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
        LDA #<hPRINL3
        LDY #>hPRINL3
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR NextPtrNextToNextPtr
        JSR CmpNextPtrWithHashBase
        BNE hPRINL5                    ; if Z=0, PC = PC + offset
        JMP YA_RTS
; -------------------------------------
hPRINL5:
        LDA #$20
        JSR OutChar
        JSR GetNextPtrTypeInfo
        BEQ hPRINL4                    ; if Z=1, PC = PC + offset
        JMP DoPrintSExpr3
; -------------------------------------
hUNBIND:
        JSR DoUNBIND
        JMP return_nil
; -------------------------------------
hRESET:
        LDA HashBasePtr+1
;       CMP #9                         ; (A - M) -> NZC
        CMP #>HashBase
; CHECKME and FIXME!
        BEQ hRESET1                    ; if Z=1, PC = PC + offset
        JSR DoUNBIND
        JMP hRESET
; -------------------------------------
hRESET1:
        JMP return_nil
; -------------------------------------
hLAST:
        JSR PopD2NextPtr
hLAST1:
        JSR GetNextPtrTypeInfo
        BEQ hLAST2                     ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hLAST2:
        JSR NextNextPtrToNodePtr
        JSR GetNodePtrTypeInfo
        BEQ hLAST3                     ; if Z=1, PC = PC + offset
        JSR NextPtrDataToNextPtr
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hLAST3:
        JSR NextPtrNextToNextPtr
        JMP hLAST1
; -------------------------------------
hMAPLIST:
        JSR PopD2NextPtr
        JSR PopD2StringPtr
hMAPLIST1:
        JSR GetNextPtrTypeInfo
        BEQ hMAPLIST2                  ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMAPLIST2:
        JSR PushStringPtr2AStack
        JSR PushNextPtr2AStack
        JSR PushStringPtr2DStack
        JSR PushNextPtr2DStack
        JSR PushHashBase2DStack
        JSR CALLhCONS
        LDA #<hAPPLY
        LDY #>hAPPLY
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2StringPtr
        JSR NextPtrNextToNextPtr
        LDA #<hMAPLIST1
        LDY #>hMAPLIST1
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hMAP:
        JSR PopD2NextPtr
        JSR PopD2StringPtr
hMAP1:
        JSR GetNextPtrTypeInfo
        BEQ hMAP2                      ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMAP2:
        JSR PushStringPtr2AStack
        JSR PushNextPtr2AStack
        JSR PushStringPtr2DStack
        JSR PushNextPtr2DStack
        JSR PushHashBase2DStack
        JSR CALLhCONS
        LDA #<hAPPLY
        LDY #>hAPPLY
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2StringPtr
        JSR PopD2NodePtr
        JSR NextPtrNextToNextPtr
        JMP hMAP1
; -------------------------------------
hMAPC:
        JSR PopD2NextPtr
        JSR PopD2StringPtr
hMAPC1:
        JSR GetNextPtrTypeInfo
        BEQ hMAPC2                     ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMAPC2:
        JSR PushStringPtr2AStack
        JSR PushNextPtr2AStack
        JSR NextPtrDataToNextPtr
        JSR PushStringPtr2DStack
        JSR PushNextPtr2DStack
        JSR PushHashBase2DStack
        JSR CALLhCONS
        LDA #<hAPPLY
        LDY #>hAPPLY
        JSR YA_JSR
        JSR PopA2NextPtr
        JSR PopA2StringPtr
        JSR PopD2NodePtr
        JSR NextPtrNextToNextPtr
        JMP hMAPC1
; -------------------------------------
hNTH:
        JSR PopNextPtr2ACC32
        JSR PopD2NextPtr
hNTH1:
        JSR GetNextPtrTypeInfo
        BEQ hNTH3                      ; if Z=1, PC = PC + offset
hNTH2:
        JMP return_nil
; -------------------------------------
hNTH3:
        LDA ACC32
        BEQ hNTH2                      ; if Z=1, PC = PC + offset
        CMP #1                         ; (A - M) -> NZC
        BNE hNTH4                      ; if Z=0, PC = PC + offset
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hNTH4:
        JSR NextPtrNextToNextPtr
        DEC ACC32
        JMP hNTH1
; -------------------------------------
hMSG:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BEQ hMSG1                      ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hMSG1:
        JSR NextDataPtrToNodePtr
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2DStack
        JSR GetNodePtrTypeInfo
        BEQ hMSG3                      ; if Z=1, PC = PC + offset
        CMP #2                         ; (A - M) -> NZC
        BNE hMSG2                      ; if Z=0, PC = PC + offset
        LDY #0
        LDA (NodePtr),Y
        TAX
        JSR hSPACES1
        JMP hMSG
; -------------------------------------
hMSG2:
        LDA #<nT
        LDY #>nT
        CPY NodePtr+1                  ; (Y - M) -> NZC
        BNE hMSG3                      ; if Z=0, PC = PC + offset
        CMP NodePtr                    ; (A - M) -> NZC
        BNE hMSG3                      ; if Z=0, PC = PC + offset
        JSR PrintCR
        JMP hMSG
; -------------------------------------
hMSG3:
        JSR PushNodePtrToDStack
        JSR CallEval
        LDA #<hPRIN1
        LDY #>hPRIN1
        JSR YA_JSR
        JSR PopDStack
        JMP hMSG
; -------------------------------------
hLBYTE:
        JSR PopNextPtr2ACC32
        LDX ACC32
hLBYTE1:
        LDA #0
        JSR MoveAX2ACC32
        JMP hDONE
; -------------------------------------
hHBYTE:
        JSR PopNextPtr2ACC32
        LDX ACC32+1
        JMP hLBYTE1
; -------------------------------------
hMAPCAN:
        LDA #<hMAPCAR
        LDY #>hMAPCAR
        JSR YA_JSR
        JMP hNCONC
; =====================================
LinkData:
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
LinkData2:
        JSR NextPtrNextToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ LinkData3                  ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
LinkData3:
        JSR PushNodePtrToDStack
        JSR NextDataPtrToStringPtr
        JSR PushStringPtr2DStack
        JSR NextPtrNextToNextPtr
        JSR NextDataPtrToStringPtr
        JSR PushStringPtr2DStack
        JSR PushNextPtr2AStack
        JSR PushNodePtrToAStack
        LDA #<hPUTPROP
        LDY #>hPUTPROP
        JSR YA_JSR
        JSR PopDStack
        JSR PopA2NodePtr
        JSR PopA2NextPtr
        JMP LinkData2

.ifdef with_cold
; -------------------------------------
hCOLD:
.ifdef with_AUTOSTART
        LDA #0
        STA InitDone
.endif
        JMP Init
.endif

; -------------------------------------
moncal   = $B000
SYSVEC   = $0A00                       ; used by monitor return

hMONITOR:
        PHA

        LDA #<JSFRTS
        STF BANK15,SYSVEC

        LDA #>JSFRTS
        STF BANK15,(SYSVEC+1)

        JSK moncal

        PLA

        JMP return_nil

; -------------------------------------
.ifdef with_MrEd
        .import StartEd
        .import InitEd

hED:
;	SEI
        JSR jimm
        .byte BANK1,>InitEd,<InitEd    ; bank 1
;	CLI

        JSR SetColors

        LDA #<aLisp
        LDY #>aLisp
        JSR Msg
hED1:
        JSK kSTOP
        BEQ hED1

; 	LDA #BANK0
; 	STA FARVEC
; 	STA MMUCR

        JMP return_nil

aLisp: .BYTE $0d, "LISP", $0d, 0

.endif

; -------------------------------------
SetColors:
        LDA #7                         ; YELLOW
        STA lCOLOR
        LDA #11                        ; DARK_GRAY
        IOSTASH vBorderCol
        IOSTASH vBackgCol0
        RTS

; -------------------------------------
; the CALL function works basically like the SYS command in BASIC
hCALL:
        JSR PopNextPtr2ACC32

        LDA MMUCR                      ; set return config
        STA FARVEC

        LDA CURCFG                     ; set bank for jsr immediate
        STA hCALL1+0

        LDA ACC32+1                    ; set store address for jimm
        STA hCALL1+1                   ; hi, lo
        LDA ACC32
        STA hCALL1+2

        LDA lSPREG                     ; pop desired status value
        PHA                            ; to stack
        LDA lSAREG                     ; load desired accu value for call
        LDX lSXREG                     ; load desired x reg value for call
        LDY lSYREG                     ; load desired y reg value for call
        PLP                            ; set status for call with saved value from stack
; 	JSR JMP2ACC32                  ; do the call
        JSR jimm
hCALL1:	.byte $00,$00,$00
        PHP                            ; save status value after call on stack
        STA lSAREG                     ; save a reg value to storage area for accu
        STX lSXREG                     ; save x reg value to storage area for x reg
        STY lSYREG                     ; save y reg value to storage area for y reg
        PLA                            ; pull status value from stack
        STA lSPREG                     ; to storage area for status reg

        JMP return_nil
; =====================================
;JMP2ACC32:
;       JMP (ACC32)
; -------------------------------------
hREMOVE:
        JSR PopD2NextPtr
        JSR PopD2NodePtr
hREMOVE1:
        JSR GetNextPtrTypeInfo
        BEQ hREMOVE2                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hREMOVE2:
        JSR NextDataPtrToStringPtr
        JSR PushNextPtr2AStack
        JSR PushNodePtrToAStack
        JSR PushNodePtrToDStack
        JSR PushStringPtr2DStack
        LDA #<hEQUAL
        LDY #>hEQUAL
        JSR YA_JSR
        JSR PopD2StringPtr
        JSR PopA2NodePtr
        JSR PopA2NextPtr
        JSR CmpStringPtrWithHashBase
        BEQ hREMOVE3                   ; if Z=1, PC = PC + offset
        JSR NextPtrNextToNextPtr
        JMP hREMOVE1
; -------------------------------------
hREMOVE3:
        JSR NextDataPtrToStringPtr
        JSR PushStringPtr2DStack
        JSR NextPtrNextToNextPtr
        LDA #<hREMOVE1
        LDY #>hREMOVE1
        JSR YA_JSR
        JMP hCONS
; =====================================
.include "lisp128st.s"
; =====================================
MkFileName_from_PopD:
        JSR PopD2StringPtr
        JSR StrPtrFormat
        LDX StrBuf
        STX zFNLEN
        LDA #<(StrBuf+1)
        LDY #>(StrBuf+1)
        STA zFNADR
        STY zFNADR+1
        RTS                            ; PC <- (Stack)
; =====================================
DeviceNumber_From_PopNextPtr:
        JSR PopNextPtr2ACC32
        LDA ACC32
        STA zFA                        ; Device Number
        RTS                            ; PC <- (Stack)
; =====================================
SecondaryAddr_From_PopNextPtr:
        JSR PopNextPtr2ACC32
        LDA ACC32
        STA zSA
        RTS                            ; PC <- (Stack)
; =====================================
LogicalAddress_From_PopNextPtr:
        JSR PopNextPtr2ACC32
        LDA ACC32
        STA zLA
        RTS                            ; PC <- (Stack)
; -------------------------------------
MissingParameterError:
        LDA #<nOPEN
        LDY #>nOPEN
        STA NodePtr
        STY NodePtr+1
        LDA #<aMissingParam            ; "MISSING PARAMETER"
        LDY #>aMissingParam            ; "MISSING PARAMETER"
        JMP ShowErrorMsg
; -------------------------------------
aMissingParam: .BYTE "MISSING PARAMETER",0
; =====================================
ChkOpenParameters:
        LDX #0
        STX zLA
        STX zSA
        STX zFNLEN
        JSR PopD2StringPtr
        JSR GetStringPtrTypeInfo
        BNE MissingParameterError      ; if Z=0, PC = PC + offset
        JSR StringDataPtrToNextPtr
        JSR PushNextPtr2DStack
        JSR LogicalAddress_From_PopNextPtr
        JSR StringNodePtrNextToStringPtr
        JSR GetStringPtrTypeInfo
        BNE MissingParameterError      ; if Z=0, PC = PC + offset
        JSR StringDataPtrToNextPtr
        JSR PushNextPtr2DStack
        JSR DeviceNumber_From_PopNextPtr
        JSR StringNodePtrNextToStringPtr
        JSR GetStringPtrTypeInfo
        BNE ChkOpenParameters2         ; if Z=0, PC = PC + offset
        JSR StringDataPtrToNextPtr
        JSR PushNextPtr2DStack
        JSR SecondaryAddr_From_PopNextPtr
        JSR StringNodePtrNextToStringPtr
        JSR GetStringPtrTypeInfo
        BNE ChkOpenParameters2         ; if Z=0, PC = PC + offset
        JSR StringDataPtrToNextPtr
        JSR PushNextPtr2DStack
        JMP MkFileName_from_PopD
; -------------------------------------
ChkOpenParameters2:
        RTS                            ; PC <- (Stack)
; =====================================
DoSave:
        JSR GetNextPtrTypeInfo
        BEQ DoSave3                    ; if Z=1, PC = PC + offset
        JSR PushHashBase2DStack
; =====================================
DoSave2:
        LDA #<hPRINT
        LDY #>hPRINT
        JSR YA_JSR
        RTS                            ; PC <- (Stack)
; -------------------------------------
DoSave3:
        JSR NextDataPtrToNodePtr
        JSR PushNodePtrToDStack
        JSR NodePtrNextToNodePtr
        JSR PushNodePtrToDStack
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2AStack
        JSR CALLhCONS
        JSR DoSave2
        JSR PopA2NextPtr
        JMP DoSave
; -------------------------------------
DoREAD:
        LDA #<hREAD
        LDY #>hREAD
        JSR YA_JSR
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BEQ ReadMore                   ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
ReadMore:
        JSR LinkData
        LDA #<DoREAD
        LDY #>DoREAD
        JSR YA_JSR
        JSR CALLhCONS
        JMP YA_RTS
; -------------------------------------
MakeStringFromChar:
        PHA                            ; Stack <- (A)
        JSR ClearReadBuf
        LDX #1
        STX READBUF                    ; length of string = 1
        PLA
        STA READBUF+1                  ; string character
        LDA #$FF
        STA Flag_1D
        JSR MakeString
        JMP YA_RTS
; -------------------------------------
IOAbortedError:
        LDA #<HashBase
        LDY #>HashBase
        STA NodePtr
        STY NodePtr+1
        LDA #<aIOAborted               ; "\r+++ I/O ABORTED +++"
        LDY #>aIOAborted               ; "\r+++ I/O ABORTED +++"
        JMP ShowErrorMsg
; -------------------------------------
aIOAborted: .BYTE $D,"+++ I/O ABORTED +++",0
; =====================================
CloseAll:
        LDA #$FF
        JSK kCLOSE
        JSK kCLRCHN
        LDA #$40                       ; '@'      ; Kernal error messages only are on
        STA zNSGFLG
        RTS                            ; PC <- (Stack)
; =====================================
IfFloppyAppendUWorUR2FN:
        CMP #8                         ; (A - M) -> NZC
        BEQ IfFloppyAppendUWorUR2FN2   ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
IfFloppyAppendUWorUR2FN2:
        STX FloppyFNExt2
        LDY zFNLEN
        LDX #0
IfFloppyAppendUWorUR2FN3:
        LDA FloppyFNExt,X
        STA (zFNADR),Y
        INY
        INX
        CPX #5                         ; (X - M) -> NZC
        BNE IfFloppyAppendUWorUR2FN3   ; if Z=0, PC = PC + offset
        DEY
        STY zFNLEN
        RTS                            ; PC <- (Stack)
; -------------------------------------
FloppyFNExt: .BYTE ','
        .BYTE 'U'
        .BYTE ','
FloppyFNExt2: .BYTE 'W'
        .BYTE 0
; -------------------------------------
hOPEN:
        JSR ChkOpenParameters
        JSK kOPEN
        BCC hOPEN1                     ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hOPEN1:
        JMP return_true
; -------------------------------------
hCLOSE:
        JSR LogicalAddress_From_PopNextPtr
        LDA zLA
        JSK kCLOSE
        BCC hCLOSE1                    ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hCLOSE1:
        JMP return_true
; -------------------------------------
hOUTPUT:
        JSR LogicalAddress_From_PopNextPtr
        LDX zLA
        JSK kCHKOUT
        BCC hOUTPUT1                   ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hOUTPUT1:
        JMP return_true
; -------------------------------------
hINPUT:
        JSR LogicalAddress_From_PopNextPtr
        LDX zLA
        JSK kCHKIN
        BCC hINPUT1                    ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hINPUT1:
        JMP return_true
; -------------------------------------
hNORMAL:
        JSR CloseAll
        JMP return_true
; -------------------------------------
hSAVE:
        JSR PopD2NextPtr
        JSR PushNextPtr2AStack
        JSR MkFileName_from_PopD
        JSR DeviceNumber_From_PopNextPtr
        LDX #$FF
        STX zLA
        LDX #$6E
        LDA zFA                        ; Device Number
        CMP #8                         ; (A - M) -> NZC
        BEQ hSAVE1                     ; if Z=1, PC = PC + offset
        LDX #$61
hSAVE1:
        STX zSA
        LDX #'W'
        JSR IfFloppyAppendUWorUR2FN    ; Append ",U,W" to filename
        JSK kOPEN
        BCC hSAVE2                     ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hSAVE2:
        LDX zLA
        JSK kCHKOUT
        BCC hSAVE3                     ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hSAVE3:
        JSR PopA2NextPtr
        JSR DoSave
        JMP hNORMAL
; -------------------------------------
hLOAD:
        JSR MkFileName_from_PopD
        JSR DeviceNumber_From_PopNextPtr
        LDX #$FF
        STX zLA
        LDX #$6E
        LDA zFA                        ; Device Number
        CMP #8                         ; (A - M) -> NZC
        BEQ hLOAD1                     ; if Z=1, PC = PC + offset
        LDX #$60
hLOAD1:
        STX zSA
        LDX #'R'
        JSR IfFloppyAppendUWorUR2FN    ; Append ",U,R" to filename
        JSK kOPEN
        BCC hLOAD2                     ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hLOAD2:
        LDX zLA
        JSK kCHKIN
        BCC hLOAD3                     ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hLOAD3:
        LDA #<DoREAD
        LDY #>DoREAD
        JSR YA_JSR
        JSR CloseAll
        JMP YA_RTS
; -------------------------------------
hREADCH:
        JSK kCHRIN
        JMP MakeStringFromChar
; -------------------------------------
hGETCHAR:
        JSK kGETIN
        JMP MakeStringFromChar
; -------------------------------------
hWAITCHAR:
        JSK kGETIN
        BEQ hWAITCHAR                  ; if Z=1, PC = PC + offset
        JMP MakeStringFromChar
; -------------------------------------
hCHAR:
        JSR PopNextPtr2ACC32
        LDA ACC32
        JMP MakeStringFromChar
; -------------------------------------
hPACK:
        JSR ClearReadBuf
        LDX #$FF
        STX Flag_1D
        JSR PopD2NodePtr
hPACK1:
        JSR GetNodePtrTypeInfo
        BEQ hPACK3                     ; if Z=1, PC = PC + offset
hPACK2:
        JSR CallReadRest1
        JMP YA_RTS
; -------------------------------------
hPACK3:
        JSR NodeDataPtrToNextPtr
        JSR GetNextPtrTypeInfo
        BEQ hPACK2                     ; if Z=1, PC = PC + offset
        CMP #2                         ; (A - M) -> NZC
        BEQ hPACK9                     ; if Z=1, PC = PC + offset
        CMP #8                         ; (A - M) -> NZC
        BEQ hPACK4                     ; if Z=1, PC = PC + offset
        LDA #0
        STA Flag_1D
hPACK4:
        JSR MoveNextPtrToStringPtr
        JSR StrPtrFormat
hPACK5:
        LDY READBUF
        LDX StrBuf
        BEQ hPACK8                     ; if Z=1, PC = PC + offset
        LDX #0
hPACK6:
        INY
        BMI hPACK7                     ; if N=1, PC = PC + offset
        INX
        LDA StrBuf,X
        STA READBUF,Y
        BNE hPACK6                     ; if Z=0, PC = PC + offset
hPACK7:
        DEY
        STY READBUF
hPACK8:
        JSR NodePtrNextToNodePtr
        JMP hPACK1
; -------------------------------------
hPACK9:
        JSR PushNodePtrToAStack
        JSR PopNextPtr2ACC32_1
        JSR ACC32_to_BSTR
        JSR PopA2NodePtr
        JMP hPACK5
; -------------------------------------
hGETDEF:
        JSR PopD2NodePtr
        JSR NodePtrCheck4NIL
        JSR PushNodePtrToAStack
        JSR NodePtrNextToStringPtr
hGETDEF1:
        JSR GetStringPtrTypeInfo
        BEQ hGETDEF2                   ; if Z=1, PC = PC + offset
        JSR PopA2NodePtr
        JMP return_nil
; -------------------------------------
hGETDEF2:
        JSR StringDataPtrToNodePtr
        JSR StringNodePtrNextToStringPtr
        JSR IsNoHandlerNode
        CPX #5                         ; <> nEXPR ?
        BNE hGETDEF3                   ; yes, ->
        LDA #<nDE
        LDY #>nDE
        JMP hGETDEF6
; -------------------------------------
hGETDEF3:
        CPX #4                         ; (X - M) -> NZC
        BNE hGETDEF4                   ; if Z=0, PC = PC + offset
        LDA #<nDM
        LDY #>nDM
        JMP hGETDEF6
; -------------------------------------
hGETDEF4:
        CPX #6                         ; (X - M) -> NZC
        BEQ hGETDEF5                   ; if Z=1, PC = PC + offset
        JSR StringNodePtrNextToStringPtr
        JMP hGETDEF1
; -------------------------------------
hGETDEF5:
        LDA #<nDF
        LDY #>nDF
hGETDEF6:
        JSR PushDStack
        JSR PopA2NodePtr
        JSR PushNodePtrToDStack
        JSR StringDataPtrToNextPtr
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2DStack
        JSR CALLhCONS
        JMP hCONS
; -------------------------------------
hOBLIST:
        LDX #0                         ; HashTableIndex = 0
hOBLIST1:
        STX HashTableIndex
        JSR InitNextPtrWithHashTableIndex
        JSR PushNextPtr2DStack         ; put HashTableIndex on DSTACK
        LDA #<hPRINT
        LDY #>hPRINT
        JSR YA_JSR
        LDX HashTableIndex             ; HashTableIndex += 2
        INX
        INX
        CPX #$40                       ; '@'      ; HashTable Index behind end of HashTable
        BNE hOBLIST1                   ; if Z=0, PC = PC + offset
        JMP return_true
; -------------------------------------
.ifdef C64
hNMI:
        PHA                            ; Stack <- (A)
        TXA
        PHA                            ; Stack <- (A)
        TYA
        PHA                            ; Stack <- (A)
        LDA #$7F                       ; DISABLE ALL NMI'S
        STA CIA2ICR                    
        LDY CIA2ICR                    ; CHECK IF REAL NMI...
        BPL hNMI3                      ; NO...RS232/OTHER
                                     ; if N=0, PC = PC + offset
hNMI2:
        JMP NMI_RS232
; -------------------------------------
hNMI3:
;
; Check for STOP Key down
;
        JSR UD60                       ; TestStopKey
        JSK kSTOP                      ; 
        BNE hNMI2                      ; NO STOP KEY...TEST FOR RS232
                                     ; if Z=0, PC = PC + offset

;	JMP hBRK

.endif

; -------------------------------------
hLINE:
        JSR PopNextPtr2ACC32
        LDA ACC32
        STA MaxLine
        JMP PushNextPtr2DStack_RTS_OnFlgsEq4
; -------------------------------------
hREVERSE:
        LDA #<HashBase
        LDY #>HashBase
        STA NodePtr
        STY NodePtr+1
        JSR PopD2NextPtr
hREVERSE1:
        JSR GetNextPtrTypeInfo
        BEQ hREVERSE2                  ; if Z=1, PC = PC + offset
        JSR PushNodePtrToDStack
        JMP YA_RTS
; -------------------------------------
hREVERSE2:
        JSR MoveNodePtr2StringPtr
        JSR AllocateNode
        JSR MoveW4toW6Indirect
        JSR sub_1A7C
        JSR NextPtrNextToNextPtr
        JMP hREVERSE1
; -------------------------------------
hREADL:
        LDA #<hREAD
        LDY #>hREAD
        JSR YA_JSR
        JSR GetCurrentFromLBuf
        BNE hREADL1                    ; if Z=0, PC = PC + offset
        JSR PushHashBase2DStack
        JMP hCONS
; -------------------------------------
hREADL1:
        LDA #<hREADL
        LDY #>hREADL
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hST:
        LDA #0
        STA zFNLEN
        LDA #$F
        LDX #8
        LDY #$F
        JSK kSETLFS
        JSK kOPEN
        BCC hST1                       ; if C=0, PC = PC + offset
        JMP IOAbortedError
; -------------------------------------
hST1:
        LDX #$F
        JSK kCHKIN
        LDA #<hREADL
        LDY #>hREADL
        JSR YA_JSR
        LDA #$F
        JSK kCLOSE
        JSK kCLRCHN
        JMP YA_RTS
; -------------------------------------
hDIR:
.ifdef C128
        LDA #0
        JSK kSETBNK
.endif
        LDA #2
        LDX #<aDollar0                 ; "$0"
        LDY #>aDollar0                 ; "$0"
        JSK kSETNAM

        LDA #$FF
        LDX #8
        LDY #0
        JSK kSETLFS
        JSK kOPEN
        LDX zLA
        JSK kCHKIN
        JSK kCHRIN
        JSK kCHRIN
hDIR1:
        JSK kCHRIN
        LDA zSTATUS
        BNE hDIR3                      ; if Z=0, PC = PC + offset
        JSK kCHRIN
        LDA zSTATUS
        BNE hDIR3                      ; if Z=0, PC = PC + offset
        JSK kCHRIN
        TAX
        JSK kCHRIN
        JSR MoveAX2ACC32
        JSR ShowACC32
        LDA #' '
hDIR2:
        JSK kCHROUT
        JSK kCHRIN
        BNE hDIR2                      ; if Z=0, PC = PC + offset
        JSR PrintCR
        JMP hDIR1
; -------------------------------------
hDIR3:
        LDA zLA
        JSK kCLOSE
        JSK kCLRCHN
        JMP return_nil
; -------------------------------------
hDISK:
        JSR MkFileName_from_PopD
        LDA #$FF
        LDX #8
        LDY #15
        JSK kSETLFS
        JSK kOPEN
        JMP hDIR3
; -------------------------------------
aDollar0: .BYTE "$0",0
; -------------------------------------
hUNPACK:
        LDX #0
        STX IndentColumn
        DEX
        STX Flag_1D
        JSR PopD2NextPtr
hUNPACK1:
        JSR PushNextPtr2AStack
        JSR GetNextPtrTypeInfo
        BEQ hUNPACK5                   ; if Z=1, PC = PC + offset
        CMP #2                         ; (A - M) -> NZC
        BNE hUNPACK2                   ; if Z=0, PC = PC + offset
        JSR PopNextPtr2ACC32_1
        JSR ACC32_to_BSTR
        JMP hUNPACK4
; -------------------------------------
hUNPACK2:
        CMP #8                         ; (A - M) -> NZC
        BEQ hUNPACK3                   ; if Z=1, PC = PC + offset
        LDA #0
        STA Flag_1D
hUNPACK3:
        JSR MoveNextPtrToStringPtr
        JSR StrPtrFormat
hUNPACK4:
        LDX IndentColumn
        CPX StrBuf                     ; (X - M) -> NZC
        BNE hUNPACK6                   ; if Z=0, PC = PC + offset
hUNPACK5:
        JSR PopA2NextPtr
        JMP return_nil
; -------------------------------------
hUNPACK6:
        INX
        STX IndentColumn
        LDA StrBuf,X
        STA READBUF+1
        LDX #1
        STX READBUF
        DEX
        STX READBUF2
        JSR CallReadRest1
        JSR PopA2NextPtr
        LDA #<hUNPACK1
        LDY #>hUNPACK1
        JSR YA_JSR
        JMP hCONS
; -------------------------------------
hREMOB:
        JSR PopD2NextPtr
        JSR GetNextPtrTypeInfo
        BEQ hREMOB1                    ; if Z=1, PC = PC + offset
        JMP return_nil
; -------------------------------------
hREMOB1:
        JSR NextDataPtrToStringPtr
        JSR NextPtrNextToNextPtr
        JSR PushNextPtr2DStack
        JSR GetStringPtrTypeInfo
        CMP #1                         ; (A - M) -> NZC
        BNE hREMOB2                    ; if Z=0, PC = PC + offset
        JSR DoREMOB
hREMOB2:
        JMP hREMOB
; =====================================
DoREMOB:
        JSR StrPtrFormat
        LDX #$7F
DoREMOB1:
        LDA StrBuf,X
        STA READBUF,X
        DEX
        BPL DoREMOB1                   ; if N=0, PC = PC + offset
        JSR MakeHash
        JSR SearchObject
        BCS DoREMOB2                   ; if C=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
DoREMOB2:
        JSR NextNextPtrToStringPtr
        JSR StringDataPtrToNodePtr
        JSR NodePtrToNextPtrData
        JSR StringNextPtrToNodePtr
        JMP NodePtrToNextPtrNext
; -------------------------------------
HashTableCopy: .WORD Chain00
        .WORD Chain01
        .WORD Chain02
        .WORD Chain03
        .WORD Chain04
        .WORD Chain05
        .WORD Chain06
        .WORD Chain07
        .WORD Chain08
        .WORD Chain09
        .WORD Chain0A
        .WORD Chain0B
        .WORD Chain0C
        .WORD Chain0D
        .WORD Chain0E
        .WORD Chain0F
        .WORD Chain10
        .WORD Chain11
        .WORD Chain12
        .WORD Chain13
        .WORD Chain14
        .WORD Chain15
        .WORD Chain16
        .WORD Chain17
        .WORD Chain18
        .WORD Chain19
        .WORD Chain1A
        .WORD Chain1B
        .WORD Chain1C
        .WORD Chain1D
        .WORD Chain1E
        .WORD Chain1F
;
        .WORD HashBase
        .WORD HashBase
; ----------------------------------------
nCAR:  .WORD hCARPROG1
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCar:  .BYTE "CAR",0
; ----------------------------------------
nCDR:  .WORD hCDRGETPROPLIST
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCdr:  .BYTE "CDR",0
; ----------------------------------------
nCONS:  .WORD hCONS
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCons:  .BYTE "CONS",0
; ----------------------------------------
nQUOTE:  .WORD hLISTQUOTE
        .WORD HashBase
        .BYTE  $F
        .BYTE 3
aQuote:  .BYTE "QUOTE",0
; ----------------------------------------
nCOND:  .WORD hCOND
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aCond:  .BYTE "COND",0
; ----------------------------------------
nATOM:  .WORD hATOM
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aAtom:  .BYTE "ATOM",0
; ----------------------------------------
nEQ:  .WORD hEQ
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aEq:  .BYTE "EQ",0
; ----------------------------------------
nNULL:  .WORD hNULL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aNull:  .BYTE "NULL",0
; ----------------------------------------
nNOT:  .WORD hNOT
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aNot:  .BYTE "NOT",0
; ----------------------------------------
nSETQ:  .WORD hSETQ
        .WORD HashBase
        .BYTE  $F
        .BYTE 3
aSetq:  .BYTE "SETQ",0
; ----------------------------------------
nSET:  .WORD hSET
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aSet:  .BYTE "SET",0
; ----------------------------------------
nEVAL:  .WORD hEVAL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aEval:  .BYTE "EVAL",0
; ----------------------------------------
nAPPLY:  .WORD hAPPLY
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aApply:  .BYTE "APPLY",0
; ----------------------------------------
nMAPCAR: .WORD hMAPCAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMapcar: .BYTE "MAPCAR",0
; ----------------------------------------
nLIST:  .WORD hLISTQUOTE
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aList:  .BYTE "LIST",0
; ----------------------------------------
nEXPR:  .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aExpr:  .BYTE "EXPR",0
; ----------------------------------------
nFEXPR:  .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aFexpr:  .BYTE "FEXPR",0
; ----------------------------------------
nLAMBDA: .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aLambda: .BYTE "LAMBDA",0
; ----------------------------------------
nNLAMBDA: .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aNlambda: .BYTE "NLAMBDA",0
; ----------------------------------------
nVALUE:  .WORD $0000
        .WORD HashBase
        .BYTE $F
        .BYTE 1
aValue:  .BYTE "VALUE",0
; ----------------------------------------
nGETPROP: .WORD hGETPROP
        .WORD HashBase
        .BYTE $F
        .BYTE 1
aGetprop: .BYTE "GETPROP",0
; ----------------------------------------
nPUTPROP: .WORD hPUTPROP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPutprop: .BYTE "PUTPROP",0
; ----------------------------------------
nGETPROPLIST: .WORD hCDRGETPROPLIST
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aGetproplist: .BYTE "GETPROPLIST",0
; ----------------------------------------
nREMPROP: .WORD hREMPROP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRemprop: .BYTE "REMPROP",0
; ----------------------------------------
nPRINT:  .WORD hPRINT
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPrint:  .BYTE "PRINT",0
; ----------------------------------------
nREAD:  .WORD hREAD
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRead:  .BYTE "READ",0
; ----------------------------------------
nNCONC:  .WORD hNCONC
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aNconc:  .BYTE "NCONC",0
; ----------------------------------------
nRPLACA: .WORD hRPLACA
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRplaca: .BYTE "RPLACA",0
; ----------------------------------------
nRPLACD: .WORD hRPLACD
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRplacd: .BYTE "RPLACD",0
; ----------------------------------------
nASSOC:  .WORD hASSOC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aAssoc:  .BYTE "ASSOC",0
; ----------------------------------------
nMAP2CAR: .WORD hMAP2CAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMap2car: .BYTE "MAP2CAR",0
; ----------------------------------------
nDE:  .WORD hDE
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aDe:  .BYTE "DE",0
; ----------------------------------------
nPRIN1:  .WORD hPRIN1
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPrin1:  .BYTE "PRIN1",0
; ----------------------------------------
nTERPRI: .WORD hTERPRI
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aTerpri: .BYTE "TERPRI",0
; ----------------------------------------
nAPPLYSTAR: .WORD hAPPLYSTAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aApply_0: .BYTE "APPLY*",0
; ----------------------------------------
nPROG:  .WORD hPROG
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aProg:  .BYTE "PROG",0
; ----------------------------------------
nRETURN: .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aReturn: .BYTE "RETURN",0
; ----------------------------------------
nGO:  .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aGo:  .BYTE "GO",0
; ----------------------------------------
nDF:  .WORD hDF
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aDf:  .BYTE "DF",0
; ----------------------------------------
nEXIT:  .WORD hEXIT
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aExit:  .BYTE "EXIT",0
; ----------------------------------------
nABS:  .WORD hABS
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aAbs:  .BYTE "ABS",0
; ----------------------------------------
nMINUSP: .WORD hMINUSP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMinusp: .BYTE "MINUSP",0
; ----------------------------------------
nCOMMENT: .WORD hCOMMENT
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aStar:  .BYTE '*',0
; ----------------------------------------
nGETDEF: .WORD hGETDEF
        .WORD HashBase
        .BYTE  $F
        .BYTE 3
aGetdef: .BYTE "GETDEF",0
; ----------------------------------------
nOBLIST: .WORD hOBLIST
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aOblist: .BYTE "OBLIST",0
; ----------------------------------------
nOPEN:  .WORD hOPEN
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aOpen:  .BYTE "OPEN",0
; ----------------------------------------
nCLOSE:  .WORD hCLOSE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aClose:  .BYTE "CLOSE",0
; ----------------------------------------
nOUTPUT: .WORD hOUTPUT
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aOutput: .BYTE "OUTPUT",0
; ----------------------------------------
nINPUT:  .WORD hINPUT
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aInput:  .BYTE "INPUT",0
; ----------------------------------------
nNORMAL: .WORD hNORMAL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aNormal: .BYTE "NORMAL",0
; ----------------------------------------
nREADCH: .WORD hREADCH
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aReadch: .BYTE "READCH",0
; ----------------------------------------
nGETCHAR: .WORD hGETCHAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aGetchar: .BYTE "GETCHAR",0
; ----------------------------------------
nWAITCHAR: .WORD hWAITCHAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aWaitchar: .BYTE "WAITCHAR",0
; ----------------------------------------
nCHAR:  .WORD hCHAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aChar:  .BYTE "CHAR",0
; ----------------------------------------
nPACK:  .WORD hPACK
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPack:  .BYTE "PACK",0
; ----------------------------------------
nLINE:  .WORD hLINE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aLine:  .BYTE "LINE",0
; ----------------------------------------
nPP:  .WORD hPP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPp:  .BYTE "PP",0
; ----------------------------------------
nSPACES: .WORD hSPACES
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aSpaces: .BYTE "SPACES",0
; ----------------------------------------
nPDEF:  .WORD hPDEF
        .WORD HashBase
        .BYTE  $F
        .BYTE 3
aPdef:  .BYTE "PDEF",0
; ----------------------------------------
nREVERSE: .WORD hREVERSE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aReverse: .BYTE "REVERSE",0
; ----------------------------------------
nAND:  .WORD hAND
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aAnd:  .BYTE "AND",0
; ----------------------------------------
nOR:  .WORD hOR
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aOr:  .BYTE "OR",0
; ----------------------------------------
nGC:  .WORD hGC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aGc:  .BYTE "GC",0
; ----------------------------------------
nEQUAL:  .WORD hEQUAL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aEqual:  .BYTE "EQUAL",0
; ----------------------------------------
nPLUS:  .WORD hPLUS
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aPlus:  .BYTE "PLUS",0
; ----------------------------------------
nDIFFERENCE: .WORD hDIFFERENCE
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aDifference: .BYTE "DIFFERENCE",0
; ----------------------------------------
nTIMES:  .WORD hTIMES
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aTimes:  .BYTE "TIMES",0
; ----------------------------------------
nQUOTIENT: .WORD hQUOTIENT
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aQuotient: .BYTE "QUOTIENT",0
; ----------------------------------------
nREMAINDER: .WORD hREMAINDER
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRemainder: .BYTE "REMAINDER",0
; ----------------------------------------
nLOGAND: .WORD hLOGAND
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aLogand: .BYTE "LOGAND",0
; ----------------------------------------

.ifdef with_MrEd
; ----------------------------------------
nED:	.WORD hED
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aEd:	.BYTE "ED",0
.endif
; ----------------------------------------

.ifdef with_dec
; ----------------------------------------
nDEC: .WORD hDEC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aDec: .BYTE "DEC",0
.endif

.ifdef with_hex
nHEX: .WORD hHEX
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aHex: .BYTE "HEX",0
.endif

.ifdef with_MrEd
; ----------------------------------------
nINFO: .WORD hInfo
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aInfo: .BYTE "INFO",0
.endif

; ----------------------------------------
nLOGOR:  .WORD hLOGOR
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aLogor:  .BYTE "LOGOR",0
; ----------------------------------------
nLOGXOR: .WORD hLOGXOR
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aLogxor: .BYTE "LOGXOR",0
; ----------------------------------------
nTAB:  .WORD hTAB
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aTab:  .BYTE "TAB",0
; ----------------------------------------
nZEROP:  .WORD hZEROP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aZerop:  .BYTE "ZEROP",0
; ----------------------------------------
nLESSP:  .WORD hLESSP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aLessp:  .BYTE "LESSP",0
; ----------------------------------------
nGREATERP: .WORD hGREATERP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aGreaterp: .BYTE "GREATERP",0
; ----------------------------------------
nNUMBERP: .WORD hNUMBERP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aNumberp: .BYTE "NUMBERP",0
; ----------------------------------------
nSTRINGP: .WORD hSTRINGP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aStringp: .BYTE "STRINGP",0
; ----------------------------------------
nMINUS:  .WORD hMINUS
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMinus:  .BYTE "MINUS",0
; ----------------------------------------
nRANDOM: .WORD hRANDOM
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRandom: .BYTE "RANDOM",0
; ----------------------------------------
nADD1:  .WORD hADD1
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aAdd1:  .BYTE "ADD1",0
; ----------------------------------------
nSUB1:  .WORD hSUB1
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aSub1:  .BYTE "SUB1",0
; ----------------------------------------
nLENGTH: .WORD hLENGTH
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aLength: .BYTE "LENGTH",0
; ----------------------------------------
nERROR:  .WORD hERROR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aError:  .BYTE "ERROR",0
; ----------------------------------------
nCALL:  .WORD hCALL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCall:  .BYTE "CALL",0
.ifdef with_cold
; ----------------------------------------
nCOLD:  .WORD hCOLD
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCold:  .BYTE "COLD",0
.endif
; ----------------------------------------
nAPPEND: .WORD hAPPEND
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aAppend: .BYTE "APPEND",0
; ----------------------------------------
nPOKE:  .WORD hPOKE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPoke:  .BYTE "POKE",0
; ----------------------------------------
nPEEK:  .WORD hPEEK
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPeek:  .BYTE "PEEK",0
; ----------------------------------------
.ifdef with_bank
nBANK:  .WORD hBANK
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
        .BYTE "BANK",0
.endif
; ----------------------------------------
nDM:  .WORD hDM
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aDm:  .BYTE "DM",0
; ----------------------------------------
nMACRO:  .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aMacro:  .BYTE "MACRO",0
; ----------------------------------------
nSASSOC: .WORD hSASSOC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aSassoc: .BYTE "SASSOC",0
; ----------------------------------------
nCOPY:  .WORD hCOPY
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCopy:  .BYTE "COPY",0
; ----------------------------------------
nLOAD:  .WORD hLOAD
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aLoad:  .BYTE "LOAD",0
; ----------------------------------------
nSAVE:  .WORD hSAVE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aSave:  .BYTE "SAVE",0
; ----------------------------------------
nCADR:  .WORD hCADR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCadr:  .BYTE "CADR",0
; ----------------------------------------
nCDDR:  .WORD hCDDR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCddr:  .BYTE "CDDR",0
; ----------------------------------------
nMEMBER: .WORD hMEMBER
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMember: .BYTE "MEMBER",0
; ----------------------------------------
nLABEL:  .WORD $0000
        .WORD HashBase
        .BYTE  $F
        .BYTE 7
aLabel:  .BYTE "LABEL",0
; ----------------------------------------
nASC:  .WORD hASC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aAsc:  .BYTE "ASC",0
; ----------------------------------------
nREADL:  .WORD hREADL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aReadl:  .BYTE "READL",0
; ----------------------------------------
nUNPACK: .WORD hUNPACK
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aUnpack: .BYTE "UNPACK",0
; ----------------------------------------
nMONITOR: .WORD hMONITOR
        .WORD HashBase
        .BYTE $F
        .BYTE 1
aMonitor: .BYTE "MONITOR",0
; ----------------------------------------
nCFGAREA: .WORD hCFGAREA
        .WORD HashBase
        .BYTE $F
        .BYTE 1
aCfgArea: .BYTE "CFGAREA",0
; ----------------------------------------
nCOMPL:  .WORD hCOMPL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCompl:  .BYTE "COMPL",0
; ----------------------------------------
nCONSP:  .WORD hCONSP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aConsp:  .BYTE "CONSP",0
; ----------------------------------------
nNTH:  .WORD hNTH
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aNth:  .BYTE "NTH",0
; ----------------------------------------
nCAAR:  .WORD hCAAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCaar:  .BYTE "CAAR",0
; ----------------------------------------
nCDAR:  .WORD hCDAR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aCdar:  .BYTE "CDAR",0
; ----------------------------------------
nLBYTE:  .WORD hLBYTE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aLbyte:  .BYTE "LBYTE",0
; ----------------------------------------
nHBYTE:  .WORD hHBYTE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aHbyte:  .BYTE "HBYTE",0
; ----------------------------------------
nNCONC1: .WORD hNCONC1
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aNconc1: .BYTE "NCONC1",0
; ----------------------------------------
nPROG1:  .WORD hCARPROG1
        .WORD HashBase
        .BYTE  $F
        .BYTE 2
aProg1:  .BYTE "PROG1",0
; ----------------------------------------
nPROGN:  .WORD hPROGN
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aProgn:  .BYTE "PROGN",0
; ----------------------------------------
nPRINC:  .WORD hPRINC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPrinc:  .BYTE "PRINC",0
; ----------------------------------------
nPRINL:  .WORD hPRINL
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aPrinl:  .BYTE "PRINL",0
; ----------------------------------------
nRESET:  .WORD hRESET
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aReset:  .BYTE "RESET",0
; ----------------------------------------
nUNBIND: .WORD hUNBIND
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aUnbind: .BYTE "UNBIND",0
; ----------------------------------------
nDEFPROP: .WORD hPUTPROP
        .WORD HashBase
        .BYTE  $F
        .BYTE 3
aDefprop: .BYTE "DEFPROP",0
; ----------------------------------------
nLAST:  .WORD hLAST
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aLast:  .BYTE "LAST",0
; ----------------------------------------
nMAPC:  .WORD hMAPC
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMapc:  .BYTE "MAPC",0
; ----------------------------------------
nMSG:  .WORD hMSG
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aMsg:  .BYTE "MSG",0
; ----------------------------------------
nMAP:  .WORD hMAP
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMap:  .BYTE "MAP",0
; ----------------------------------------
nMAPLIST: .WORD hMAPLIST
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMaplist: .BYTE "MAPLIST",0
; ----------------------------------------
nST:  .WORD hST
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aSt:  .BYTE "ST",0
; ----------------------------------------
nDIR:  .WORD hDIR
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aDir:  .BYTE "DIR",0
; ----------------------------------------
nDISK:  .WORD hDISK
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aDisk:  .BYTE "DISK",0
; ----------------------------------------
nREMOB:  .WORD hREMOB
        .WORD HashBase
        .BYTE  $F
        .BYTE 4
aRemob:  .BYTE "REMOB",0
; ----------------------------------------
nMAPCAN: .WORD hMAPCAN
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aMapcan: .BYTE "MAPCAN",0
; ----------------------------------------
nREMOVE: .WORD hREMOVE
        .WORD HashBase
        .BYTE  $F
        .BYTE 1
aRemove: .BYTE "REMOVE",0
; ----------------------------------------
;
; NODE-List
;
NodeList: .WORD nF
        .WORD NodeList_1
        .BYTE 'A'
NodeList_1: .WORD nVALUE
        .WORD NodeList_2
        .BYTE 0
NodeList_2: .WORD HashBase
        .WORD HashBase
        .BYTE 0
nF:  .WORD aF
; "F"
        .WORD HashBase
        .BYTE 0
aF:  .BYTE 'F',0
        .BYTE 0
        .BYTE 0
        .BYTE 4
nT:  .WORD hT
        .WORD NodeList_3
        .BYTE 'A'
NodeList_3: .WORD nVALUE
        .WORD NodeList_4
        .BYTE 0
NodeList_4: .WORD nT
        .WORD HashBase
        .BYTE 0
hT:  .WORD aT
; "T"
        .WORD HashBase
        .BYTE 0
aT:  .BYTE 'T',0
        .BYTE 0
        .BYTE 0
        .BYTE 4
nNIL:  .WORD aNil
; "NIL"
        .WORD HashBase
        .BYTE 0
aNil:  .BYTE "NIL",0
        .BYTE 4
        .WORD nVALUE
        .WORD off_387A
        .BYTE 0
off_387A: .WORD HashBase
        .WORD HashBase
        .BYTE 0
Chain00: .WORD nGO
        .WORD Chain00_1
        .BYTE 0
Chain00_1: .WORD nGC
        .WORD HashBase
        .BYTE 0
Chain01: .WORD nPP
        .WORD Chain01_1
        .BYTE 0
Chain01_1: .WORD nOPEN
        .WORD Chain01_2
        .BYTE 0
Chain01_2: .WORD nOUTPUT
        .WORD Chain01_3
        .BYTE 0
Chain01_3: .WORD nOBLIST
        .WORD HashBase
        .BYTE 0
.ifdef with_hex
Chain02: .WORD nGETPROP
        .WORD Chain02_1
        .BYTE 0
Chain02_1: .WORD nHEX
        .WORD HashBase
        .BYTE 0
.else
Chain02: .WORD nGETPROP
        .WORD HashBase
        .BYTE 0
.endif
Chain03: .WORD nGREATERP
        .WORD Chain03_1
        .BYTE 0
Chain03_1: .WORD nGETDEF
        .WORD HashBase
        .BYTE 0
Chain04: .WORD nPRINT
        .WORD Chain04_1
        .BYTE 0
Chain04_1: .WORD nPRINL
        .WORD Chain04_2
        .BYTE 0
Chain04_2: .WORD nGETCHAR
        .WORD Chain04_3
        .BYTE 0
Chain04_3: .WORD nPOKE
        .WORD HashBase
        .BYTE 0
Chain05: .WORD nPRIN1
        .WORD Chain05_1
        .BYTE 0
Chain05_1: .WORD nPDEF
        .WORD Chain05_2
        .BYTE 0
Chain05_2: .WORD nPROG1
        .WORD Chain05_3
        .BYTE 0
Chain05_3: .WORD nHBYTE
        .WORD Chain05_4
        .BYTE 0
Chain05_4: .WORD nWAITCHAR
        .WORD HashBase
        .BYTE 0
Chain06: .WORD nPLUS
        .WORD Chain06_1
        .BYTE 0
Chain06_1: .WORD nPUTPROP
        .WORD Chain06_2
        .BYTE 0
Chain06_2: .WORD nGETPROPLIST
        .WORD Chain06_3
        .BYTE 0
Chain06_3: .WORD nPROG
        .WORD Chain06_4
        .BYTE 0
Chain06_4: .WORD nPROGN
        .WORD Chain06_5
        .BYTE 0
Chain06_5: .WORD nPACK
        .WORD Chain06_6
        .BYTE 0
Chain06_6: .WORD nPEEK
        .WORD Chain06_7
        .BYTE 0
Chain06_7: .WORD nAND
        .WORD HashBase
        .BYTE 0
Chain07: .WORD nPRINC
        .WORD HashBase
        .BYTE 0
Chain08: .WORD nATOM
        .WORD Chain08_1
        .BYTE 0
Chain08_1: .WORD nADD1
        .WORD Chain08_2
        .BYTE 0
Chain08_2: .WORD nINPUT
        .WORD HashBase
        .BYTE 0
Chain09: .WORD nAPPEND
        .WORD Chain09_1
        .BYTE 0
Chain09_1: .WORD nAPPLY
        .WORD Chain09_2
        .BYTE 0
Chain09_2: .WORD nQUOTE
        .WORD Chain09_3
        .BYTE 0
Chain09_3: .WORD nASC
        .WORD Chain09_4
        .BYTE 0
Chain09_4: .WORD nABS
        .WORD HashBase
        .BYTE 0

Chain0A: .WORD nCOMMENT
.ifdef with_MrEd
        .WORD Chain0A_1
.else
        .WORD HashBase
.endif
        .BYTE 0

.ifdef with_MrEd
Chain0A_1: .WORD nINFO
        .WORD HashBase
        .BYTE 0
.endif

Chain0B: .WORD nREAD
        .WORD Chain0B_1
        .BYTE 0
Chain0B_1: .WORD nQUOTIENT
        .WORD Chain0B_2
        .BYTE 0
Chain0B_2: .WORD nASSOC
        .WORD Chain0B_3
        .BYTE 0
Chain0B_3: .WORD nAPPLYSTAR
        .WORD HashBase
        .BYTE 0
Chain0C: .WORD nZEROP
        .WORD Chain0C_1
        .BYTE 0
Chain0C_1: .WORD nRESET
        .WORD Chain0C_2
        .BYTE 0
Chain0C_2: .WORD nREADL
        .WORD HashBase
        .BYTE 0
Chain0D: .WORD nRPLACD
        .WORD Chain0D_1
        .BYTE 0
Chain0D_1: .WORD nREADCH
        .WORD Chain0D_2
        .BYTE 0
Chain0D_2: .WORD nST
        .WORD HashBase
        .BYTE 0
Chain0E: .WORD nRPLACA
        .WORD Chain0E_1
        .BYTE 0
Chain0E_1: .WORD nREMPROP
        .WORD Chain0E_2
        .BYTE 0
Chain0E_2: .WORD nREMOB
        .WORD Chain0E_3
        .BYTE 0
Chain0E_3: .WORD nREMOVE
        .WORD Chain0E_4
        .BYTE 0
Chain0E_4: .WORD nSET
        .WORD Chain0E_5
        .BYTE 0
Chain0E_5: .WORD nRANDOM
.ifdef with_bank
        .WORD Chain0E_6
.else
        .WORD HashBase
.endif
        .BYTE 0

.ifdef with_bank
Chain0E_6: .WORD nBANK
        .WORD HashBase
        .BYTE 0
.endif

Chain0F: .WORD nCOND
        .WORD Chain0F_1
        .BYTE 0
Chain0F_1: .WORD nREVERSE
        .WORD Chain0F_2
        .BYTE 0
Chain0F_2: .WORD nRETURN
        .WORD Chain0F_3
        .BYTE 0
.ifdef with_cold
Chain0F_3: .WORD nCALL
        .WORD Chain0F_4
        .BYTE 0
Chain0F_4: .WORD nCOLD
        .WORD HashBase
        .BYTE 0
.else
Chain0F_3: .WORD nCALL
        .WORD HashBase
        .BYTE 0
.endif
Chain10: .WORD nCDR
        .WORD Chain10_1
        .BYTE 0
Chain10_1: .WORD nCAR
        .WORD Chain10_2
        .BYTE 0
Chain10_2: .WORD nT
        .WORD Chain10_3
        .BYTE 0
Chain10_3: .WORD nSETQ
        .WORD Chain10_4
        .BYTE 0
Chain10_4: .WORD nCOPY
        .WORD Chain10_5
        .BYTE 0
Chain10_5: .WORD nSUB1
        .WORD Chain10_6
        .BYTE 0
Chain10_6: .WORD nCONSP
        .WORD Chain10_7
        .BYTE 0
Chain10_7: .WORD nCOMPL
        .WORD Chain10_8
        .BYTE 0
Chain10_8: .WORD nSAVE
        .WORD HashBase
        .BYTE 0
Chain11: .WORD nCLOSE
        .WORD Chain11_1
        .BYTE 0
Chain11_1: .WORD nCADR
        .WORD Chain11_2
        .BYTE 0
Chain11_2: .WORD nCDDR
        .WORD Chain11_3
        .BYTE 0
Chain11_3: .WORD nCDAR
        .WORD Chain11_4
        .BYTE 0
Chain11_4: .WORD nCAAR
        .WORD Chain11_5
        .BYTE 0
Chain11_5: .WORD nCHAR
        .WORD HashBase
        .BYTE 0
Chain12: .WORD nCONS
        .WORD Chain12_1
        .BYTE 0
Chain12_1: .WORD nDE
        .WORD Chain12_2
        .BYTE 0
Chain12_2: .WORD nDM
        .WORD Chain12_3
        .BYTE 0
Chain12_3: .WORD nREMAINDER
        .WORD Chain12_4
        .BYTE 0
Chain12_4: .WORD nSTRINGP
        .WORD HashBase
        .BYTE 0
Chain13: .WORD nDF
        .WORD Chain13_1
        .BYTE 0
Chain13_1: .WORD nLIST
        .WORD Chain13_2
        .BYTE 0
Chain13_2: .WORD nLAST
        .WORD Chain13_3
        .BYTE 0
Chain13_3: .WORD nLOAD
        .WORD Chain13_4
        .BYTE 0

Chain13_4: .WORD nCFGAREA
        .WORD HashBase
        .BYTE 0

Chain14: .WORD nLESSP
        .WORD Chain14_1
        .BYTE 0
Chain14_1: .WORD nTAB
        .WORD Chain14_2
        .BYTE 0
Chain14_2: .WORD nSASSOC
        .WORD Chain14_3
        .BYTE 0
Chain14_3: .WORD nSPACES
        .WORD Chain14_4
        .BYTE 0
Chain14_4: .WORD nDIR
        .WORD Chain14_5
        .BYTE 0
Chain14_5: .WORD nLINE
        .WORD Chain14_6
        .BYTE 0
Chain14_6: .WORD nLABEL
        .WORD HashBase
        .BYTE 0

Chain15: .WORD nLENGTH
        .WORD Chain15_1
        .BYTE 0

Chain15_1: .WORD nLBYTE
.ifdef with_MrEd
        .WORD Chain15_2
.else
        .WORD Chain15_3
.endif
        .BYTE 0

.ifdef with_MrEd
Chain15_2: .WORD nED
        .WORD Chain15_3
        .BYTE 0
.endif
.ifdef with_dec
Chain15_3: .WORD nLOGAND
        .WORD Chain15_4
        .BYTE 0
Chain15_4: .WORD nDEC
        .WORD HashBase
        .BYTE 0
.else
Chain15_3: .WORD nLOGAND
        .WORD HashBase
        .BYTE 0
.endif
Chain16: .WORD nEQ
        .WORD Chain16_1
        .BYTE 0
Chain16_1: .WORD nTERPRI
        .WORD Chain16_2
        .BYTE 0
Chain16_2: .WORD nDISK
        .WORD Chain16_3
        .BYTE 0
Chain16_3: .WORD nLOGOR
        .WORD Chain16_4
        .BYTE 0
Chain16_4: .WORD nDEFPROP
        .WORD Chain16_5
        .BYTE 0
Chain16_5: .WORD nMAP
        .WORD Chain16_6
        .BYTE 0
Chain16_6: .WORD nLAMBDA
        .WORD HashBase
        .BYTE 0
Chain17: .WORD nEVAL
        .WORD Chain17_1
        .BYTE 0
Chain17_1: .WORD nTIMES
        .WORD Chain17_2
        .BYTE 0
Chain17_2: .WORD nLOGXOR
        .WORD Chain17_3
        .BYTE 0
Chain17_3: .WORD nEXIT
        .WORD HashBase
        .BYTE 0
Chain18: .WORD nEQUAL
        .WORD HashBase
        .BYTE 0
Chain19: .WORD nMINUSP
        .WORD Chain19_1
        .BYTE 0
Chain19_1: .WORD nMSG
        .WORD Chain19_2
        .BYTE 0
Chain19_2: .WORD nUNBIND
        .WORD Chain19_3
        .BYTE 0
Chain19_3: .WORD nEXPR
        .WORD HashBase
        .BYTE 0
Chain1A: .WORD HashBase
        .WORD Chain1A_1
        .BYTE 0
Chain1A_1: .WORD NodeList
        .WORD Chain1A_2
        .BYTE 0
Chain1A_2: .WORD nNOT
        .WORD Chain1A_3
        .BYTE 0
Chain1A_3: .WORD nDIFFERENCE
        .WORD Chain1A_4
        .BYTE 0
Chain1A_4: .WORD nNTH
        .WORD Chain1A_5
        .BYTE 0
Chain1A_5: .WORD nMAPC
        .WORD Chain1A_6
        .BYTE 0
Chain1A_6: .WORD nMAPLIST
        .WORD Chain1A_7
        .BYTE 0
Chain1A_7: .WORD nERROR
        .WORD HashBase
        .BYTE 0
Chain1B: .WORD nNULL
        .WORD Chain1B_1
        .BYTE 0
Chain1B_1: .WORD nMAPCAR
        .WORD Chain1B_2
        .BYTE 0
Chain1B_2: .WORD nMAPCAN
        .WORD Chain1B_3
        .BYTE 0
Chain1B_3: .WORD nMEMBER
        .WORD Chain1B_4
        .BYTE 0
Chain1B_4: .WORD nMACRO
        .WORD Chain1B_5
        .BYTE 0
Chain1B_5: .WORD nMINUS
        .WORD HashBase
        .BYTE 0
Chain1C: .WORD nMAP2CAR
        .WORD Chain1C_1
        .BYTE 0
Chain1C_1: .WORD nUNPACK
        .WORD Chain1C_2
        .BYTE 0
Chain1C_2: .WORD nMONITOR
        .WORD HashBase
        .BYTE 0

Chain1D: .WORD nVALUE
        .WORD Chain1D_1
        .BYTE 0
Chain1D_1: .WORD nNORMAL
        .WORD HashBase
        .BYTE 0
Chain1E: .WORD nNUMBERP
        .WORD Chain1E_1
        .BYTE 0
Chain1E_1: .WORD nNCONC1
        .WORD Chain1E_2
        .BYTE 0
Chain1E_2: .WORD nFEXPR
        .WORD HashBase
        .BYTE 0
Chain1F: .WORD nOR
        .WORD Chain1F_1
        .BYTE 0
Chain1F_1: .WORD nNCONC
        .WORD Chain1F_2
        .BYTE 0
Chain1F_2: .WORD nNLAMBDA
        .WORD HashBase
        .BYTE 0
EndOfProg:

; ----------------------------------------------------------------------------

Bootstrap:
        jsr InstallJSF

        lda #$0E
        sta MMUCR
        STA FARVEC

        lda #$05
        sta $D506

.ifdef with_MrEd
        jsr TransferMrEd

        .import endpos,sobpos

        lda #<sobpos
        sta endpos
        lda #>sobpos
        sta endpos+1

.endif
.ifdef with_RaDi
        jsr TransferRaDi
.endif
        lda #BANK0
        sta MMUCR
        STA FARVEC

        lda #$04
        sta $D506

        rts


        .import transfer
        .importzp to,tol,toh,tob
        .importzp t1,t1l,t1h,t1b
        .importzp t2,t2l,t2h,t2b
        .import temps

; transfer JSF to bank 0 and 1
InstallJSF:
        lda #$0E
        sta MMUCR
        STA FARVEC

        lda #0
        sta JSF_DestBank+1
        jsr TransJSF

        lda #1
        sta JSF_DestBank+1
        jsr TransJSF

        .import JSF_START, JSF_END
TransJSF:
        lda #<$0D00                  ; destination
        sta tol
        lda #>$0D00
        sta toh
JSF_DestBank:
        lda #1                       ; dest bank
        sta tob

        lda #<((JSF_END-1)-JSF_START)  ; block size
        sta t1l
        lda #>((JSF_END-1)-JSF_START)
        sta t1h
        lda #0
        sta t1b

        lda #<JSF_START              ; source
        sta t2l
        lda #>JSF_START
        sta t2h
        lda #0                       ; bank 0
        sta t2b

        lda #<(JSF_END-1)
        sta temps
        lda #>(JSF_END-1)
        sta temps+1
        lda #0                       ; bank 0
        sta temps+2

        jsr transfer
        rts

.ifdef with_MrEd
        .import MRED_START, MRED_END
        .import TransEx
TransferMrEd:
        lda #<MrEd_DestAddr ; #<$d000 ; #<$1c01                  ; destination
        sta tol
        lda #>MrEd_DestAddr ; #>$d000 ; #>$1c01
        sta toh
        lda #MrEd_DestBank ; BANK1                   ; dest bank
        sta tob

        lda #<((MRED_END-1)-MRED_START); block size
        sta t1l
        lda #>((MRED_END-1)-MRED_START)
        sta t1h
        lda #0
        sta t1b

        lda #<MRED_START             ; source
        sta t2l
        lda #>MRED_START
        sta t2h
        lda #0 ; BANK0                   ; bank 0
        sta t2b

        lda #<(MRED_END-1)
        sta temps
        lda #>(MRED_END-1)
        sta temps+1
        lda #0 ; BANK0
        sta temps+2

        jsr transfer
;       jsr TransEx
        rts
.endif

.ifdef with_RaDi
        .import RADI_START, RADI_END

TransferRaDi:
        lda #<RaDi_DestAddr                  ; destination
        sta tol
        lda #>RaDi_DestAddr
        sta toh
        lda #RaDi_DestBank ; BANK0                   ; dest bank
        sta tob

        lda #<((RADI_END-1)-RADI_START); block size
        sta t1l
        lda #>((RADI_END-1)-RADI_START)
        sta t1h
        lda #0
        sta t1b

        lda #<RADI_START             ; source
        sta t2l
        lda #>RADI_START
        sta t2h
        lda #0 ; BANK0                   ; bank 0
        sta t2b

        lda #<(RADI_END-1)
        sta temps
        lda #>(RADI_END-1)
        sta temps+1
        lda #0 ; BANK0
        sta temps+2

        jsr transfer
;       jsr TransEx
        rts
.endif
